/*-------------------------------------------------------------------
 * NAME:        autoexec.sas
 * VERSION:     6.1
 * DESCRIPTION: Set up the SAS Financial Crimes Foundation
 *              environment.  Used as an autoexec or include.
 * PRODUCT:     SAS Financial Crimes Foundation
 *-------------------------------------------------------------------*/

/*-------------------------------------------------------------------
 * Copyright (c) 2012-2013 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_run_autoexec;

  %global fcf_autoexec_run;

  /*---------------------------------------
   * Global Error Handling Macro Variables
   *---------------------------------------*/
  %global _fcfmodule_
          _fcfmodulever_
          _fcfmsg_
          job_rc
          trans_rc;

  /*---------------------
   * Global FCF settings
   *---------------------*/
  %global ml_risk tf_risk sysadmin fcf_enable_bulkload enable_formats
          enable_base_currency replication_period replicate_suppressed_alerts
          autoroute2currentinvestigator watch_list_processing_method rptxml
          currency_code teradata_mode fcf_application_name fcfroot cmnroot
          fcfdata fcfland fcf_business_unit_normalized fcf_xml_map_path
		  iem_performance_load;

  /*---------------------------------
   * Global DBMS connection settings
   *---------------------------------*/
  %global dbflavor
          kcSchema         kcDBUser       kcDBConnOpts     kcDBLibOpts
          segKCSchema      segKCDBUser    segKCDBConnOpts  segKCDBLibOpts
          coreSchema       coreDBUser     coreDBConnOpts   coreDBLibOpts
          coreAuthDomain   kcAuthDomain   segkcAuthDomain  ecm_user_group
          db_core_libname  db_kc_libname  seg_kc_libname   ecm_db_conopts
          fcf_max_udf_len                                  ecm_user
          ;

  /*---------------------------------
   * Global ECM reservations.
   *---------------------------------*/
  %global reserved_case_config reserved_incident_config
          reserved_party_config reserved_financial_item_config;

  /*---------------------------------------
   * Make sure this code is run only once.
   *---------------------------------------*/
  %if &fcf_autoexec_run ne 1 %then %do;

    %let metaserver      = "@iomsrv.metadatasrv.host@";
    %let metaport        =  @iomsrv.metadatasrv.port@;
    %let metarepository  = "@oma.repository.foundation.name@";

    options metaserver     = &metaserver
            metaport       = &metaport
            metaprotocol   = bridge
            metarepository = &metarepository;

    /*-----------------------
     * Assign path to macros
     *-----------------------*/
    %local fcf_cmacro_path;
    %local fcf_ucmacro_path;
    %let fcf_cmacro_path=@install.cmacros.dir@;
    %let fcf_ucmacro_path=@custsrc@;

    /*--------------------------
     * Configuration parameters
     *--------------------------*/
    options set=FCFROOT "@fcfroot@";
    %let FCFROOT=%sysfunc(sysget(FCFROOT));
    options set=CMNROOT "@cmnroot@";
    %let CMNROOT=%sysfunc(sysget(CMNROOT));
    options set=FCFDATA "@fcfdata@";
    %let FCFDATA=%sysfunc(sysget(FCFDATA));
    options set=FCFLAND "@fcfland@";
    %let FCFLAND=%sysfunc(sysget(FCFLAND));

    @sysadmin.email@

    %let fcf_max_udf_len = 1024; /* max size of the CHAR UDF for FCF/AML parties in the ECM database */

    /*----------------------------------------------------------
     * Risk Ranking
     * By default, alerts are not create for Risk Factors.
     * TF_RISK and ML_RISK set the score threshold for creating
     * risk factor alerts, the value 1000 means disabled.
     *----------------------------------------------------------*/
    %let tf_risk=@agp.mlrisk@;
    %let ml_risk=@agp.tfrisk@;

    /*---------------------------------------------------------------
     * Risk Ranking
     * Sets the maximum number of days of alert history that
     * will be considered as input in the ranking of future alerts.
     *---------------------------------------------------------------*/
    %let replication_period = @agp.scoring.duration@;

    /*--------------------------------------------------------------------
     * Replication
     * By default, only transactions for active alerts are replicated.
     * N - replicate transactions for active alerts.
     * Y - replicate transactions for active and batch suppressed alerts.
     *--------------------------------------------------------------------*/
    %let replicate_suppressed_alerts=@agp.replicate.suppressed.alerts@;

    /*---------------------
     * Match Code Settings
     *---------------------*/
    options DQLOCALE=(ENUSA);
    options DQSETUPLOC="PATH NOT SET";

    /*----------------------------------------------------------------------
     * Routing
     * By default, automatically route new alerts to current investigator.
     * Y - Default, automatically route new alerts to current investigator.
     * N - Do NOT automatically route new alerts to current investigator.
     *----------------------------------------------------------------------*/
    %let autoroute2currentinvestigator = @agp.autoroute@;

    /*------------------------------------------------------------------
     * Enable Formats
     * N - Formats are not enabled - valid during installation.
     * Y - Enable formats after the install.sas successfully completes.
     *     When set to Y then AGP and Deploy To FCF button in Scenario
     *     Administrator execute fcf_create_formats macro.
     *------------------------------------------------------------------*/
    %let enable_formats=N;

    /*---------------------------------------------------------------------------
     * Enable Base Currency
     * N - Base Currency is not enabled - valid during installation.
     * Y - Base Currency available after the install.sas successfully completes.
     *---------------------------------------------------------------------------*/
    %let enable_base_currency=N;

    /*----------------------------------------------------------------------
     * Set get_scenario_performance_stats to Y so fcf_agp_codegen generates
     * code to gather performance statistics for scenarios.
     *----------------------------------------------------------------------*/
    %let get_scenario_performance_stats = N;

    /*------------------------------------------------------------------
     * Valid entries: db2, odbc, oracle, sqlserver or teradata:
     * all lower-case
     *------------------------------------------------------------------*/
    %let dbflavor = @dbms.engine@;

    @dbms.mode@

    /*------------------------------------------------------
     * Note: Schema, User and Password must match database
     *       and all settings are case sensitive.
     *------------------------------------------------------*/
    /*-------------------------
     * Authentication Domains
     *-------------------------*/
    %let coreAuthDomain=Core;
    %let kcAuthDomain=Knowledge Center;
    %let segkcAuthDomain=Segmented Knowledge Center;

    /*-------------------------
     * Common Knowledge Center
     *-------------------------*/
    %let kcSchema     = @dbms.kc.schema@;
    %let kcDBUser     = @dbms.kc.userid@;
    %let kcDBConnOpts = %str(@dbms.kc.connopts@);
    %let kcDBLibOpts  = %str(@dbms.libopts@);

    /*----------------------------
     * Segmented Knowledge Center
     *----------------------------*/
    %let segKCSchema     = @dbms.segkc.schema@;
    %let segKCDBUser     = @dbms.segkc.userid@;
    %let segKCDBConnOpts = %str(@dbms.segkc.connopts@);
    %let segKCDBLibOpts  = %str(@dbms.libopts@);

    /*-------------
     * Common Core
     *-------------*/
    %let coreSchema     = @dbms.core.schema@;
    %let coreDBUser     = @dbms.core.userid@;
    %let coreDBConnOpts = %str(@dbms.core.connopts@);
    %let coreDBLibOpts  = %str(@dbms.libopts@);

    /*-------------
     * ECM options
     *-------------*/
    %let ecm_db_conopts=@ecm.db.conopts@;

    /*-----------------------------------------------------------------------
     * ECM reservations.
     *
     * These values are set based on the maximum key values of the ECM
     * load_post_install_data.sas program.
     *-----------------------------------------------------------------------*/

    %let reserved_case_config=4;
    %let reserved_incident_config=4;
    %let reserved_party_config=3;
    %let reserved_financial_item_config=7;

    /*-----------------------------------------------------------------------
     * !! WARNING: It is recommended that you do NOT edit below this line !!
     *-----------------------------------------------------------------------*/

    options
        fmtsearch  = (work fmts sashelp.mapfmts library)
        mstored
        mexecsize  = 128K
        minoperator
        ;

    /*----------------------------------
     * Site-specific custom source code
     *----------------------------------*/
     libname custsrc "@custsrc@";

    /*---------
     * Formats
     *---------*/
     libname fmts "@custfmt@";

    /*-----------
     * Job stats
     *-----------*/
     libname job_stat "@jobstat@";

    /*-------------
     * Common Data
     *-------------*/
     libname cmndata "@cmndata@";

    /*--------
     * Master
     *--------*/
     libname mst_aler '!FCFDATA/master/alert';
     libname mst_prep '!FCFDATA/master/prep';
     libname mst_rpt  '!FCFDATA/master/report';
     libname mst_rc   '!FCFDATA/master/rc';
     libname mst_nbrs '!FCFDATA/master/nn';
     libname mst_ca   '!FCFDATA/master/ca';       /*Compliance Analytics*/

    /*-------
     * Stage
     *-------*/
     libname stg_aler '!FCFDATA/stage/alert';
     libname stg_bdg  '!FCFDATA/stage/bridge';
     libname stg_ctrl '!FCFDATA/stage/control';
     libname stg_dim  '!FCFDATA/stage/dim';
     libname stg_fact '!FCFDATA/stage/fact';
     libname stg_wtch '!FCFDATA/stage/watchlist';
     libname stg_err  '!FCFDATA/stage/error';
     libname stg_rc   '!FCFDATA/stage/rc';
     libname stgrcacc '!FCFDATA/stage/rc/acc';
     libname stgrcppf '!FCFDATA/stage/rc/ppf';
     libname stgrcpty '!FCFDATA/stage/rc/pty';
     libname stg_ca   '!FCFDATA/stage/ca';        /*Compliance Analytics*/
     libname stg_xref '!FCFDATA/stage/xref';

     /*----------------------------------
      * IEM-specific settings
      *----------------------------------*/
     %let ecm_user_group=Ent Case Mgmt Users;
     %let ecm_user=ecmadmin;
     %let iem_performance_load=yes;
	 
	 libname iem '!FCFDATA/iem';
	 

    /*------------------------------
     * Stored Compiled Macro Source
     *------------------------------*/
    %macro fcf_assign_source_library;
        %let old_source_libs = %sysfunc(kcompress(%sysfunc(getoption(sasmstore)),%str(%(%))));
        %if (%sysfunc(libref(source))) ne 0 %then %do;  /*returns 0 if already assigned*/
            libname source (custsrc "&fcf_cmacro_path" &old_source_libs) access=readonly;
            %if &syslibrc eq 0 %then %do;
                options sasmstore=source;
            %end;
            %else %do;
              %put ERROR: FCF: Cannot assign SOURCE library.;
            %end;
        %end;
        %else %do;
            %put NOTE: FCF: SOURCE library already assigned.;
             options sasmstore=source;
        %end;
    %mend fcf_assign_source_library;
    %fcf_assign_source_library

    /*-----------------------------------------------
     * Figure out what the former autocall path was.
     *-----------------------------------------------*/
    %let old_autocall_path = %sysfunc(kcompress(%sysfunc(getoption(sasautos)),%str(%(%))));

    /*-------------------------------------------
     * Add FCF macros in front of existing path.
     *-------------------------------------------*/
    options sasautos=( "&fcf_ucmacro_path" &old_autocall_path );

    /* SCENARIO ADMINISTRATOR */
    libname parms  xml '!FCFDATA/scenario/xml/ruleconf.xml';

    /*--------------------------------------------
     * Knowledge Center (KC) Database Connections
     *--------------------------------------------*/
    libname db_kc &dbflavor authdomain="&kcAuthDomain."
                               &kcDBConnOpts
                               &kcDBLibOpts;

    %if (%sysfunc(libref(db_kc))) %then %do;
      %put %sysfunc(sysmsg());
      %abort cancel 4;
    %end;

    %let db_kc_libname = db_kc;

    libname seg_kc &dbflavor authdomain="&segkcAuthDomain."
                                &segKCDBConnOpts
                                &segKCDBLibOpts;

    %if (%sysfunc(libref(seg_kc))) %then %do;
      %put %sysfunc(sysmsg());
      %abort cancel 4;
    %end;

    %let seg_kc_libname = seg_kc;

    /*---------------------------
     * Core Database Connections
     *---------------------------*/
    libname db_core &dbflavor authdomain="&coreAuthDomain."
                                   &coreDBConnOpts
                                   &coreDBLibOpts;

    %if (%sysfunc(libref(db_core))) %then %do;
      %put %sysfunc(sysmsg());
      %abort cancel 4;
    %end;

    %let db_core_libname = db_core;

    /*-----------------------------------------------------------------
     * Alert Reminders
     * Used in fcf_notify_manager and fcf_send_alert_reminder,
     * for example: %include custsrc(send_alert_reminder_messages.sas)
     *-----------------------------------------------------------------*/
    filename custsrc "@custsrc@";

    /*----------------------
     * Enable Base Currency
     *----------------------*/
    %macro fcf_enable_base_currency;
        %if &enable_base_currency eq Y %then %do;
            proc sql noprint;
                select lov_type_code into :currency_code
                from seg_kc.fsk_lov
                where upcase(lov_type_name) = 'BASE CURRENCY';
            quit;
        %end;
        %else %do;
            %put NOTE: FCF: Base currency is not enabled.;
        %end;
    %mend fcf_enable_base_currency;
    %fcf_enable_base_currency;

    /*------------------------------
     * Watch List Processing Method
     * INDEX_ARRAY - 32-bit OS.
     * HASH_ARRAY - 64 bit OS.
     *------------------------------*/
    %macro fcf_wlproc;
        %if  &sysmaxlong lt 2147483648 %then %do;
            %let watch_list_processing_method = INDEX_ARRAY;
        %end;
        %else %do;
            %let watch_list_processing_method = HASH_ARRAY;
        %end;
        %put ;
        %put NOTE: WATCH_LIST_PROCESSING_METHOD = &watch_list_processing_method;
    %mend fcf_wlproc;
    %fcf_wlproc;

    /*-----------------------------
     * Display release information
     *-----------------------------*/
    %let _fcfmodule_=%str(SAS Financial Crimes Foundation);
    %let _fcfmsg_=;
    %put ;
    %fcf_put_error;
    %let _fcfmodule_=;

    /*--------------------------------
     * Reference the Application Name
     *--------------------------------*/
    %let fcf_application_name=%str(@fcf.software.component.name@);

    /*-----------------------------------------------------
     * Location of XML maps for SNA Web Service processing
     *-----------------------------------------------------*/
    %let fcf_xml_map_path=%str(@install.xml.map@);

    /*-----------------------------------------------------
     * ECM Source System Code
     *-----------------------------------------------------*/
    %global fcf_source_system_cd;
    %let fcf_source_system_cd=%str(@source_system_cd@);

    /*-----------------------------------------------------
     * Set to make sure the FCF autoexec is run only once.
     *-----------------------------------------------------*/
    %let fcf_autoexec_run = 1;

  %end;

%mend fcf_run_autoexec;

%fcf_run_autoexec;

