/*----------------------------------------------------------------------------
 *    NAME: fcf_agp.sas
 *   TITLE: Alert Generation Process (AGP)
 * PRODUCT: SAS Financial Crimes Foundation
 *  SYSTEM: UNIX/Windows
 *----------------------------------------------------------------------------*/

/*-------------------------------------------------------------------
 * Copyright (c) 2012-2013 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

/*---------------------------------------------------------
 *  Host and Site specific customizations may be REQUIRED
 *---------------------------------------------------------*/

%macro fcf_agp;

 /*----------------
  * Enable Formats
  *----------------*/
  %macro fcf_enable_formats;
        %if &enable_formats eq Y %then %do;
            %fcf_create_formats;
        %end;
        %else %do;
            %put NOTE: FCF: Formats are not enabled.;
        %end;
    %mend fcf_enable_formats;
  %fcf_enable_formats;


  /*------------------------------------------------------------------
   * Store start datetime of the FCF AGP process in a data set.
   *------------------------------------------------------------------*/
  %fcf_start_datetime_init;
  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
    %goto MACRO_END;
  %end;

  /*------------------------------------------------------------------
   * Make sure the FCF start datetime has been set before proceeding.
   *------------------------------------------------------------------*/
  %fcf_start_datetime_read;
  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
    %goto MACRO_END;
  %end;

  /*------------------------------------------------------------------
   * FCF IEM pre-AGP push from AML into ECM
   *
   * Notes:
   * Input datasets are byproducts of the AML ETL process, and must be 
   * located in the IEM work library.
   *------------------------------------------------------------------*/

  %fcf_iem_load_account(ds=iem.ACCOUNT_NEW);

  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
    %goto MACRO_END;
  %end;

  %fcf_iem_update_account;

  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
    %goto MACRO_END;
  %end;

  %fcf_iem_load_associate(ds=iem.associate_NEW);

  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
    %goto MACRO_END;
  %end;

  %fcf_iem_update_associate;

  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
    %goto MACRO_END;
  %end;

  %fcf_iem_load_party(ds=iem.PARTY_NEW);

  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
    %goto MACRO_END;
  %end;

  %fcf_iem_update_party;

  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
    %goto MACRO_END;
  %end;

  %fcf_iem_load_household(ds=iem.HOUSEHOLD_NEW);

  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
    %goto MACRO_END;
  %end;

  %fcf_iem_update_household;

  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
    %goto MACRO_END;
  %end;

  %fcf_iem_load_ext_party(ds=iem.EXT_PARTY_ACCOUNT_NEW);

  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
    %goto MACRO_END;
  %end;

  %fcf_iem_close_dimension_rels(ds_HhdPty=iem.HOUSEHOLD_PARTY_OLD, ds_HhdAcc=iem.HOUSEHOLD_ACCOUNT_OLD, ds_PtyAcc=iem.PARTY_ACCOUNT_OLD);

  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
    %goto MACRO_END;
  %end;
  
  %fcf_iem_load_dimension_rels(ds_HhdPty=iem.HOUSEHOLD_PARTY_NEW, ds_HhdAcc=iem.HOUSEHOLD_ACCOUNT_NEW, ds_PtyAcc=iem.PARTY_ACCOUNT_NEW);

  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
    %goto MACRO_END;
  %end;

  /*------------------------------------------------------------------
   * FCF IEM pre-AGP push from ECM into AML
   *------------------------------------------------------------------*/

  %fcf_iem_fb_man_alerts_risks;

  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
    %goto MACRO_END;
  %end;
     
  %fcf_iem_fb_alerts_risks;

  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
    %goto MACRO_END;
  %end;

  %fcf_iem_fb_party;

  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
    %goto MACRO_END;
  %end;

  /*----------------------------------------------------------------------
   * Notes:
   *  When parameters are updated for a scenario, the header code must be
   *  re-generated and the Alert Generation Process (AGP) re-executed.
   *  If re-executing for the same RUNDATE in the same sessions, then
   *  be certain to clear the STG_ALER directory before executing.
   *  Otherwise the STG_ALER data sets will contain duplicate alerts.
   *  Set get_scenario_performance_stats to Y in the autoexec so
   *  fcf_agp_codegen generates code to gather performance statistics
   *  for scenarios.
   *----------------------------------------------------------------------*/
  %fcf_agp_codegen(outLib=STG_ALER,
                   outDS=generated_headers);
   
  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
     %goto MACRO_END;
  %end;

  %fcf_run_generated_headers(inLib=STG_ALER,
                             inDS=generated_headers,
                             exclude_header=empty_header);
   
  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
     %goto MACRO_END;
  %end;

  /*------------------------------------------------
   * An example of how to include other headers not
   * processed by today's run of fcf_agp_codegen
   * and fcf_run_generated_headers:
   *------------------------------------------------*/
  /*
  %fcf_section_header(Section=ANOTHER_HEADER);
     %include '!FCFROOT/scenario/codegen/another_header.sas';
     %fcf_rcset(&syserr)
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmodule_=ANOTHER_HEADER;
        %let _fcfmsg_=ERROR: FCF: another_header.sas failed check log.;
        %fcf_put_error;
        %goto MACRO_END;
     %end;
  %fcf_section_footer(Section=ANOTHER_HEADER);
  */


  /*--------------------------
   * Process generated alerts
   *--------------------------*/


  /*----------------
   * Combine alerts
   *----------------*/
  %fcf_section_header(Section=AGP_COMBINE_ALERTS);
     %fcf_combine_alerts(inlib=STG_ALER,
                         outLib=STG_ALER,
                         outDS=combined_alerts,
                         outRiskLib=STG_ALER,
                         outRiskDS=risk_factor_alerts);
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
     %end;
  %fcf_section_footer(Section=AGP_COMBINE_ALERTS);

  /*------------------------------------------------------------
   * Test for zero alerts generated.  If no alerts end process.
   *------------------------------------------------------------*/
  %if %sysfunc(exist(STG_ALER.combined_alerts)) %then %do;
     %let dsid = %sysfunc(open(STG_ALER.combined_alerts));
     %let nobs = %sysfunc(attrn(&dsid, nobs));
     %let rc = %sysfunc(close(&dsid));
     %if &nobs eq 0 %then %do;
        %let _fcfmsg_=NOTE: FCF: No Alerts generated for &runasofdate..;
        %fcf_put_error;
        %goto MACRO_END;
     %end;
  %end;
  %else %goto MACRO_END;

  /*--------------
   * Risk ranking
   *--------------*/
  %fcf_section_header(Section=AGP_RISKRANKING);
     %fcf_risk_ranking(inLib=STG_ALER,
                       inDS=combined_alerts,
                       inRiskLib=STG_ALER,
                       inRiskDS=risk_factor_alerts,
                       outLib=STG_ALER,
                       outDS=ranked_alerts,
                       outRiskLib=MST_ALER,
                       outRiskDS=ranked_risk_factor_alerts,
                       riskonly=Y,
                       %if %upcase(&dbflavor) eq TERADATA %then %do;
                        bl_temp=NO,
                       %end;
                       %else %do;
                        bl_temp=YES,
                       %end;
                       bl_tempload=);
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
     %end;
  %fcf_section_footer(Section=AGP_RISKRANKING);

  /*-------------
   * Suppression
   *-------------*/
  %fcf_section_header(Section=AGP_SUPPRESSION);
     %fcf_suppression(inLib=STG_ALER,
                      inDS=ranked_alerts,
                      outLib=STG_ALER,
                      outDS=suppressed_alerts);
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
     %end;
  %fcf_section_footer(Section=AGP_SUPPRESSION);

  /*-----------------------
   *Get entity information
   *-----------------------*/
  %fcf_section_header(Section=AGP_GET_ENTITY_INFO);
     %fcf_get_entity_info(inLIB=STG_ALER,
                          inDS=suppressed_alerts,
                          outLIB=STG_ALER,
                          outDS=entity_alerts,
                          %if %upcase(&dbflavor) eq TERADATA %then %do;
                           bl_temp=NO,
                          %end;
                          %else %do;
                           bl_temp=YES,
                          %end;
                          bl_tempload=);
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
     %end;
  %fcf_section_footer(Section=AGP_GET_ENTITY_INFO);

  /*---------
   * Routing
   *---------*/
  %fcf_section_header(Section=AGP_ROUTE);
     %fcf_route(inLib=STG_ALER,
                inDS=entity_alerts,
                outLib=MST_ALER,
                outDS=routed_alerts);
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
     %end;
  %fcf_section_footer(Section=AGP_ROUTE);

  /*----------------------------------
   * Make the UI unavailable to users
   *----------------------------------*/
  %fcf_set_ui_availability(availability_ind=N);

  /*-------------------------
   * Load alerts into the KC
   *-------------------------*/
  %fcf_section_header(Section=AGP_LOADALERTS);
     %fcf_load_alerts(routedAlertLib=MST_ALER,
                      routedAlertTable=routed_alerts);
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
     %end;
  %fcf_section_footer(Section=AGP_LOADALERTS);

  /*----------------------
   * Update alert history
   *----------------------*/
  %fcf_section_header(Section=AGP_UPDATE_ALERT_HISTORY);
     %fcf_update_alert_history(inLib=MST_ALER,
                               inDS=routed_alerts,
                               bulkload=);
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
     %end;
  %fcf_section_footer(Section=AGP_UPDATE_ALERT_HISTORY);

  /*-------------------------------
   * Load risk factors into the KC
   *-------------------------------*/
  %fcf_section_header(Section=AGP_LOADRISKFACTORS);
     %fcf_load_risk_factors(riskFactorLib=MST_ALER,
                            riskFactorTable=ranked_risk_factor_alerts);
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
     %end;
  %fcf_section_footer(Section=AGP_LOADRISKFACTORS);

  /*-------------------------------------------------------- 
   * Create replication temporary tables
   * %fcf_get_replicate_info must be run before
   * %fcf_replicate_transactions and %fcf_replicate_profile
   *--------------------------------------------------------*/
  %fcf_section_header(Section=AGP_GET_REPLICATE_INFO);
     %fcf_get_replicate_info(routedAlertLib=MST_ALER,
                             routedAlertTable=routed_alerts,
                             %if %upcase(&dbflavor) eq TERADATA %then %do;
                              bl_temp=NO,
                             %end;
                             %else %do;
                              bl_temp=YES,
                             %end;
                             bl_tempload=);
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
     %end;
  %fcf_section_footer(Section=AGP_GET_REPLICATE_INFO);

  /*---------------------------------------------------
   * Replicate transactions for FSC_ACCOUNT_EVENT_FACT
   *---------------------------------------------------*/
  %fcf_section_header(Section=AGP_REPLICATE_ACCOUNT_EVENT);
     %fcf_replicate_transactions(routedAlertLib=MST_ALER,
                                 routedAlertTable=routed_alerts,
                                 coreFactTable=fsc_account_event_fact,
                                 kcFactTable=fsk_account_event_alert);
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
     %end;
  %fcf_section_footer(Section=AGP_REPLICATE_ACCOUNT_EVENT);

  /*---------------------------------------------------
   * Replicate transactions for FSC_ACCOUNT_TRADE_FACT
   *---------------------------------------------------*/
  %fcf_section_header(Section=AGP_REPLICATE_ACCOUNT_TRADE);
     %fcf_replicate_transactions(routedAlertLib=MST_ALER,
                                 routedAlertTable=routed_alerts,
                                 coreFactTable=fsc_account_trade_fact,
                                 kcFactTable=fsk_account_trade_alert,
                                 date_key=%str(settlement_date_key));
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
     %end;
  %fcf_section_footer(Section=AGP_REPLICATE_ACCOUNT_TRADE);

  /*-----------------------------------------------
   * Replicate transactions for FSC_CASH_FLOW_FACT
   *-----------------------------------------------*/
  %fcf_section_header(Section=AGP_REPLICATE_CASH_FLOW);
     %fcf_replicate_transactions(routedAlertLib=MST_ALER,
                                 routedAlertTable=routed_alerts,
                                 coreFactTable=fsc_cash_flow_fact,
                                 kcFactTable=fsk_cash_flow_alert);
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
     %end;
  %fcf_section_footer(Section=AGP_REPLICATE_CASH_FLOW);

  /*------------------------------------------------
   * Replicate profile for FSC_ACCOUNT_PROFILE_FACT
   *------------------------------------------------*/
  %fcf_section_header(Section=AGP_REPLICATE_ACC_PROFILE);
     %fcf_replicate_profile(routedAlertLib=MST_ALER,
                            routedAlertTable=routed_alerts,
                            coreFactTable=fsc_account_profile_fact,
                            kcFactTable=fsk_account_profile_alert,
                            bulkload=);
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
     %end;
  %fcf_section_footer(Section=AGP_REPLICATE_ACC_PROFILE);

  /*----------------------------------------------
   * Replicate profile for FSC_PARTY_PROFILE_FACT
   *----------------------------------------------*/
  %fcf_section_header(Section=AGP_REPLICATE_PTY_PROFILE);
     %fcf_replicate_profile(routedAlertLib=MST_ALER,
                            routedAlertTable=routed_alerts,
                            coreFactTable=fsc_party_profile_fact,
                            kcFactTable=fsk_party_profile_alert,
                            bulkload=);
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
     %end;
  %fcf_section_footer(Section=AGP_REPLICATE_PTY_PROFILE);

  /*--------------------------------
   * Make the UI available to users
   *--------------------------------*/
  %fcf_set_ui_availability(availability_ind=Y);

  /*-----------------------------------------------------------
   * Optional:
   * Notify immediate managers and their reports of alerts
   * greater than a certain age.
   * The data set managerLib.managerDS must contain employees
   * and their managers in the following format, with the user
   * IDs being valid SAS metadata users:
   * variable:  employee_user_long_id  manager_user_long_id
   *            ---------------------  --------------------
   *            john_smith             bill_jones
   *            jim_chien              bill_jones
   *            janet_klein            bill_jones
   *            amit_patel             cindy_miller
   *            ...
   *-----------------------------------------------------------*/
   
   /*
       OPTIONAL COMPONENT -- Leave commented out until conditions
       listed above are satisfied and a manager data set has
       been created.

  %fcf_section_header(Section=AGP_NOTIFY_MANAGER);
     %fcf_notify_manager(age=30,
                         managerLib=mst_prep,
                         managerDS=employee_manager,
                         delimtype=CSV);
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
     %end;
  %fcf_section_footer(Section=AGP_NOTIFY_MANAGER);
  */

  /*----------------------
   * Send reminder e-mail
   *----------------------*/
  %fcf_section_header(Section=AGP_SEND_ALERT_REMINDER);
     %fcf_send_alert_reminder;
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
     %end;
  %fcf_section_footer(Section=AGP_SEND_ALERT_REMINDER);

  /*------------------------------------------------------------------
   * FCF IEM post-AGP push from AML into ECM
   *
   * Notes:
   * Input datasets are byproducts of the AMLAGP process, and must be 
   * located in the IEM work library.
   *------------------------------------------------------------------*/

  %fcf_iem_load_alerts(ds=iem.NEW_ALERTS);
  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
    %goto MACRO_END;
  %end;

  %fcf_iem_load_alert_rels(ds=iem.NEW_ALERTS);
  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
    %goto MACRO_END;
  %end;

  /*------------------------------------------------------------------
   * Clear the start datetime of the FCF AGP process.
   *------------------------------------------------------------------*/
  %fcf_start_datetime_clear;
  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
    %goto MACRO_END;
  %end;

  %MACRO_END:

   %if &trans_rc eq 3 or &trans_rc ge 5 %then
      %fcf_execute_macro(fcf_error);
   %else %do;
      %fcf_execute_macro(fcf_success);
   %end;

%mend fcf_agp;
