/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_iem_cdd_load_case.sas
| PRODUCT: SAS Financial Crimes Foundation
| SYSTEM:  Windows, Unix
| DESC:    Load CDD sourced Case into ECM
+------------------------------------------------------------------------------+
| USAGE:   %fcf_iem_cdd_load_case(ds=);
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2012-2013 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_iem_cdd_load_case(ds=);


    /*------------------------------------------
    * Set error code to 0 and message to blank
    *------------------------------------------*/
    %let trans_rc = 0;
    %let _fcfmsg_=;


	/*==========================================================================* 
	 * Step:            Join on Type, Cat, and Subcat                           * 
	 * Transform:       Join                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   CASE_CONFIG - ecm_db.CASE_CONFIG                        * 
	 *                  Input ds - &ds                       					* 
	 * Target Table:    Type, Cat, and Subcat match -                           * 
	 *                   work.type_cat_subcat_match                             * 
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete type_cat_subcat_match;
	quit;

	proc sql;
	   create view work.type_cat_subcat_match as
	   select
	      cdd_new_cases.CASE_RK,
	      cdd_new_cases.VALID_FROM_DTTM,
	      cdd_new_cases.VALID_TO_DTTM,
	      cdd_new_cases.CASE_ID,
	      cdd_new_cases.SOURCE_SYSTEM_CD,
	      cdd_new_cases.CASE_TYPE_CD,
	      cdd_new_cases.CASE_CATEGORY_CD,
	      cdd_new_cases.CASE_SUBCATEGORY_CD,
	      cdd_new_cases.CASE_STATUS_CD,
	      cdd_new_cases.CASE_DISPOSITION_CD,
	      cdd_new_cases.CASE_DESC,
	      cdd_new_cases.CASE_LINK_SK,
	      cdd_new_cases.PRIORITY_CD,
	      cdd_new_cases.REGULATORY_RPT_RQD_FLG,
	      cdd_new_cases.INVESTIGATOR_USER_ID,
	      cdd_new_cases.OPEN_DTTM,
	      cdd_new_cases.REOPEN_DTTM,
	      cdd_new_cases.CLOSE_DTTM,
	      cdd_new_cases.UI_DEF_FILE_NM,
	      cdd_new_cases.CREATE_USER_ID,
	      cdd_new_cases.CREATE_DTTM,
	      cdd_new_cases.UPDATE_USER_ID,
	      cdd_new_cases.VERSION_NO,
	      cdd_new_cases.DELETE_FLG,
	      CASE_CONFIG.CASE_CONFIG_SEQ_NO
	   from
	      &ds as cdd_new_cases, 
	      ecm_db.CASE_CONFIG
	   where
	      cdd_new_cases.CASE_TYPE_CD = CASE_CONFIG.CASE_TYPE_CD
	      and cdd_new_cases.CASE_CATEGORY_CD = CASE_CONFIG.CASE_CATEGORY_CD
	      and cdd_new_cases.CASE_SUBCATEGORY_CD = CASE_CONFIG.CASE_SUBCATEGORY_CD
		  and CASE_CONFIG.CASE_CATEGORY_CD NOT IS NULL 
	      and CASE_CONFIG.CASE_SUBCATEGORY_CD NOT IS NULL 
	   ;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to match Type, Category, and Subcategory for Case load;
		%fcf_put_error;
		%goto MACRO_END;
	%end;
	 


	/*==========================================================================* 
	 * Step:            Join on Type, and Cat                                   * 
	 * Transform:       Join                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   CASE_CONFIG - ecm_db.CASE_CONFIG                        * 
	 *                  Input ds - &ds                       					* 
	 * Target Table:    Type, and Cat match -                                   * 
	 *                   work.type_cat_match                                    * 
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete type_cat_match;
	quit;

	proc sql;
	   create view work.type_cat_match as
	   select
	      cdd_new_cases.CASE_RK,
	      cdd_new_cases.VALID_FROM_DTTM,
	      cdd_new_cases.VALID_TO_DTTM,
	      cdd_new_cases.CASE_ID,
	      cdd_new_cases.SOURCE_SYSTEM_CD,
	      cdd_new_cases.CASE_TYPE_CD,
	      cdd_new_cases.CASE_CATEGORY_CD,
	      cdd_new_cases.CASE_SUBCATEGORY_CD,
	      cdd_new_cases.CASE_STATUS_CD,
	      cdd_new_cases.CASE_DISPOSITION_CD,
	      cdd_new_cases.CASE_DESC,
	      cdd_new_cases.CASE_LINK_SK,
	      cdd_new_cases.PRIORITY_CD,
	      cdd_new_cases.REGULATORY_RPT_RQD_FLG,
	      cdd_new_cases.INVESTIGATOR_USER_ID,
	      cdd_new_cases.OPEN_DTTM,
	      cdd_new_cases.REOPEN_DTTM,
	      cdd_new_cases.CLOSE_DTTM,
	      cdd_new_cases.UI_DEF_FILE_NM,
	      cdd_new_cases.CREATE_USER_ID,
	      cdd_new_cases.CREATE_DTTM,
	      cdd_new_cases.UPDATE_USER_ID,
	      cdd_new_cases.VERSION_NO,
	      cdd_new_cases.DELETE_FLG,
	      CASE_CONFIG.CASE_CONFIG_SEQ_NO
	   from
	      &ds as cdd_new_cases, 
	      ecm_db.CASE_CONFIG
	   where
	      cdd_new_cases.CASE_TYPE_CD = CASE_CONFIG.CASE_TYPE_CD
	      and cdd_new_cases.CASE_CATEGORY_CD = CASE_CONFIG.CASE_CATEGORY_CD
	      and cdd_new_cases.CASE_SUBCATEGORY_CD IS NULL 
	      and CASE_CONFIG.CASE_CATEGORY_CD NOT IS NULL 
	   ;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to match Type, and Category for Case load;
		%fcf_put_error;
		%goto MACRO_END;
	%end;
	 

	/*==========================================================================* 
	 * Step:            Join on Type                                            * 
	 * Transform:       Join                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   CASE_CONFIG - ecm_db.CASE_CONFIG                        * 
	 *                  Input ds - &ds                       					* 
	 * Target Table:    Join on Type match - work.type_match                    * 
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete type_match;
	quit;

	proc sql;
	   create view work.type_match as
	   select
	      cdd_new_cases.CASE_RK,
	      cdd_new_cases.VALID_FROM_DTTM,
	      cdd_new_cases.VALID_TO_DTTM,
	      cdd_new_cases.CASE_ID,
	      cdd_new_cases.SOURCE_SYSTEM_CD,
	      cdd_new_cases.CASE_TYPE_CD,
	      cdd_new_cases.CASE_CATEGORY_CD,
	      cdd_new_cases.CASE_SUBCATEGORY_CD,
	      cdd_new_cases.CASE_STATUS_CD,
	      cdd_new_cases.CASE_DISPOSITION_CD,
	      cdd_new_cases.CASE_DESC,
	      cdd_new_cases.CASE_LINK_SK,
	      cdd_new_cases.PRIORITY_CD,
	      cdd_new_cases.REGULATORY_RPT_RQD_FLG,
	      cdd_new_cases.INVESTIGATOR_USER_ID,
	      cdd_new_cases.OPEN_DTTM,
	      cdd_new_cases.REOPEN_DTTM,
	      cdd_new_cases.CLOSE_DTTM,
	      cdd_new_cases.UI_DEF_FILE_NM,
	      cdd_new_cases.CREATE_USER_ID,
	      cdd_new_cases.CREATE_DTTM,
	      cdd_new_cases.UPDATE_USER_ID,
	      cdd_new_cases.VERSION_NO,
	      cdd_new_cases.DELETE_FLG,
	      CASE_CONFIG.CASE_CONFIG_SEQ_NO
	   from
	      &ds as cdd_new_cases, 
	      ecm_db.CASE_CONFIG
	   where
	      CASE_CONFIG.CASE_TYPE_CD = cdd_new_cases.CASE_TYPE_CD
	      and CASE_CONFIG.CASE_CATEGORY_CD IS NULL 
	      and CASE_CONFIG.CASE_SUBCATEGORY_CD IS NULL
	   ;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to match Type for Case load;
		%fcf_put_error;
		%goto MACRO_END;
	%end;
	 

	/*==========================================================================* 
	 * Step:            Append all matched sequence numbers                     * 
	 * Transform:       Append                                                  * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   Type, Cat, and Subcat match -                           * 
	 *                   work.type_cat_subcat_match                             * 
	 *                  Type, and Cat match -                                   * 
	 *                   work.type_cat_match                                    * 
	 *                  Join on Type match - work.type_match                    * 
	 * Target Table:    Case RKs with Group Sequence Numbers                    * 
	 *                   - work.case_rks_w_all_seq                              * 
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete case_rks_w_all_seq;
	quit;

	data work.case_rks_w_all_seq / view=work.case_rks_w_all_seq;
	   set work.type_cat_subcat_match
	       work.type_cat_match
	       work.type_match;
	   keep CASE_RK VALID_FROM_DTTM VALID_TO_DTTM CASE_ID SOURCE_SYSTEM_CD CASE_TYPE_CD 
	    CASE_CATEGORY_CD CASE_SUBCATEGORY_CD CASE_STATUS_CD CASE_DISPOSITION_CD 
	    CASE_DESC CASE_LINK_SK PRIORITY_CD REGULATORY_RPT_RQD_FLG 
	    INVESTIGATOR_USER_ID OPEN_DTTM REOPEN_DTTM CLOSE_DTTM UI_DEF_FILE_NM 
	    CREATE_USER_ID CREATE_DTTM UPDATE_USER_ID VERSION_NO DELETE_FLG 
	    CASE_CONFIG_SEQ_NO; 
	run;

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to append Type, Cat, and Subcat matches for Case load;
		%fcf_put_error;
		%goto MACRO_END;
	%end;
	 

	/*==========================================================================* 
	 * Step:            Get lowest Group Sequence numbers                       * 
	 * Transform:       Extract                                                 * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    Case RKs with Group Sequence Numbers                    * 
	 *                   - work.case_rks_w_all_seq                              * 
	 * Target Table:    Case RKs with Minimum Group Sequence                    * 
	 *                   Numbers - work.case_rk_w_min_seq                       * 
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(work.case_rks_w_all_seq); 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete case_rk_w_min_seq;
	quit;

	proc sql;
	   create view work.case_rk_w_min_seq as
	      select
	         CASE_RK,
	         VALID_FROM_DTTM,
	         VALID_TO_DTTM,
	         CASE_ID,
	         SOURCE_SYSTEM_CD,
	         CASE_TYPE_CD,
	         CASE_CATEGORY_CD,
	         CASE_SUBCATEGORY_CD,
	         CASE_STATUS_CD,
	         CASE_DISPOSITION_CD,
	         CASE_DESC,
	         CASE_LINK_SK,
	         PRIORITY_CD,
	         REGULATORY_RPT_RQD_FLG,
	         INVESTIGATOR_USER_ID,
	         OPEN_DTTM,
	         REOPEN_DTTM,
	         CLOSE_DTTM,
	         UI_DEF_FILE_NM,
	         CREATE_USER_ID,
	         CREATE_DTTM,
	         UPDATE_USER_ID,
	         VERSION_NO,
	         DELETE_FLG,
	         CASE_CONFIG_SEQ_NO
	   from &SOURCEDS
	      where CASE_CONFIG_SEQ_NO IN (SELECT MIN(CASE_CONFIG_SEQ_NO) AS CASE_CONFIG_SEQ_NO FROM case_rks_w_all_seq GROUP BY CASE_RK)
	   ;
	quit;

	%let SOURCEDS = work.case_rk_w_min_seq;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to find lowest Group Sequence Number for Case load;
		%fcf_put_error;
		%goto MACRO_END;
	%end;
	 

	/*==========================================================================* 
	 * Step:            Join                                                    * 
	 * Transform:       Join                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   Case RKs with Minimum Group Sequence                    * 
	 *                   Numbers - work.case_rk_w_min_seq                       * 
	 *                  CASE_CONFIG - ecm_db.CASE_CONFIG                        * 
	 * Target Table:    Staged Cases - work.staged_cases                        * 
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete staged_cases;
	quit;

	proc sql;
	   create view work.staged_cases as
	   select
	      case_rk_w_min_seq.CASE_RK,
	      case_rk_w_min_seq.VALID_FROM_DTTM,
	      case_rk_w_min_seq.VALID_TO_DTTM,
	      case_rk_w_min_seq.CASE_ID,
	      case_rk_w_min_seq.SOURCE_SYSTEM_CD,
	      case_rk_w_min_seq.CASE_TYPE_CD,
	      case_rk_w_min_seq.CASE_CATEGORY_CD,
	      case_rk_w_min_seq.CASE_SUBCATEGORY_CD,
	      case_rk_w_min_seq.CASE_STATUS_CD,
	      case_rk_w_min_seq.CASE_DISPOSITION_CD,
	      case_rk_w_min_seq.CASE_DESC,
	      case_rk_w_min_seq.CASE_LINK_SK,
	      case_rk_w_min_seq.PRIORITY_CD,
	      case_rk_w_min_seq.REGULATORY_RPT_RQD_FLG,
	      case_rk_w_min_seq.INVESTIGATOR_USER_ID,
	      case_rk_w_min_seq.OPEN_DTTM,
	      case_rk_w_min_seq.REOPEN_DTTM,
	      case_rk_w_min_seq.CLOSE_DTTM,
	      CASE_CONFIG.UI_DEF_FILE_NM,
	      case_rk_w_min_seq.CREATE_USER_ID,
	      case_rk_w_min_seq.CREATE_DTTM,
	      case_rk_w_min_seq.UPDATE_USER_ID,
	      case_rk_w_min_seq.VERSION_NO,
	      case_rk_w_min_seq.DELETE_FLG
	   from
	      work.case_rk_w_min_seq as case_rk_w_min_seq, 
	      ecm_db.CASE_CONFIG as CASE_CONFIG
	   where
	      case_rk_w_min_seq.CASE_CONFIG_SEQ_NO = CASE_CONFIG.CASE_CONFIG_SEQ_NO
	   ;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to retrieve UI Def file name for Case load;
		%fcf_put_error;
		%goto MACRO_END;
	%end;

	/*==========================================================================* 
	 * Step:            Load into CASE_LIVE                                     * 
	 * Transform:       Table Loader (Version 2.1)                              * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    Staged Cases - work.staged_cases                        * 
	 * Target Table:    CASE_LIVE - ecm_db.CASE_LIVE                            * 
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(work.staged_cases); 
	   
	proc append base = ecm_db.CASE_LIVE 
		data = &SOURCEDS; 
	run; 

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to append to CASE_LIVE for Case load;
		%fcf_put_error;
		%goto MACRO_END;
	%end;
	 
	/*==========================================================================* 
	 * Step:            Load into CASE_VERSION                                  * 
	 * Transform:       Table Loader (Version 2.1)                              * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    Staged Cases - work.staged_cases                        * 
	 * Target Table:    CASE_VERSION - ecm_db.CASE_VERSION                      * 
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(work.staged_cases); 
	   
	proc append base = ecm_db.CASE_VERSION 
		data = &SOURCEDS; 
	run; 

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to append to CASE_VERSION for Case load;
		%fcf_put_error;
		%goto MACRO_END;
	%end;
	 
	   
	/*==========================================================================* 
	 * Step:            Get all applicable user groups                          * 
	 * Transform:       Join                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   Case RKs with Minimum Group Sequence                    * 
	 *                   Numbers - work.case_rk_w_min_seq                       * 
	 *                  CASE_CONFIG_X_USER_GROUP -                              * 
	 *                   ecm_db.CASE_CONFIG_X_USER_GROUP                        * 
	 * Target Table:    Case RKs with User Group names -                        * 
	 *                   work.case_rk_w_groups                                  * 
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete case_rk_w_groups;
	quit;

	proc sql;
	   create view work.case_rk_w_groups as
	   select
	      case_rk_w_min_seq.CASE_RK,
	      CASE_CONFIG_X_USER_GROUP.USER_GROUP_NM
	   from
	      work.case_rk_w_min_seq, 
	      ecm_db.CASE_CONFIG_X_USER_GROUP
	   where
	      case_rk_w_min_seq.CASE_CONFIG_SEQ_NO = CASE_CONFIG_X_USER_GROUP.CASE_CONFIG_SEQ_NO
	   ;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to retrieve User Groups for Case load;
		%fcf_put_error;
		%goto MACRO_END;
	%end;
	 

	/*==========================================================================* 
	 * Step:            Load into CASE_X_USER_GROUP                             * 
	 * Transform:       Table Loader (Version 2.1)                              * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    Case RKs with User Group names -                        * 
	 *                   work.case_rk_w_groups                                  * 
	 * Target Table:    CASE_X_USER_GROUP -                                     * 
	 *                   ecm_db.CASE_X_USER_GROUP                               * 
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(work.case_rk_w_groups); 
	   
	proc append base = ecm_db.CASE_X_USER_GROUP 
		data = &SOURCEDS; 
	run; 

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to append to CASE_X_USER_GROUP for Case load;
		%fcf_put_error;
		%goto MACRO_END;
	%end;
 
      
    /*==========================================================================* 
     * Step:            Create events                                           * 
     * Transform:       Table Loader (Version 2.1)                              * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    Input ds - &ds                                          * 
     * Target Table:    New CDD Events - iem.new_cdd_events                     * 
     *==========================================================================*/ 

    %let SOURCEDS = %nrquote(&ds);

    proc datasets lib = iem nolist nowarn memtype = (data view);
        delete new_cdd_events;
    quit;

	data iem.new_cdd_events;
		if 0 then set ecm_db.ecm_event;
		set &SOURCEDS. (keep= CASE_RK CREATE_USER_ID CREATE_DTTM);
		BUSINESS_OBJECT_NM = "CASE";
		BUSINESS_OBJECT_RK = CASE_RK;
		EVENT_TYPE_CD = "LOADEN";
		EVENT_DESC = "event.etl.load.txt";
		CREATE_USER_ID = CREATE_USER_ID;
		CREATE_DTTM = CREATE_DTTM;
		drop CASE_RK;
	run;

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to create IEM.NEW_EVENTS;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

	%fcf_get_ecm_sequence_ids(in=iem.new_cdd_events,out=iem.new_cdd_events,type=event,block=&iem_performance_load);

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to generate new EVENT_RK primary keys for SUBJECT load;

        %fcf_put_error;
        %goto MACRO_END;
    %end;
	
    /*==========================================================================* 
     * Step:            Load into ECM_EVENT                                     * 
     * Transform:       Table Loader (Version 2.1)                              * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    New CDD Events - iem.new_cdd_events                     * 
     * Target Table:    ECM_EVENT - ecm_db.ECM_EVENT                            * 
     *==========================================================================*/ 

    %let SOURCEDS = %nrquote(iem.new_cdd_events); 
       
    proc append base = ecm_db.ECM_EVENT 
        data = &SOURCEDS force; 
    run; 

    /*==========================================================================* 
     * Step:            Get all char UDFs                                       * 
     * Transform:       Extract                                                 * 
     * Description:                                                             * 
     *                                                                          * 
	 * Source Table:    Input ds - &ds			                                * 
     * Target Table:    Extracted char values -                                 * 
     *                   work.extracted_char                                    * 
     *==========================================================================*/ 

    %let SOURCEDS = %nrquote(&ds); 

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete extracted_char;
    quit;

    proc sql;
        create view work.extracted_char as
        select
             CASE_RK as ECM_RK,
             VALID_FROM_DTTM,
             PRIMARY_ENTITY_NUMBER
        from &SOURCEDS
        ;
    quit;

    %let SOURCEDS = work.extracted_char;

    /*==========================================================================* 
     * Step:            Transpose                                               * 
     * Transform:       Transpose                                               * 
     *                                                                          * 
     * Source Table:    Extracted char values -                                 * 
     *                   work.extracted_char                                    * 
     * Target Table:    Transposed char values -                                * 
     *                   work.transposed_char                                   * 
     *==========================================================================*/ 

    %let SOURCEDS = %nrquote(work.extracted_char); 

    %let _INPUT = work.extracted_char;
    %let _OUTPUT = work.transposed_char;
	%let _OUTPUT_options = %nrquote(where = (n(ecm_rk)>0));

    proc datasets lib=work nolist nowarn memtype = (data view);
       delete transposed_char;
    quit;

    %let varColumns = PRIMARY_ENTITY_NUMBER;
    %let byColumns = ECM_RK VALID_FROM_DTTM;

    proc transpose data = &_INPUT out = &_OUTPUT (&_OUTPUT_options);
        by &byColumns;
        var &varColumns;
    run;

    %fcf_rcset(&syserr);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to transpose Character UDFs;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*==========================================================================* 
     * Step:            Load into CASE_UDF_CHAR_VALUE                           * 
     * Transform:       Table Loader (Version 2.1)                              * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    Transposed char values -                                * 
     *                   work.transposed_char                                   * 
     * Target Table:    CASE_UDF_CHAR_VALUE -                                   * 
     *                   ecm_db.CASE_UDF_CHAR_VALUE                             * 
     *==========================================================================*/ 

    %let SOURCEDS = %nrquote(work.transposed_char); 
       
    proc datasets lib = work nolist nowarn memtype = (data view);
        delete temp_map;
    quit;

	data temp_map;
		if 0 then set ecm_db.case_udf_char_value;
		set &SOURCEDS. (keep= ECM_RK _NAME_ COL1 VALID_FROM_DTTM);
		CASE_RK = ECM_RK;
		VALID_FROM_DTTM = VALID_FROM_DTTM;
		UDF_TABLE_NM = "CASE";
		UDF_NM = _NAME_;
		ROW_NO = 1;
		UDF_VALUE = COL1;
		drop ECM_RK _NAME_ COL1;
	run;

    %let SOURCEDS = work.temp_map;

    proc append base = ecm_db.CASE_UDF_CHAR_VALUE 
        data = &SOURCEDS force;
    run;

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to append to CASE_UDF_CHAR_VALUE;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete temp_map;
    quit;
       
    /*==========================================================================* 
     * Step:            Get all date UDFs                                       * 
     * Transform:       Extract                                                 * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    Input ds - &ds			                                * 
     * Target Table:    Extracted num values -                                  * 
     *                   work.extracted_date                                    * 
     *==========================================================================*/ 

    %let SOURCEDS = %nrquote(&ds); 

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete extracted_date;
    quit;

    proc sql;
        create view work.extracted_date as
        select
             CASE_RK as ECM_RK,
             VALID_FROM_DTTM,
             NEXT_REVIEW_DATE,
             LAST_REVIEW_DATE
        from &SOURCEDS
        ;
    quit;

    /*==========================================================================* 
     * Step:            Transpose                                               * 
     * Transform:       Transpose                                               * 
     *                                                                          * 
     * Source Table:    Extracted num values -                                  * 
     *                   work.extracted_date                                    * 
     * Target Table:    Transposed num values -                                 * 
     *                   work.transposed_date                                   * 
     *==========================================================================*/ 

    %let SOURCEDS = %nrquote(work.extracted_date); 

    %let _INPUT = work.extracted_date;
    %let _OUTPUT = work.transposed_date;
	%let _OUTPUT_options = %nrquote(where = (n(ecm_rk)>0));

    proc datasets lib=work nolist nowarn memtype = (data view);
        delete transposed_date;
    quit;

    %let varColumns = NEXT_REVIEW_DATE LAST_REVIEW_DATE;
    %let byColumns = ECM_RK VALID_FROM_DTTM;

    proc transpose data = &_INPUT out = &_OUTPUT (&_OUTPUT_options);
        by &byColumns;
        var &varColumns;
    run;

    %fcf_rcset(&syserr);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to transpose Date UDFs;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*==========================================================================* 
    * Step:            Remove null dates                                       * 
    * Transform:       Data Validation                                         * 
    * Description:                                                             * 
    *                                                                          * 
    * Source Table:    Transposed num values -                                 * 
    *                   work.transposed_num                                    * 
    * Target Tables:   Valid UDF dates - work.staged_date_udfs                 * 
    *                  Invalid UDF dates -                                     * 
    *                   work.invalid_dates                                     * 
    *==========================================================================*/ 

    %let SOURCEDS = %nrquote(work.transposed_date); 

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete staged_date_udfs;
    quit;

    data work.staged_date_udfs
            (keep = ECM_RK VALID_FROM_DTTM _NAME_ _LABEL_ COL1)
        work.invalid_dates
            (keep = ECM_RK VALID_FROM_DTTM _NAME_ _LABEL_ COL1);
        set &SOURCEDS end=eof;

        if (_NAME_ = "NEXT_REVIEW_DATE"  & COL1 = .) then do; 
            i_move = 1; 
        end;

        if (_NAME_ = "LAST_REVIEW_DATE"  & COL1 = .) then do; 
            i_move = 1; 
        end;

        if (i_move) then 
            output work.invalid_dates;
        else 
            output work.staged_date_udfs;
    run;

    %fcf_rcset(&syserr);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to validate DAte UDFs;
        %fcf_put_error;
        %goto MACRO_END;
    %end;          
      
    proc datasets library=work memtype=(data view) nolist nowarn;
        delete instaged_date_udfs;
    quit;

                
     /*=========================================================================* 
     * Step:            Load into CASE_UDF_DATE_VALUE                           * 
     * Transform:       Table Loader (Version 2.1)                              * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    Valid records table - work.staged_date_udfs             * 
     * Target Table:    CASE_UDF_DATE_VALUE -                                   * 
     *                   ecm_db.CASE_UDF_DATE_VALUE                             * 
     *==========================================================================*/ 

    %let SOURCEDS = %nrquote(work.staged_date_udfs); 

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete temp_map;
    quit;

	data temp_map;
		if 0 then set ecm_db.case_udf_date_value;
		set &SOURCEDS. (keep= ECM_RK _NAME_ COL1 VALID_FROM_DTTM);
		CASE_RK = ECM_RK;
		VALID_FROM_DTTM = VALID_FROM_DTTM;
		UDF_TABLE_NM = "CASE";
		UDF_NM = _NAME_;
		ROW_NO = 1;
		UDF_VALUE = COL1;
		drop ECM_RK _NAME_ COL1;
	run;

    %let SOURCEDS = work.temp_map;

    proc append base = ecm_db.CASE_UDF_DATE_VALUE 
        data = &SOURCEDS;
    run;

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to append to CASE_UDF_DATE_VALUE;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete temp_map;
    quit;
 

  %MACRO_END:

   %if &trans_rc eq 3 or &trans_rc ge 5 %then
      %fcf_execute_macro(fcf_error);
   %else %do;
      %fcf_execute_macro(fcf_success);
   %end;

%mend; 
