/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_iem_cdd_load_rel.sas
| PRODUCT: SAS Financial Crimes Foundation
| SYSTEM:  Windows, Unix
| DESC:    FCF to ECM Case and Incident creation and synchronization
+------------------------------------------------------------------------------+
| USAGE:   %fcf_iem_cdd_load_rel(ds=);
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2012-2013 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/
 

%macro fcf_iem_cdd_load_rel(ds=);

    /*------------------------------------------
    * Set error code to 0 and message to blank
    *------------------------------------------*/
    %let trans_rc = 0;
    %let _fcfmsg_=;

	/*==========================================================================* 
	 * Step:            Table Loader                                            * 
	 * Transform:       Table Loader (Version 2.1)                              * 
	 *																			*
	 * Source Table:    Macro input ds - &ds            			            *
	 * Target Table:    PARTY_X_PARTY - ecm_db.PARTY_X_PARTY                    * 
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(&ds); 
	   
	proc append base = ecm_db.PARTY_X_PARTY 
		data = &SOURCEDS force ; 
	run; 
	   
    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to load PARTY_X_PARTY relationships;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

	/*==========================================================================* 
	 * Step:            Table Loader                                            * 
	 * Transform:       Table Loader (Version 2.1)                              * 
	 *                                                                          *
	 * Source Table:    Macro input ds - &ds            			            *
	 * Target Table:    New CDD Events - iem.new_cdd_events                     * 
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(&ds); 

	
	data _null_;
    	call symput('current_dt',put(datetime(),datetime22.));
	run;

    proc datasets lib = iem nolist nowarn memtype = (data view);
        delete new_events;
    quit;

	data iem.new_events;
		if 0 then set ecm_db.ecm_event;
		set &SOURCEDS. (keep= PARTY_RK MEMBER_PARTY_RK);
		BUSINESS_OBJECT_NM = "PARTY";
		BUSINESS_OBJECT_RK = PARTY_RK;
		EVENT_TYPE_CD = "ADDPAR";
		EVENT_DESC = "event.etl.load.txt";
		CREATE_USER_ID = "&ecm_user";
		CREATE_DTTM = "&current_dt"dt;
		LINKED_OBJECT_NM = "PARTY";
		LINKED_OBJECT_RK = MEMBER_PARTY_RK;
		drop PARTY_RK MEMBER_PARTY_RK;
	run;

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_CDD_LOAD_REL;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	%let SOURCEDS = iem.new_events;

	%fcf_get_ecm_sequence_ids(in=iem.new_events,out=iem.new_events,type=event,block=&iem_performance_load);
	
	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_CDD_LOAD_REL;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	proc append base = ecm_db.ECM_EVENT 
		data = &SOURCEDS force; 
	run; 

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_CDD_LOAD_REL;
		%fcf_put_error;
		%goto MACRO_END;
	%end;

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_CDD_LOAD_REL;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 
	
	%MACRO_END:

    %if &trans_rc eq 3 or &trans_rc ge 5 %then
        %fcf_execute_macro(fcf_error);
    %else %do;
        %fcf_execute_macro(fcf_success);
    %end;
	   
%mend;
