/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_iem_cdd_upd_sc_event.sas
| PRODUCT: SAS Financial Crimes Foundation
| SYSTEM:  Windows, Unix
| DESC:    Update Scoring Event table with Case and Inc tracking information
+------------------------------------------------------------------------------+
| USAGE:   %fcf_iem_cdd_upd_sc_event(ds=,tracking=);
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2012-2013 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_iem_cdd_upd_sc_event(ds=,tracking=);

	/*------------------------------------------
	 * Set error code to 0 and message to blank
 	*------------------------------------------*/

	%let trans_rc = 0;
	%let _fcfmsg_=;

	/*==========================================================================* 
	 * Step:            Update                                                  * 
	 * Transform:       Update                                                  * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   Source DS -                                             * 
	 *                   &ds                                                    * 
	 *                  FSK_CDD_SCORING_EVENT -                                 * 
	 *                   &db_kc_libname..FSK_CDD_SCORING_EVENT                  * 
	 * Target Table:    FSK_CDD_SCORING_EVENT -                                 * 
	 *                   &db_kc_libname..FSK_CDD_SCORING_EVENT                  * 
 	*==========================================================================*/

	proc sql;
		connect to &dbflavor as &dbflavor(authdomain="&segkcauthdomain."
			&segKCDBConnOpts);
		execute (delete from &segKCSchema..FSK_TEMP_TABLE) by &dbflavor;
		execute (commit) by &dbflavor;
		disconnect from &dbflavor;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to empty FSK_TEMP_TABLE;
		%fcf_put_error;
		%goto MACRO_END;
	%end;

	data fsk_temp_table;
		if 0 then set &seg_kc_libname..fsk_temp_table;
		set &ds.	(keep=CDD_SCORING_EVENT_KEY INVESTIGATION_RK);

		TEMP_NUM_1_VALUE=CDD_SCORING_EVENT_KEY;
		TEMP_NUM_2_VALUE=INVESTIGATION_RK;

		drop 	CDD_SCORING_EVENT_KEY INVESTIGATION_RK;
	run;

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to create staging dataset for FSK_TEMP_TABLE load;
		%fcf_put_error;
		%goto MACRO_END;
	%end;

	proc append	base=&seg_kc_libname..fsk_temp_table data=work.fsk_temp_table force;
	quit;

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to append to FSK_TEMP_TABLE;
		%fcf_put_error;
		%goto MACRO_END;
	%end;

	proc sql;
		connect to &dbflavor as &dbflavor (authdomain="&segkcauthdomain." &segKCDBConnOpts);
		execute (
			merge into fsk_cdd_scoring_event sc_e using (
				select 	TEMP_NUM_1_VALUE,
						TEMP_NUM_2_VALUE
				from 	FSK_TEMP_TABLE
			) fsk_tmp
			on (
				fsk_tmp.TEMP_NUM_1_VALUE = sc_e.cdd_scoring_event_key 
			)
			when matched then update set 
				sc_e.investigation_rk = fsk_tmp.TEMP_NUM_2_VALUE,
				%if %upcase(&tracking=ALERT) %then %do;
					sc_e.investigation_object_code = 'A',
				%end;

				%if %upcase(&tracking=CASE) %then %do;
					sc_e.investigation_object_code = 'C',
				%end;

				sc_e.investigation_ind = 'N'
		) by &dbflavor;
		disconnect from &dbflavor;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to update generated ECM foreign keys in FSK_CDD_SCORING_EVENT;
		%fcf_put_error;
		%goto MACRO_END;
	%end;

%MACRO_END:

	%if &trans_rc eq 3 or &trans_rc ge 5 %then
		%fcf_execute_macro(fcf_error);
	%else %do;
		%fcf_execute_macro(fcf_success);
	%end;
%mend;