/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_iem_fb_man_alerts_risks.sas
| PRODUCT: SAS Financial Crimes Foundation
| SYSTEM:  Windows, Unix
| DESC:    ECM to FCF Incident creation and synchronization
+------------------------------------------------------------------------------+
| USAGE:   %fcf_iem_fb_man_alerts_risks;
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2012-2013 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_iem_fb_man_alerts_risks;

    /*------------------------------------------
     * Set error code to 0 and message to blank
     *------------------------------------------*/

    %let trans_rc = 0;
    %let _fcfmsg_=;

	/*==========================================================================* 
	 * Step:            Extract                                                 *
	 * Transform:       Extract                                                 * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    INCIDENT_LIVE - ecm_db.INCIDENT_LIVE                    *
	 * Target Table:    Extract - work.manual_alerts                            *
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(ecm_db.INCIDENT_LIVE); 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete manual_alerts;
	quit;

	proc sql;
	   create view work.manual_alerts as
	      select
	         INCIDENT_RK,
	         VALID_FROM_DTTM,
	         VALID_TO_DTTM,
	         CASE_RK,
	         ((input(substr(INCIDENT_ID,5),13.))) as INCIDENT_ID,
	         SOURCE_SYSTEM_CD,
	         INCIDENT_TYPE_CD,
	         INCIDENT_CATEGORY_CD,
	         INCIDENT_SUBCATEGORY_CD,
	         INCIDENT_DESC,
	         INCIDENT_FROM_DT,
	         INCIDENT_FROM_TM,
	         INCIDENT_TO_DT,
	         INCIDENT_TO_TM,
	         DETECTION_DT,
	         DETECTION_TM,
	         NOTIFICATION_DT,
	         NOTIFICATION_TM,
	         UI_DEF_FILE_NM,
	         CREATE_USER_ID,
	         CREATE_DTTM,
	         UPDATE_USER_ID,
	         VERSION_NO,
	         DELETE_FLG,
	         INVESTIGATOR_USER_ID,
	         INCIDENT_DISPOSITION_CD,
	         CLOSE_DTTM,
	         INCIDENT_STATUS_CD
	   from &SOURCEDS
	      where date() - datepart(VALID_FROM_DTTM) <= 100 /*& SOURCE_SYSTEM_CD="&segKCschema"*/ & (INCIDENT_TYPE_CD = "AMLALT"  & INCIDENT_CATEGORY_CD = "MANUAL")
	   ;
	quit;

    %fcf_rcset(&sqlrc);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_FB_MAN_ALERTS_RISKS;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

	/*==========================================================================* 
	 * Step:            Data Validation                                         *
	 * Transform:       Data Validation                                         * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    Extract - work.manual_alerts                            *
	 * Target Table:    Data Validation - work.fixed_alerts                      *
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(work.manual_alerts); 

	 
	proc datasets lib = work nolist nowarn memtype = (data view);
	  delete fixed_alerts;
	quit;

	/*---- Begin checks for Custom, Missing Values, and Invalid Values  ----*/ 
	data work.fixed_alerts
	     (keep = INCIDENT_RK VALID_FROM_DTTM VALID_TO_DTTM CASE_RK INCIDENT_ID 
	                     SOURCE_SYSTEM_CD INCIDENT_TYPE_CD INCIDENT_CATEGORY_CD 
	                     INCIDENT_SUBCATEGORY_CD INCIDENT_DESC INCIDENT_FROM_DT 
	                     INCIDENT_FROM_TM INCIDENT_TO_DT INCIDENT_TO_TM DETECTION_DT 
	                     DETECTION_TM NOTIFICATION_DT NOTIFICATION_TM UI_DEF_FILE_NM 
	                     CREATE_USER_ID CREATE_DTTM UPDATE_USER_ID VERSION_NO 
	                     DELETE_FLG INVESTIGATOR_USER_ID INCIDENT_DISPOSITION_CD 
	                     CLOSE_DTTM INCIDENT_STATUS_CD);

		set &SOURCEDS end=eof;

		if (INCIDENT_DISPOSITION_CD = 'SUP') then 
		do; 
			 INCIDENT_DISPOSITION_CD = 'ACT'; 
		end;
		else 
		do; 
		end;  /* else */

		output work.fixed_alerts;
	  
	run; 

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_FB_MAN_ALERTS_RISKS;

        %fcf_put_error;
        %goto MACRO_END;
    %end;
	   
	/*==========================================================================* 
	 * Step:            Lookup                                                  *
	 * Transform:       Lookup                                                  * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   Data Validation - work.fixed_alerts                     *
	 *                  FSK_ALERT - &db_kc_libname..FSK_ALERT                   *
	 * Target Tables:   Lookup - work.existing_alerts                           *
	 *                  Error Table - work.new_alerts_temp                      *
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete new_alerts_temp;
	quit;

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete existing_alerts;
	quit;

	DATA  work.existing_alerts
	         (keep = INCIDENT_RK VALID_FROM_DTTM VALID_TO_DTTM CASE_RK INCIDENT_ID 
	                 SOURCE_SYSTEM_CD INCIDENT_TYPE_CD INCIDENT_CATEGORY_CD 
	                 INCIDENT_SUBCATEGORY_CD INCIDENT_DESC INCIDENT_FROM_DT 
	                 INCIDENT_FROM_TM INCIDENT_TO_DT INCIDENT_TO_TM DETECTION_DT 
	                 DETECTION_TM NOTIFICATION_DT NOTIFICATION_TM UI_DEF_FILE_NM 
	                 CREATE_USER_ID CREATE_DTTM UPDATE_USER_ID VERSION_NO DELETE_FLG 
	                 INVESTIGATOR_USER_ID INCIDENT_DISPOSITION_CD CLOSE_DTTM 
	                 INCIDENT_STATUS_CD )
	      work.new_alerts_temp
	         (keep = INCIDENT_RK VALID_FROM_DTTM VALID_TO_DTTM CASE_RK INCIDENT_ID 
	                 SOURCE_SYSTEM_CD INCIDENT_TYPE_CD INCIDENT_CATEGORY_CD 
	                 INCIDENT_SUBCATEGORY_CD INCIDENT_DESC INCIDENT_FROM_DT 
	                 INCIDENT_FROM_TM INCIDENT_TO_DT INCIDENT_TO_TM DETECTION_DT 
	                 DETECTION_TM NOTIFICATION_DT NOTIFICATION_TM UI_DEF_FILE_NM 
	                 CREATE_USER_ID CREATE_DTTM UPDATE_USER_ID VERSION_NO DELETE_FLG 
	                 INVESTIGATOR_USER_ID INCIDENT_DISPOSITION_CD CLOSE_DTTM 
	                 INCIDENT_STATUS_CD); 
	   
	   retain missing0-missing0 0; 

	      
	   etls_assign_target_value = 0; 
	   /* Assign variables used for error/exception handling  */ 
	   etls_source_row      = _n_;
	   etls_write_to_error_table  = 0; 
	   
	   /* Build hash objects from lookup tables before reading first source row  */ 
	   if (_n_ = 1) then 
	   do; 
	      /* Build hash h0 from lookup table &db_kc_libname..FSK_ALERT */ 
	      nlobs = .; 
	      dsid = open("&db_kc_libname..FSK_ALERT"); 
	      if (dsid > 0) then 
	      do; 
	         if ( attrc(dsid, 'MTYPE') = 'DATA' ) then 
	            nlobs = attrn(dsid, 'NLOBS'); 
	         else 
	            nlobs = -1; 
	            
	         dsid = close(dsid); 
	         if (nlobs ^= 0) then 
	         do; 
	            if (nlobs > 0) then 
	               exponent = ceil(log2(nlobs)); 
	            else 
	               exponent = 8; 
	            declare hash h0(dataset: "&db_kc_libname..FSK_ALERT", hashexp: exponent);
	            h0.defineKey( "ALERT_ID");
	            h0.defineDone();
	            if (nlobs = -1) then 
	            do; 
	               if (h0.Num_Items < 1) then
	               do; 
	                  put "NOTE: Lookup table is empty: &db_kc_libname..FSK_ALERT";
	                  missing0 = 98; 
	               end; 
	            end; 
	         end; 
	         else 
	         do; 
	            put "NOTE: Lookup table is empty: &db_kc_libname..FSK_ALERT";
	            missing0 = 98; 
	         end; 
	      end; 
	      else 
	      do; 
	         put "NOTE: Lookup table does not exist or cannot be opened:"
	              " &db_kc_libname..FSK_ALERT";
	         
	         put "NOTE: Abort action indicated, condition= Lookup table missing:"
	              " &db_kc_libname..FSK_ALERT";
	         
	         abort 3; 
	      end; 
	      
	      call missing (ALERT_ID); 
	   
	   end; /* All hash objects have been defined */
	   
	   /* Read a row from the source table  */ 
	   set work.fixed_alerts end = eof;
	   
	   /* Is the current key value stored in hash h0?  */ 
	   if (missing0 < 1) then 
	   do; 
	      rc0 = h0.check(key: INCIDENT_ID);
	   end; 
	   else 
	      rc0 = missing0; 
	      
	   /* Examine success of lookups  */ 
	   if ( rc0=0 ) then 
	   do; 
	      /* Write row to target  */ 
	      output work.existing_alerts; 
	   end; 
	   else 
	   do; 
	      
	      if (rc0 ^= 0) then 
	      do; 

	         if (rc0 = 98) then 
	         do; 
	            /* Check: Lookup table has no rows-Write exception row  */ 
	            etls_write_to_error_table = 1; 
	            /* Check: Lookup table has no rows-Set target columns to value  */ 
	         end; 
	         
	         else 
	         do; 
	            etls_write_to_error_table = 1; 
	         end; 
	      end; 
	      
	      /* Write row to error table?  */ 
	      if (etls_write_to_error_table eq 1) then 
	         output work.new_alerts_temp; 
	   
	   end; /* One or more lookups failed */
	   
	   if (eof = 1) then 
	   do; 
	      put "New Alerts to be loaded: " error_total ; 
	   end; 

	run; 

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_FB_MAN_ALERTS_RISKS;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete new_alerts;
	quit;

	proc sql;
		create table work.new_alerts as (
			select
				new.*,
				substr(p_l.party_id,1,3) as primary_entity_level_code
			from
				work.new_alerts_temp new
			left join
				ecm_db.incident_x_party i_x_p
			on
				new.incident_rk = i_x_p.incident_rk
			left join
				ecm_db.party_live p_l
			on
				i_x_p.party_rk = p_l.party_rk
			where
				i_x_p.relation_type_cd = "ALT2SUB" and
				p_l.party_type_cd = "FCFSUB"
		);
	quit;

    %fcf_rcset(&sqlrc);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_FB_MAN_ALERTS_RISKS;

        %fcf_put_error;
        %goto MACRO_END;
    %end;


	/*==========================================================================* 
	 * Step:            Extract                                                 *
	 * Transform:       Extract                                                 * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    INCIDENT_LIVE - ecm_db.INCIDENT_LIVE                    *
	 * Target Table:    Extract - work.manual_risks                             *
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(ecm_db.INCIDENT_LIVE); 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete manual_risks;
	quit;

	proc sql;
	   create view work.manual_risks as
	      select
	         INCIDENT_RK,
	         VALID_FROM_DTTM,
	         VALID_TO_DTTM,
	         CASE_RK,
	         ((input(substr(INCIDENT_ID,5),13.))) as INCIDENT_ID,
	         SOURCE_SYSTEM_CD,
	         INCIDENT_TYPE_CD,
	         INCIDENT_CATEGORY_CD,
	         INCIDENT_SUBCATEGORY_CD,
	         INCIDENT_DESC,
	         INCIDENT_FROM_DT,
	         INCIDENT_FROM_TM,
	         INCIDENT_TO_DT,
	         INCIDENT_TO_TM,
	         DETECTION_DT,
	         DETECTION_TM,
	         NOTIFICATION_DT,
	         NOTIFICATION_TM,
	         UI_DEF_FILE_NM,
	         CREATE_USER_ID,
	         CREATE_DTTM,
	         UPDATE_USER_ID,
	         VERSION_NO,
	         DELETE_FLG,
	         INVESTIGATOR_USER_ID,
	         INCIDENT_DISPOSITION_CD,
	         CLOSE_DTTM,
	         INCIDENT_STATUS_CD
	   from &SOURCEDS
	      where date() - datepart(VALID_FROM_DTTM) <= 100 /*& SOURCE_SYSTEM_CD="&segKCschema" */ & (INCIDENT_TYPE_CD  = "AMLRISK"  & INCIDENT_CATEGORY_CD = "MANRISK")
	   ;
	quit;

    %fcf_rcset(&sqlrc);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_FB_MAN_ALERTS_RISKS;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

	%let SOURCEDS = work.manual_risks;


	/*==========================================================================* 
	 * Step:            Lookup                                                  *
	 * Transform:       Lookup                                                  * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   Extract - work.manual_risks                             *
	 *                  FSK_RISK_ASSESSMENT -                                   *
	 *                   &db_kc_libname..FSK_RISK_ASSESSMENT                    * 
	 * Target Tables:   Lookup - work.existing_risks                            *
	 *                  Error Table - work.new_risks                            *
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete new_risks;
	quit;

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete existing_risks;
	quit;

	DATA  work.existing_risks
	         (keep = INCIDENT_RK VALID_FROM_DTTM VALID_TO_DTTM CASE_RK INCIDENT_ID 
	                 SOURCE_SYSTEM_CD INCIDENT_TYPE_CD INCIDENT_CATEGORY_CD 
	                 INCIDENT_SUBCATEGORY_CD INCIDENT_DESC INCIDENT_FROM_DT 
	                 INCIDENT_FROM_TM INCIDENT_TO_DT INCIDENT_TO_TM DETECTION_DT 
	                 DETECTION_TM NOTIFICATION_DT NOTIFICATION_TM UI_DEF_FILE_NM 
	                 CREATE_USER_ID CREATE_DTTM UPDATE_USER_ID VERSION_NO DELETE_FLG 
	                 INVESTIGATOR_USER_ID INCIDENT_DISPOSITION_CD CLOSE_DTTM 
	                 INCIDENT_STATUS_CD )
	      work.new_risks
	         (keep = INCIDENT_RK VALID_FROM_DTTM VALID_TO_DTTM CASE_RK INCIDENT_ID 
	                 SOURCE_SYSTEM_CD INCIDENT_TYPE_CD INCIDENT_CATEGORY_CD 
	                 INCIDENT_SUBCATEGORY_CD INCIDENT_DESC INCIDENT_FROM_DT 
	                 INCIDENT_FROM_TM INCIDENT_TO_DT INCIDENT_TO_TM DETECTION_DT 
	                 DETECTION_TM NOTIFICATION_DT NOTIFICATION_TM UI_DEF_FILE_NM 
	                 CREATE_USER_ID CREATE_DTTM UPDATE_USER_ID VERSION_NO DELETE_FLG 
	                 INVESTIGATOR_USER_ID INCIDENT_DISPOSITION_CD CLOSE_DTTM 
	                 INCIDENT_STATUS_CD); 
	   	   
	   retain missing0-missing0 0; 
	   
	      
	   etls_assign_target_value = 0; 
	   /* Assign variables used for error/exception handling  */ 
	   etls_source_row      = _n_;
	   etls_write_to_error_table  = 0; 
	   
	   /* Build hash objects from lookup tables before reading first source row  */ 
	   if (_n_ = 1) then 
	   do; 
	      /* Build hash h0 from lookup table &db_kc_libname..FSK_RISK_ASSESSMENT */ 
	      nlobs = .; 
	      dsid = open("&db_kc_libname..FSK_RISK_ASSESSMENT"); 
	      if (dsid > 0) then 
	      do; 
	         if ( attrc(dsid, 'MTYPE') = 'DATA' ) then 
	            nlobs = attrn(dsid, 'NLOBS'); 
	         else 
	            nlobs = -1; 
	            
	         dsid = close(dsid); 
	         if (nlobs ^= 0) then 
	         do; 
	            if (nlobs > 0) then 
	               exponent = ceil(log2(nlobs)); 
	            else 
	               exponent = 8; 
	            declare hash h0(dataset: "&db_kc_libname..FSK_RISK_ASSESSMENT", hashexp: exponent);
	            h0.defineKey( "RISK_ASSESSMENT_ID");
	            h0.defineDone();
	            if (nlobs = -1) then 
	            do; 
	               if (h0.Num_Items < 1) then
	               do; 
	                  put "NOTE: Lookup table is empty: &db_kc_libname..FSK_RISK_ASSESSMENT";
	                  missing0 = 98; 
	               end; 
	            end; 
	         end; 
	         else 
	         do; 
	            put "NOTE: Lookup table is empty: &db_kc_libname..FSK_RISK_ASSESSMENT";
	            missing0 = 98; 
	         end; 
	      end; 
	      else 
	      do; 
	         put "NOTE: Lookup table does not exist or cannot be opened:"
	              " &db_kc_libname..FSK_RISK_ASSESSMENT";
	         
	         put "NOTE: Abort action indicated, condition= Lookup table missing:"
	              " &db_kc_libname..FSK_RISK_ASSESSMENT";
	         
	         abort 3; 
	      end; 
	      
	      call missing (RISK_ASSESSMENT_ID); 
	   
	   end; /* All hash objects have been defined */
	   
	   /* Read a row from the source table  */ 
	   set work.manual_risks end = eof;
	   
	   /* Is the current key value stored in hash h0?  */ 
	   if (missing0 < 1) then 
	   do; 
	      rc0 = h0.check(key: INCIDENT_ID);
	   end; 
	   else 
	      rc0 = missing0; 
	      
	   /* Examine success of lookups  */ 
	   if ( rc0=0 ) then 
	   do; 
	      /* Write row to target  */ 
	      output work.existing_risks; 
	   end; 
	   else 
	   do; 
	      error_total + 1; 
	      
	      if (rc0 ^= 0) then 
	      do; 
	         exception_total + 1; 
	         if (rc0 = 98) then 
	         do; 
	            /* Check: Lookup table has no rows-Write exception row  */ 
	            etls_write_to_error_table = 1; 
	            /* Check: Lookup table has no rows-Set target columns to value  */ 
	         end; 
	         
	         else 
	         do; 
	            etls_write_to_error_table = 1; 
	         end; 
	      end; 
	      
	      /* Write row to error table?  */ 
	      if (etls_write_to_error_table eq 1) then 
	         output work.new_risks; 
	   
	   end; /* One or more lookups failed */
	   
	   if (eof = 1) then 
	   do; 
	      put "New Risks to be loaded: " error_total ;
	   end; 

	run; 

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_FB_MAN_ALERTS_RISKS;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

	/*==========================================================================* 
	 * Step:            Join                                                    *
	 * Transform:       Join                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   INCIDENT_UDF_NUM_VALUE -                                *
	 *                   ecm_db.INCIDENT_UDF_NUM_VALUE                          * 
	 *                  Error Table - work.new_alerts                           *
	 * Target Table:    Join - work.all_alt_num_udfs                            *
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete all_alt_num_udfs;
	quit;

	proc sql;
	   create view work.all_alt_num_udfs as
	   select
	      INCIDENT_UDF_NUM_VALUE.INCIDENT_RK,
	      INCIDENT_UDF_NUM_VALUE.VALID_FROM_DTTM,
	      INCIDENT_UDF_NUM_VALUE.UDF_TABLE_NM,
	      INCIDENT_UDF_NUM_VALUE.UDF_NM,
	      INCIDENT_UDF_NUM_VALUE.UDF_VALUE
	   from
	      work.new_alerts, 
	      ecm_db.INCIDENT_UDF_NUM_VALUE
	   where
	      new_alerts.INCIDENT_RK = INCIDENT_UDF_NUM_VALUE.INCIDENT_RK
	      and new_alerts.VALID_FROM_DTTM = INCIDENT_UDF_NUM_VALUE.VALID_FROM_DTTM
	   order by
	      new_alerts.VALID_FROM_DTTM,
	      new_alerts.INCIDENT_RK
	   ;
	quit;

    %fcf_rcset(&sqlrc);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_FB_MAN_ALERTS_RISKS;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

	/*==========================================================================* 
	 * Step:            Join                                                    *
	 * Transform:       Join                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   INCIDENT_UDF_DATE_VALUE -                               *
	 *                   ecm_db.INCIDENT_UDF_DATE_VALUE                         * 
	 *                  Error Table - work.new_alerts                           *
	 * Target Table:    Join - work.all_alt_date_udfs                           *
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete all_alt_date_udfs;
	quit;

	proc sql;
	   create view work.all_alt_date_udfs as
	   select
	      INCIDENT_UDF_DATE_VALUE.INCIDENT_RK,
	      INCIDENT_UDF_DATE_VALUE.VALID_FROM_DTTM,
	      INCIDENT_UDF_DATE_VALUE.UDF_NM,
	      INCIDENT_UDF_DATE_VALUE.UDF_VALUE
	   from
	      work.new_alerts, 
	      ecm_db.INCIDENT_UDF_DATE_VALUE
	   where
	      new_alerts.INCIDENT_RK = INCIDENT_UDF_DATE_VALUE.INCIDENT_RK
	      and new_alerts.VALID_FROM_DTTM = INCIDENT_UDF_DATE_VALUE.VALID_FROM_DTTM
	   order by
	      new_alerts.VALID_FROM_DTTM,
	      new_alerts.INCIDENT_RK
	   ;
	quit;

    %fcf_rcset(&sqlrc);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_FB_MAN_ALERTS_RISKS;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

	/*==========================================================================* 
	 * Step:            Join                                                    *
	 * Transform:       Join                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   INCIDENT_UDF_CHAR_VALUE -                               *
	 *                   ecm_db.INCIDENT_UDF_CHAR_VALUE                         * 
	 *                  Error Table - work.new_alerts                           *
	 * Target Table:    Join - work.all_alt_char_udfs                           *
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete all_alt_char_udfs;
	quit;

	proc sql;
	   create view work.all_alt_char_udfs as
	   select
	      INCIDENT_UDF_CHAR_VALUE.INCIDENT_RK,
	      INCIDENT_UDF_CHAR_VALUE.VALID_FROM_DTTM,
	      INCIDENT_UDF_CHAR_VALUE.UDF_NM,
	      INCIDENT_UDF_CHAR_VALUE.UDF_VALUE
	   from
	      work.new_alerts, 
	      ecm_db.INCIDENT_UDF_CHAR_VALUE
	   where
	      new_alerts.INCIDENT_RK = INCIDENT_UDF_CHAR_VALUE.INCIDENT_RK
	      and new_alerts.VALID_FROM_DTTM = INCIDENT_UDF_CHAR_VALUE.VALID_FROM_DTTM
	   order by
	      new_alerts.VALID_FROM_DTTM,
	      new_alerts.INCIDENT_RK
	   ;
	quit;

    %fcf_rcset(&sqlrc);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_FB_MAN_ALERTS_RISKS;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

	/*==========================================================================* 
	 * Step:            Join                                                    *
	 * Transform:       Join                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   Error Table - work.new_risks                            *
	 *                  INCIDENT_UDF_NUM_VALUE -                                *
	 *                   ecm_db.INCIDENT_UDF_NUM_VALUE                          * 
	 * Target Table:    Join - work.all_rsk_num_udfs                            *
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete all_rsk_num_udfs;
	quit;

	proc sql;
	   create view work.all_rsk_num_udfs as
	   select
	      INCIDENT_UDF_NUM_VALUE.INCIDENT_RK,
	      INCIDENT_UDF_NUM_VALUE.VALID_FROM_DTTM,
	      INCIDENT_UDF_NUM_VALUE.UDF_NM,
	      INCIDENT_UDF_NUM_VALUE.UDF_VALUE
	   from
	      work.new_risks, 
	      ecm_db.INCIDENT_UDF_NUM_VALUE
	   where
	      new_risks.INCIDENT_RK = INCIDENT_UDF_NUM_VALUE.INCIDENT_RK
	      and new_risks.VALID_FROM_DTTM = INCIDENT_UDF_NUM_VALUE.VALID_FROM_DTTM
	   ;
	quit;

    %fcf_rcset(&sqlrc);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_FB_MAN_ALERTS_RISKS;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

	/*==========================================================================* 
	 * Step:            Join                                                    *
	 * Transform:       Join                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   Error Table - work.new_risks                            *
	 *                  INCIDENT_UDF_DATE_VALUE -                               *
	 *                   ecm_db.INCIDENT_UDF_DATE_VALUE                         * 
	 * Target Table:    Join - work.all_rsk_date_udfs                           *
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete all_rsk_date_udfs;
	quit;

	proc sql;
	   create view work.all_rsk_date_udfs as
	   select
	      INCIDENT_UDF_DATE_VALUE.INCIDENT_RK,
	      INCIDENT_UDF_DATE_VALUE.VALID_FROM_DTTM,
	      INCIDENT_UDF_DATE_VALUE.UDF_NM,
	      INCIDENT_UDF_DATE_VALUE.UDF_VALUE
	   from
	      work.new_risks, 
	      ecm_db.INCIDENT_UDF_DATE_VALUE
	   where
	      new_risks.INCIDENT_RK = INCIDENT_UDF_DATE_VALUE.INCIDENT_RK
	      and new_risks.VALID_FROM_DTTM = INCIDENT_UDF_DATE_VALUE.VALID_FROM_DTTM
	   ;
	quit;

	%fcf_rcset(&sqlrc);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_FB_MAN_ALERTS_RISKS;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

	/*==========================================================================* 
	 * Step:            Join                                                    *
	 * Transform:       Join                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   Error Table - work.new_risks                            *
	 *                  INCIDENT_UDF_CHAR_VALUE -                               *
	 *                   ecm_db.INCIDENT_UDF_CHAR_VALUE                         * 
	 * Target Table:    Join - work.all_rsk_char_udfs                           *
	 *==========================================================================*/ 
	 
	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete all_rsk_char_udfs;
	quit;

	proc sql;
	   create view work.all_rsk_char_udfs as
	   select
	      INCIDENT_UDF_CHAR_VALUE.INCIDENT_RK,
	      INCIDENT_UDF_CHAR_VALUE.VALID_FROM_DTTM,
	      INCIDENT_UDF_CHAR_VALUE.UDF_NM,
	      INCIDENT_UDF_CHAR_VALUE.UDF_VALUE
	   from
	      work.new_risks, 
	      ecm_db.INCIDENT_UDF_CHAR_VALUE
	   where
	      new_risks.INCIDENT_RK = INCIDENT_UDF_CHAR_VALUE.INCIDENT_RK
	      and new_risks.VALID_FROM_DTTM = INCIDENT_UDF_CHAR_VALUE.VALID_FROM_DTTM
	   ;
	quit;

    %fcf_rcset(&sqlrc);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_FB_MAN_ALERTS_RISKS;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

	/*==========================================================================* 
	 * Step:            Lookup                                                  *
	 * Transform:       Lookup                                                  * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   Error Table - work.new_alerts                           *
	 *                  Join - work.all_alt_num_udfs                            *
	 * Target Table:    Lookup - work.alt_pty_key                               *
	 *==========================================================================*/ 

	%let SOURCEDS=work.new_alerts;

    %let obs_count = %fcf_get_observation_count(table=&SOURCEDS,SASDATASET=Y);

    %if &obs_count>0 %then %do;

		proc datasets lib = work nolist nowarn memtype = (data view);
		   delete etls_temp_lookupview0;
		quit;

		proc sql; create view etls_temp_lookupview0 as 
		   select * from work.all_alt_num_udfs
		   where UDF_NM = 'X_SASFCS_PTY_KEY';
		quit;
		proc datasets lib = work nolist nowarn memtype = (data view);
		   delete alt_pty_key;
		quit;

		/*---- Begin DATA step to perform lookups  ----*/ 
		DATA  work.alt_pty_key
		         (keep = INCIDENT_RK VALID_FROM_DTTM VALID_TO_DTTM CASE_RK INCIDENT_ID 
		                 SOURCE_SYSTEM_CD INCIDENT_TYPE_CD INCIDENT_CATEGORY_CD 
		                 INCIDENT_SUBCATEGORY_CD INCIDENT_DESC INCIDENT_FROM_DT 
		                 INCIDENT_FROM_TM INCIDENT_TO_DT INCIDENT_TO_TM DETECTION_DT 
		                 DETECTION_TM NOTIFICATION_DT NOTIFICATION_TM UI_DEF_FILE_NM 
		                 CREATE_USER_ID CREATE_DTTM UPDATE_USER_ID VERSION_NO DELETE_FLG 
		                 INVESTIGATOR_USER_ID INCIDENT_DISPOSITION_CD CLOSE_DTTM 
		                 INCIDENT_STATUS_CD PRIMARY_ENTITY_LEVEL_CODE X_SASFCS_PTY_KEY )
		      ; 
		   
		   retain missing0-missing0 0; 
		   
		   etls_assign_target_value = 0; 
		   /* Build hash objects from lookup tables before reading first source row  */ 
		   if (_n_ = 1) then 
		   do; 
		      /* Build hash h0 from lookup table etls_temp_lookupview0 */ 
		      nlobs = .; 
		      dsid = open("etls_temp_lookupview0"); 
		      if (dsid > 0) then 
		      do; 
		         if ( attrc(dsid, 'MTYPE') = 'DATA' ) then 
		            nlobs = attrn(dsid, 'NLOBS'); 
		         else 
		            nlobs = -1; 
		            
		         dsid = close(dsid); 
		         if (nlobs ^= 0) then 
		         do; 
		            if (nlobs > 0) then 
		               exponent = ceil(log2(nlobs)); 
		            else 
		               exponent = 8; 
		            declare hash h0(dataset: "etls_temp_lookupview0", hashexp: exponent);
		            h0.defineKey( "INCIDENT_RK");
		            h0.defineData( "UDF_VALUE");
		            h0.defineDone();
		            if (nlobs = -1) then 
		            do; 
		               if (h0.Num_Items < 1) then
		               do; 
		                  put "NOTE: Lookup table is empty: etls_temp_lookupview0";
		                  missing0 = 98; 
		               end; 
		            end; 
		         end; 
		         else 
		         do; 
		            put "NOTE: Lookup table is empty: etls_temp_lookupview0";
		            missing0 = 98; 
		         end; 
		      end; 
		      else 
		      do; 
		         put "NOTE: Lookup table does not exist or cannot be opened:"
		              " etls_temp_lookupview0";
		         
		         put "NOTE: Abort action indicated, condition= Lookup table missing:"
		              " etls_temp_lookupview0";
		         
		         abort 3; 
		      end; 
		      
		      call missing (INCIDENT_RK, UDF_VALUE); 
		   
		   end; /* All hash objects have been defined */
		   
		   /* Read a row from the source table  */ 
		   set work.new_alerts end = eof;
		   
		   /* Is the current key value stored in hash h0?  */ 
		   if (missing0 < 1) then 
		   do; 
		      rc0 = h0.find(key: INCIDENT_RK);
		      X_SASFCS_PTY_KEY = UDF_VALUE;
		   end; 
		   else 
		      rc0 = missing0; 
		      
		   /* Examine success of lookups  */ 
		   if ( rc0=0 ) then 
		   do; 
		      /* Write row to target  */ 
		      output work.alt_pty_key; 
		   end; 
		   else 
		   do; 
		      error_total + 1; 
		      
		      if (rc0 ^= 0) then 
		      do; 
		         exception_total + 1; 
		         if (rc0 = 98) then 
		         do; 
		            /* Check: Lookup table has no rows-Write exception row  */ 
		            /* Check: Lookup table has no rows-Set target columns to value  */ 
		            X_SASFCS_PTY_KEY = .;
		            etls_assign_target_value = 1; 
		         end; 
		         
		         else 
		         do; 
		            /* Check: Lookup value not found-Set target columns to value  */ 
		            X_SASFCS_PTY_KEY = .;
		            etls_assign_target_value = 1; 
		         end; 
		      end; 
		      
		      /* Set target columns to value/missing requested?  */ 
		      if (etls_assign_target_value eq 1) then 
		         /* Write row to target  */ 
		         output work.alt_pty_key; 
		   
		   end; /* One or more lookups failed */
		   
		   if (eof = 1) then 
		   do; 
		      put "Source records with errors: " error_total ; 
		      put "Total lookup exceptions: " exception_total ; 
		   end; 

		run; 

	    %fcf_rcset(&syserr);

	    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
	        %let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_FB_MAN_ALERTS_RISKS;

	        %fcf_put_error;
	        %goto MACRO_END;
	    %end;


		/*==========================================================================* 
		 * Step:            Lookup                                                  *
		 * Transform:       Lookup                                                  * 
		 * Description:                                                             * 
		 *                                                                          * 
		 * Source Tables:   Lookup - work.alt_pty_key                               *
		 *                  Join - work.all_alt_num_udfs                            *
		 * Target Table:    Lookup - work.alt_ml_rs                                 *
		 *==========================================================================*/ 

		proc datasets lib = work nolist nowarn memtype = (data view);
		   delete etls_temp_lookupview0;
		quit;

		proc sql; create view etls_temp_lookupview0 as 
		   select * from work.all_alt_num_udfs
		   where UDF_NM = 'X_SASFCS_ML_RS';
		quit;
		proc datasets lib = work nolist nowarn memtype = (data view);
		   delete alt_ml_rs;
		quit;

		DATA  work.alt_ml_rs
		         (keep = INCIDENT_RK VALID_FROM_DTTM VALID_TO_DTTM CASE_RK INCIDENT_ID 
		                 SOURCE_SYSTEM_CD INCIDENT_TYPE_CD INCIDENT_CATEGORY_CD 
		                 INCIDENT_SUBCATEGORY_CD INCIDENT_DESC INCIDENT_FROM_DT 
		                 INCIDENT_FROM_TM INCIDENT_TO_DT INCIDENT_TO_TM DETECTION_DT 
		                 DETECTION_TM NOTIFICATION_DT NOTIFICATION_TM UI_DEF_FILE_NM 
		                 CREATE_USER_ID CREATE_DTTM UPDATE_USER_ID VERSION_NO DELETE_FLG 
		                 INVESTIGATOR_USER_ID INCIDENT_DISPOSITION_CD CLOSE_DTTM 
		                 INCIDENT_STATUS_CD PRIMARY_ENTITY_LEVEL_CODE X_SASFCS_PTY_KEY
						 X_SASFCS_ML_RS )
		      ; 
		   
		   retain missing0-missing0 0; 
		   
		   etls_assign_target_value = 0; 
		   /* Build hash objects from lookup tables before reading first source row  */ 
		   if (_n_ = 1) then 
		   do; 
		      /* Build hash h0 from lookup table etls_temp_lookupview0 */ 
		      nlobs = .; 
		      dsid = open("etls_temp_lookupview0"); 
		      if (dsid > 0) then 
		      do; 
		         if ( attrc(dsid, 'MTYPE') = 'DATA' ) then 
		            nlobs = attrn(dsid, 'NLOBS'); 
		         else 
		            nlobs = -1; 
		            
		         dsid = close(dsid); 
		         if (nlobs ^= 0) then 
		         do; 
		            if (nlobs > 0) then 
		               exponent = ceil(log2(nlobs)); 
		            else 
		               exponent = 8; 
		            declare hash h0(dataset: "etls_temp_lookupview0", hashexp: exponent);
		            h0.defineKey( "INCIDENT_RK");
		            h0.defineData( "UDF_VALUE");
		            h0.defineDone();
		            if (nlobs = -1) then 
		            do; 
		               if (h0.Num_Items < 1) then
		               do; 
		                  put "NOTE: Lookup table is empty: etls_temp_lookupview0";
		                  missing0 = 98; 
		               end; 
		            end; 
		         end; 
		         else 
		         do; 
		            put "NOTE: Lookup table is empty: etls_temp_lookupview0";
		            missing0 = 98; 
		         end; 
		      end; 
		      else 
		      do; 
		         put "NOTE: Lookup table does not exist or cannot be opened:"
		              " etls_temp_lookupview0";
		         
		         put "NOTE: Abort action indicated, condition= Lookup table missing:"
		              " etls_temp_lookupview0";
		         
		         abort 3; 
		      end; 
		      
		      call missing (INCIDENT_RK, UDF_VALUE); 
		   
		   end; /* All hash objects have been defined */
		   
		   /* Read a row from the source table  */ 
		   set work.alt_pty_key end = eof;
		   
		   /* Is the current key value stored in hash h0?  */ 
		   if (missing0 < 1) then 
		   do; 
		      rc0 = h0.find(key: INCIDENT_RK);
		      X_SASFCS_ML_RS = UDF_VALUE;
		   end; 
		   else 
		      rc0 = missing0; 
		      
		   /* Examine success of lookups  */ 
		   if ( rc0=0 ) then 
		   do; 
		      /* Write row to target  */ 
		      output work.alt_ml_rs; 
		   end; 
		   else 
		   do; 
		      error_total + 1; 
		      
		      if (rc0 ^= 0) then 
		      do; 
		         exception_total + 1; 
		         if (rc0 = 98) then 
		         do; 
		            /* Check: Lookup table has no rows-Write exception row  */ 
		            /* Check: Lookup table has no rows-Set target columns to value  */ 
		            X_SASFCS_ML_RS = .;
		            etls_assign_target_value = 1; 
		         end; 
		         
		         else 
		         do; 
		            /* Check: Lookup value not found-Set target columns to value  */ 
		            X_SASFCS_ML_RS = .;
		            etls_assign_target_value = 1; 
		         end; 
		      end; 
		      
		      /* Set target columns to value/missing requested?  */ 
		      if (etls_assign_target_value eq 1) then 
		         /* Write row to target  */ 
		         output work.alt_ml_rs; 
		   
		   end; /* One or more lookups failed */
		   
		   if (eof = 1) then 
		   do; 
		      put "Source records with errors: " error_total ; 
		      put "Total lookup exceptions: " exception_total ; 
		   end; 

		run; 

	    %fcf_rcset(&syserr);

	    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
	        %let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_FB_MAN_ALERTS_RISKS;

	        %fcf_put_error;
	        %goto MACRO_END;
	    %end;

		/*==========================================================================* 
		 * Step:            Lookup                                                  *
		 * Transform:       Lookup                                                  * 
		 * Description:                                                             * 
		 *                                                                          * 
		 * Source Tables:   Lookup - work.alt_ml_rs                                 *
		 *                  Join - work.all_alt_date_udfs                           *
		 * Target Table:    Lookup - work.alt_run_date                              *
		 *==========================================================================*/ 

		proc datasets lib = work nolist nowarn memtype = (data view);
		   delete etls_temp_lookupview0;
		quit;

		proc sql; create view etls_temp_lookupview0 as 
		   select * from work.all_alt_date_udfs
		   where UDF_NM = 'X_ALT_RUN_DATE';
		quit;
		proc datasets lib = work nolist nowarn memtype = (data view);
		   delete alt_run_date;
		quit;

		DATA  work.alt_run_date
		         (keep = INCIDENT_RK VALID_FROM_DTTM VALID_TO_DTTM CASE_RK INCIDENT_ID 
		                 SOURCE_SYSTEM_CD INCIDENT_TYPE_CD INCIDENT_CATEGORY_CD 
		                 INCIDENT_SUBCATEGORY_CD INCIDENT_DESC INCIDENT_FROM_DT 
		                 INCIDENT_FROM_TM INCIDENT_TO_DT INCIDENT_TO_TM DETECTION_DT 
		                 DETECTION_TM NOTIFICATION_DT NOTIFICATION_TM UI_DEF_FILE_NM 
		                 CREATE_USER_ID CREATE_DTTM UPDATE_USER_ID VERSION_NO DELETE_FLG 
		                 INVESTIGATOR_USER_ID INCIDENT_DISPOSITION_CD CLOSE_DTTM 
		                 INCIDENT_STATUS_CD PRIMARY_ENTITY_LEVEL_CODE X_SASFCS_PTY_KEY
						 X_SASFCS_ML_RS X_ALT_RUN_DATE )
		      ; 
		   
		   retain missing0-missing0 0; 
		   
		   etls_assign_target_value = 0; 
		   /* Build hash objects from lookup tables before reading first source row  */ 
		   if (_n_ = 1) then 
		   do; 
		      /* Build hash h0 from lookup table etls_temp_lookupview0 */ 
		      nlobs = .; 
		      dsid = open("etls_temp_lookupview0"); 
		      if (dsid > 0) then 
		      do; 
		         if ( attrc(dsid, 'MTYPE') = 'DATA' ) then 
		            nlobs = attrn(dsid, 'NLOBS'); 
		         else 
		            nlobs = -1; 
		            
		         dsid = close(dsid); 
		         if (nlobs ^= 0) then 
		         do; 
		            if (nlobs > 0) then 
		               exponent = ceil(log2(nlobs)); 
		            else 
		               exponent = 8; 
		            declare hash h0(dataset: "etls_temp_lookupview0", hashexp: exponent);
		            h0.defineKey( "INCIDENT_RK");
		            h0.defineData( "UDF_VALUE");
		            h0.defineDone();
		            if (nlobs = -1) then 
		            do; 
		               if (h0.Num_Items < 1) then
		               do; 
		                  put "NOTE: Lookup table is empty: etls_temp_lookupview0";
		                  missing0 = 98; 
		               end; 
		            end; 
		         end; 
		         else 
		         do; 
		            put "NOTE: Lookup table is empty: etls_temp_lookupview0";
		            missing0 = 98; 
		         end; 
		      end; 
		      else 
		      do; 
		         put "NOTE: Lookup table does not exist or cannot be opened:"
		              " etls_temp_lookupview0";
		         
		         put "NOTE: Abort action indicated, condition= Lookup table missing:"
		              " etls_temp_lookupview0";
		         
		         abort 3; 
		      end; 
		      
		      call missing (INCIDENT_RK, UDF_VALUE); 
		   
		   end; /* All hash objects have been defined */
		   
		   /* Read a row from the source table  */ 
		   set work.alt_ml_rs end = eof;
		   
		   /* Is the current key value stored in hash h0?  */ 
		   if (missing0 < 1) then 
		   do; 
		      rc0 = h0.find(key: INCIDENT_RK);
		      X_ALT_RUN_DATE = UDF_VALUE;
		   end; 
		   else 
		      rc0 = missing0; 
		      
		   /* Examine success of lookups  */ 
		   if ( rc0=0 ) then 
		   do; 
		      /* Write row to target  */ 
		      output work.alt_run_date; 
		   end; 
		   else 
		   do; 
		      error_total + 1; 
		      
		      if (rc0 ^= 0) then 
		      do; 
		         exception_total + 1; 
		         if (rc0 = 98) then 
		         do; 
		            /* Check: Lookup table has no rows-Write exception row  */ 
		            /* Check: Lookup table has no rows-Set target columns to value  */ 
		            X_ALT_RUN_DATE = .;
		            etls_assign_target_value = 1; 
		         end; 
		         
		         else 
		         do; 
		            /* Check: Lookup value not found-Set target columns to value  */ 
		            X_ALT_RUN_DATE = .;
		            etls_assign_target_value = 1; 
		         end; 
		      end; 
		      
		      /* Set target columns to value/missing requested?  */ 
		      if (etls_assign_target_value eq 1) then 
		         /* Write row to target  */ 
		         output work.alt_run_date; 
		   
		   end; /* One or more lookups failed */
		   
		   if (eof = 1) then 
		   do; 
		      put "Source records with errors: " error_total ; 
		      put "Total lookup exceptions: " exception_total ; 
		   end; 

		run; 

	    %fcf_rcset(&syserr);

	    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
	        %let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_FB_MAN_ALERTS_RISKS;

	        %fcf_put_error;
	        %goto MACRO_END;
	    %end;

		/*==========================================================================* 
		 * Step:            Lookup                                                  *
		 * Transform:       Lookup                                                  * 
		 * Description:                                                             * 
		 *                                                                          * 
		 * Source Tables:   Lookup - work.alt_run_date                              *
		 *                  Join - work.all_alt_date_udfs                           *
		 * Target Table:    Lookup - work.alt_supp_end_date                         *
		 *==========================================================================*/ 

		proc datasets lib = work nolist nowarn memtype = (data view);
		   delete etls_temp_lookupview0;
		quit;

		proc sql; create view etls_temp_lookupview0 as 
		   select * from work.all_alt_date_udfs
		   where UDF_NM = 'ALT_SUPPRESSION_END_DATE';
		quit;
		proc datasets lib = work nolist nowarn memtype = (data view);
		   delete alt_supp_end_date;
		quit;

		DATA  work.alt_supp_end_date
		         (keep = INCIDENT_RK VALID_FROM_DTTM VALID_TO_DTTM CASE_RK INCIDENT_ID 
		                 SOURCE_SYSTEM_CD INCIDENT_TYPE_CD INCIDENT_CATEGORY_CD 
		                 INCIDENT_SUBCATEGORY_CD INCIDENT_DESC INCIDENT_FROM_DT 
		                 INCIDENT_FROM_TM INCIDENT_TO_DT INCIDENT_TO_TM DETECTION_DT 
		                 DETECTION_TM NOTIFICATION_DT NOTIFICATION_TM UI_DEF_FILE_NM 
		                 CREATE_USER_ID CREATE_DTTM UPDATE_USER_ID VERSION_NO DELETE_FLG 
		                 INVESTIGATOR_USER_ID INCIDENT_DISPOSITION_CD CLOSE_DTTM 
		                 INCIDENT_STATUS_CD PRIMARY_ENTITY_LEVEL_CODE X_SASFCS_PTY_KEY
						 X_SASFCS_ML_RS X_ALT_RUN_DATE ALT_SUPPRESSION_END_DATE )
		      ; 
		   
		   retain missing0-missing0 0; 
		   
		   etls_assign_target_value = 0; 
		   /* Build hash objects from lookup tables before reading first source row  */ 
		   if (_n_ = 1) then 
		   do; 
		      /* Build hash h0 from lookup table etls_temp_lookupview0 */ 
		      nlobs = .; 
		      dsid = open("etls_temp_lookupview0"); 
		      if (dsid > 0) then 
		      do; 
		         if ( attrc(dsid, 'MTYPE') = 'DATA' ) then 
		            nlobs = attrn(dsid, 'NLOBS'); 
		         else 
		            nlobs = -1; 
		            
		         dsid = close(dsid); 
		         if (nlobs ^= 0) then 
		         do; 
		            if (nlobs > 0) then 
		               exponent = ceil(log2(nlobs)); 
		            else 
		               exponent = 8; 
		            declare hash h0(dataset: "etls_temp_lookupview0", hashexp: exponent);
		            h0.defineKey( "INCIDENT_RK");
		            h0.defineData( "UDF_VALUE");
		            h0.defineDone();
		            if (nlobs = -1) then 
		            do; 
		               if (h0.Num_Items < 1) then
		               do; 
		                  put "NOTE: Lookup table is empty: etls_temp_lookupview0";
		                  missing0 = 98; 
		               end; 
		            end; 
		         end; 
		         else 
		         do; 
		            put "NOTE: Lookup table is empty: etls_temp_lookupview0";
		            missing0 = 98; 
		         end; 
		      end; 
		      else 
		      do; 
		         put "NOTE: Lookup table does not exist or cannot be opened:"
		              " etls_temp_lookupview0";
		         
		         put "NOTE: Abort action indicated, condition= Lookup table missing:"
		              " etls_temp_lookupview0";
		         
		         abort 3; 
		      end; 
		      
		      call missing (INCIDENT_RK, UDF_VALUE); 
		   
		   end; /* All hash objects have been defined */
		   
		   /* Read a row from the source table  */ 
		   set work.alt_run_date end = eof;
		   
		   /* Is the current key value stored in hash h0?  */ 
		   if (missing0 < 1) then 
		   do; 
		      rc0 = h0.find(key: INCIDENT_RK);
		      ALT_SUPPRESSION_END_DATE = UDF_VALUE;
		   end; 
		   else 
		      rc0 = missing0; 
		      
		   /* Examine success of lookups  */ 
		   if ( rc0=0 ) then 
		   do; 
		      /* Write row to target  */ 
		      output work.alt_supp_end_date; 
		   end; 
		   else 
		   do; 
		      error_total + 1; 
		      
		      if (rc0 ^= 0) then 
		      do; 
		         exception_total + 1; 
		         if (rc0 = 98) then 
		         do; 
		            /* Check: Lookup table has no rows-Write exception row  */ 
		            /* Check: Lookup table has no rows-Set target columns to value  */ 
		            ALT_SUPPRESSION_END_DATE = .;
		            etls_assign_target_value = 1; 
		         end; 
		         
		         else 
		         do; 
		            /* Check: Lookup value not found-Set target columns to value  */ 
		            ALT_SUPPRESSION_END_DATE = .;
		            etls_assign_target_value = 1; 
		         end; 
		      end; 
		      
		      /* Set target columns to value/missing requested?  */ 
		      if (etls_assign_target_value eq 1) then 
		         /* Write row to target  */ 
		         output work.alt_supp_end_date; 
		   
		   end; /* One or more lookups failed */
		   
		   if (eof = 1) then 
		   do; 
		      put "Source records with errors: " error_total ; 
		      put "Total lookup exceptions: " exception_total ; 
		   end; 

		run;

		%fcf_rcset(&syserr);

	    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
	        %let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_FB_MAN_ALERTS_RISKS;

	        %fcf_put_error;
	        %goto MACRO_END;
	    %end; 

		/*==========================================================================* 
		 * Step:            Lookup                                                  *
		 * Transform:       Lookup                                                  * 
		 * Description:                                                             * 
		 *                                                                          * 
		 * Source Tables:   Lookup - work.alt_supp_end_date                         *
		 *                  Join - work.all_alt_char_udfs                           *
		 * Target Table:    Lookup - work.alt_pty_nm                                *
		 *==========================================================================*/ 

		proc datasets lib = work nolist nowarn memtype = (data view);
		   delete etls_temp_lookupview0;
		quit;

		proc sql; create view etls_temp_lookupview0 as 
		   select * from work.all_alt_char_udfs
		   where UDF_NM = 'X_SASFCS_PTY_NM';
		quit;
		proc datasets lib = work nolist nowarn memtype = (data view);
		   delete alt_pty_nm;
		quit;

		/*---- Begin DATA step to perform lookups  ----*/ 
		DATA  work.alt_pty_nm
		         (keep = INCIDENT_RK VALID_FROM_DTTM VALID_TO_DTTM CASE_RK INCIDENT_ID 
		                 SOURCE_SYSTEM_CD INCIDENT_TYPE_CD INCIDENT_CATEGORY_CD 
		                 INCIDENT_SUBCATEGORY_CD INCIDENT_DESC INCIDENT_FROM_DT 
		                 INCIDENT_FROM_TM INCIDENT_TO_DT INCIDENT_TO_TM DETECTION_DT 
		                 DETECTION_TM NOTIFICATION_DT NOTIFICATION_TM UI_DEF_FILE_NM 
		                 CREATE_USER_ID CREATE_DTTM UPDATE_USER_ID VERSION_NO DELETE_FLG 
		                 INVESTIGATOR_USER_ID INCIDENT_DISPOSITION_CD CLOSE_DTTM 
		                 INCIDENT_STATUS_CD PRIMARY_ENTITY_LEVEL_CODE X_SASFCS_PTY_KEY
						 X_SASFCS_ML_RS X_ALT_RUN_DATE ALT_SUPPRESSION_END_DATE
						 X_SASFCS_PTY_NM )
		      ; 
		   	  attrib UDF_VALUE length= $4000; 
		   retain missing0-missing0 0; 
		   
		   etls_assign_target_value = 0; 
		   /* Build hash objects from lookup tables before reading first source row  */ 
		   if (_n_ = 1) then 
		   do; 
		      /* Build hash h0 from lookup table etls_temp_lookupview0 */ 
		      nlobs = .; 
		      dsid = open("etls_temp_lookupview0"); 
		      if (dsid > 0) then 
		      do; 
		         if ( attrc(dsid, 'MTYPE') = 'DATA' ) then 
		            nlobs = attrn(dsid, 'NLOBS'); 
		         else 
		            nlobs = -1; 
		            
		         dsid = close(dsid); 
		         if (nlobs ^= 0) then 
		         do; 
		            if (nlobs > 0) then 
		               exponent = ceil(log2(nlobs)); 
		            else 
		               exponent = 8; 
		            declare hash h0(dataset: "etls_temp_lookupview0", hashexp: exponent);
		            h0.defineKey( "INCIDENT_RK");
		            h0.defineData( "UDF_VALUE");
		            h0.defineDone();
		            if (nlobs = -1) then 
		            do; 
		               if (h0.Num_Items < 1) then
		               do; 
		                  put "NOTE: Lookup table is empty: etls_temp_lookupview0";
		                  missing0 = 98; 
		               end; 
		            end; 
		         end; 
		         else 
		         do; 
		            put "NOTE: Lookup table is empty: etls_temp_lookupview0";
		            missing0 = 98; 
		         end; 
		      end; 
		      else 
		      do; 
		         put "NOTE: Lookup table does not exist or cannot be opened:"
		              " etls_temp_lookupview0";
		         
		         put "NOTE: Abort action indicated, condition= Lookup table missing:"
		              " etls_temp_lookupview0";
		         
		         abort 3; 
		      end; 
		      
		      call missing (INCIDENT_RK, UDF_VALUE); 
		   
		   end; /* All hash objects have been defined */
		   
		   /* Read a row from the source table  */ 
		   set work.alt_supp_end_date end = eof;
		   
		   /* Is the current key value stored in hash h0?  */ 
		   if (missing0 < 1) then 
		   do; 
		      rc0 = h0.find(key: INCIDENT_RK);
		      X_SASFCS_PTY_NM = UDF_VALUE;
		   end; 
		   else 
		      rc0 = missing0; 
		      
		   /* Examine success of lookups  */ 
		   if ( rc0=0 ) then 
		   do; 
		      /* Write row to target  */ 
		      output work.alt_pty_nm; 
		   end; 
		   else 
		   do; 
		      error_total + 1; 
		      
		      if (rc0 ^= 0) then 
		      do; 
		         exception_total + 1; 
		         if (rc0 = 98) then 
		         do; 
		            /* Check: Lookup table has no rows-Write exception row  */ 
		            /* Check: Lookup table has no rows-Set target columns to value  */ 
		            X_SASFCS_PTY_NM = '';
		            etls_assign_target_value = 1; 
		         end; 
		         
		         else 
		         do; 
		            /* Check: Lookup value not found-Set target columns to value  */ 
		            X_SASFCS_PTY_NM = '';
		            etls_assign_target_value = 1; 
		         end; 
		      end; 
		      
		      /* Set target columns to value/missing requested?  */ 
		      if (etls_assign_target_value eq 1) then 
		         /* Write row to target  */ 
		         output work.alt_pty_nm; 
		   
		   end; /* One or more lookups failed */
		   
		   if (eof = 1) then 
		   do; 
		      put "Source records with errors: " error_total ; 
		      put "Total lookup exceptions: " exception_total ; 
		   end; 

		run; 

	    %fcf_rcset(&syserr);

	    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
	        %let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_FB_MAN_ALERTS_RISKS;

	        %fcf_put_error;
	        %goto MACRO_END;
	    %end;

		/*==========================================================================* 
		 * Step:            Lookup                                                  *
		 * Transform:       Lookup                                                  * 
		 * Description:                                                             * 
		 *                                                                          * 
		 * Source Tables:   Lookup - work.alt_pty_nm                                *
		 *                  Join - work.all_alt_char_udfs                           *
		 * Target Table:    Lookup - work.alt_pty_no                                *
		 *==========================================================================*/ 

		proc datasets lib = work nolist nowarn memtype = (data view);
		   delete etls_temp_lookupview0;
		quit;

		proc sql; create view etls_temp_lookupview0 as 
		   select * from work.all_alt_char_udfs
		   where UDF_NM = 'X_SASFCS_PTY_NO';
		quit;
		proc datasets lib = work nolist nowarn memtype = (data view);
		   delete alt_pty_no;
		quit;

		/*---- Begin DATA step to perform lookups  ----*/ 
		DATA  work.alt_pty_no
		         (keep = INCIDENT_RK VALID_FROM_DTTM VALID_TO_DTTM CASE_RK INCIDENT_ID 
		                 SOURCE_SYSTEM_CD INCIDENT_TYPE_CD INCIDENT_CATEGORY_CD 
		                 INCIDENT_SUBCATEGORY_CD INCIDENT_DESC INCIDENT_FROM_DT 
		                 INCIDENT_FROM_TM INCIDENT_TO_DT INCIDENT_TO_TM DETECTION_DT 
		                 DETECTION_TM NOTIFICATION_DT NOTIFICATION_TM UI_DEF_FILE_NM 
		                 CREATE_USER_ID CREATE_DTTM UPDATE_USER_ID VERSION_NO DELETE_FLG 
		                 INVESTIGATOR_USER_ID INCIDENT_DISPOSITION_CD CLOSE_DTTM 
		                 INCIDENT_STATUS_CD PRIMARY_ENTITY_LEVEL_CODE X_SASFCS_PTY_KEY
						 X_SASFCS_ML_RS X_ALT_RUN_DATE ALT_SUPPRESSION_END_DATE
						 X_SASFCS_PTY_NM X_SASFCS_PTY_NO )
		      ; 
		   	   attrib UDF_VALUE length= $4000;
		   retain missing0-missing0 0; 
		   
		   etls_assign_target_value = 0; 
		   /* Build hash objects from lookup tables before reading first source row  */ 
		   if (_n_ = 1) then 
		   do; 
		      /* Build hash h0 from lookup table etls_temp_lookupview0 */ 
		      nlobs = .; 
		      dsid = open("etls_temp_lookupview0"); 
		      if (dsid > 0) then 
		      do; 
		         if ( attrc(dsid, 'MTYPE') = 'DATA' ) then 
		            nlobs = attrn(dsid, 'NLOBS'); 
		         else 
		            nlobs = -1; 
		            
		         dsid = close(dsid); 
		         if (nlobs ^= 0) then 
		         do; 
		            if (nlobs > 0) then 
		               exponent = ceil(log2(nlobs)); 
		            else 
		               exponent = 8; 
		            declare hash h0(dataset: "etls_temp_lookupview0", hashexp: exponent);
		            h0.defineKey( "INCIDENT_RK");
		            h0.defineData( "UDF_VALUE");
		            h0.defineDone();
		            if (nlobs = -1) then 
		            do; 
		               if (h0.Num_Items < 1) then
		               do; 
		                  put "NOTE: Lookup table is empty: etls_temp_lookupview0";
		                  missing0 = 98; 
		               end; 
		            end; 
		         end; 
		         else 
		         do; 
		            put "NOTE: Lookup table is empty: etls_temp_lookupview0";
		            missing0 = 98; 
		         end; 
		      end; 
		      else 
		      do; 
		         put "NOTE: Lookup table does not exist or cannot be opened:"
		              " etls_temp_lookupview0";
		         
		         put "NOTE: Abort action indicated, condition= Lookup table missing:"
		              " etls_temp_lookupview0";
		         
		         abort 3; 
		      end; 
		      
		      call missing (INCIDENT_RK, UDF_VALUE); 
		   
		   end; /* All hash objects have been defined */
		   
		   /* Read a row from the source table  */ 
		   set work.alt_pty_nm end = eof;
		   
		   /* Is the current key value stored in hash h0?  */ 
		   if (missing0 < 1) then 
		   do; 
		      rc0 = h0.find(key: INCIDENT_RK);
		      X_SASFCS_PTY_NO = UDF_VALUE;
		   end; 
		   else 
		      rc0 = missing0; 
		      
		   /* Examine success of lookups  */ 
		   if ( rc0=0 ) then 
		   do; 
		      /* Write row to target  */ 
		      output work.alt_pty_no; 
		   end; 
		   else 
		   do; 
		      error_total + 1; 
		      
		      if (rc0 ^= 0) then 
		      do; 
		         exception_total + 1; 
		         if (rc0 = 98) then 
		         do; 
		            /* Check: Lookup table has no rows-Write exception row  */ 
		            /* Check: Lookup table has no rows-Set target columns to value  */ 
		            X_SASFCS_PTY_NO = '';
		            etls_assign_target_value = 1; 
		         end; 
		         
		         else 
		         do; 
		            /* Check: Lookup value not found-Set target columns to value  */ 
		            X_SASFCS_PTY_NO = '';
		            etls_assign_target_value = 1; 
		         end; 
		      end; 
		      
		      /* Set target columns to value/missing requested?  */ 
		      if (etls_assign_target_value eq 1) then 
		         /* Write row to target  */ 
		         output work.alt_pty_no; 
		   
		   end; /* One or more lookups failed */
		   
		   if (eof = 1) then 
		   do; 
		      put "Source records with errors: " error_total ; 
		      put "Total lookup exceptions: " exception_total ; 
		   end; 

		run; 

	    %fcf_rcset(&syserr);

	    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
	        %let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_FB_MAN_ALERTS_RISKS;

	        %fcf_put_error;
	        %goto MACRO_END;
	    %end;

		/*==========================================================================* 
		 * Step:            Lookup                                                  *
		 * Transform:       Lookup                                                  * 
		 * Description:                                                             * 
		 *                                                                          * 
		 * Source Tables:   Lookup - work.alt_pty_no                                *
		 *                  Join - work.all_alt_char_udfs                           *
		 * Target Table:    Lookup - work.alt_manreason                             *
		 *==========================================================================*/ 

		proc datasets lib = work nolist nowarn memtype = (data view);
		   delete etls_temp_lookupview0;
		quit;

		proc sql; create view etls_temp_lookupview0 as 
		   select * from work.all_alt_char_udfs
		   where UDF_NM = 'X_MANALT_REASON';
		quit;
		proc datasets lib = work nolist nowarn memtype = (data view);
		   delete alt_manreason;
		quit;

		/*---- Begin DATA step to perform lookups  ----*/ 
		DATA  work.alt_manreason
		         (keep = INCIDENT_RK VALID_FROM_DTTM VALID_TO_DTTM CASE_RK INCIDENT_ID 
		                 SOURCE_SYSTEM_CD INCIDENT_TYPE_CD INCIDENT_CATEGORY_CD 
		                 INCIDENT_SUBCATEGORY_CD INCIDENT_DESC INCIDENT_FROM_DT 
		                 INCIDENT_FROM_TM INCIDENT_TO_DT INCIDENT_TO_TM DETECTION_DT 
		                 DETECTION_TM NOTIFICATION_DT NOTIFICATION_TM UI_DEF_FILE_NM 
		                 CREATE_USER_ID CREATE_DTTM UPDATE_USER_ID VERSION_NO DELETE_FLG 
		                 INVESTIGATOR_USER_ID INCIDENT_DISPOSITION_CD CLOSE_DTTM 
		                 INCIDENT_STATUS_CD PRIMARY_ENTITY_LEVEL_CODE X_SASFCS_PTY_KEY
						 X_SASFCS_ML_RS X_ALT_RUN_DATE ALT_SUPPRESSION_END_DATE
						 X_SASFCS_PTY_NM X_SASFCS_PTY_NO X_MANALT_REASON )
		      ; 
		   	   attrib UDF_VALUE length= $4000;
		   retain missing0-missing0 0; 
		   
		   etls_assign_target_value = 0; 
		   /* Build hash objects from lookup tables before reading first source row  */ 
		   if (_n_ = 1) then 
		   do; 
		      /* Build hash h0 from lookup table etls_temp_lookupview0 */ 
		      nlobs = .; 
		      dsid = open("etls_temp_lookupview0"); 
		      if (dsid > 0) then 
		      do; 
		         if ( attrc(dsid, 'MTYPE') = 'DATA' ) then 
		            nlobs = attrn(dsid, 'NLOBS'); 
		         else 
		            nlobs = -1; 
		            
		         dsid = close(dsid); 
		         if (nlobs ^= 0) then 
		         do; 
		            if (nlobs > 0) then 
		               exponent = ceil(log2(nlobs)); 
		            else 
		               exponent = 8; 
		            declare hash h0(dataset: "etls_temp_lookupview0", hashexp: exponent);
		            h0.defineKey( "INCIDENT_RK");
		            h0.defineData( "UDF_VALUE");
		            h0.defineDone();
		            if (nlobs = -1) then 
		            do; 
		               if (h0.Num_Items < 1) then
		               do; 
		                  put "NOTE: Lookup table is empty: etls_temp_lookupview0";
		                  missing0 = 98; 
		               end; 
		            end; 
		         end; 
		         else 
		         do; 
		            put "NOTE: Lookup table is empty: etls_temp_lookupview0";
		            missing0 = 98; 
		         end; 
		      end; 
		      else 
		      do; 
		         put "NOTE: Lookup table does not exist or cannot be opened:"
		              " etls_temp_lookupview0";
		         
		         put "NOTE: Abort action indicated, condition= Lookup table missing:"
		              " etls_temp_lookupview0";
		         
		         abort 3; 
		      end; 
		      
		      call missing (INCIDENT_RK, UDF_VALUE); 
		   
		   end; /* All hash objects have been defined */
		   
		   /* Read a row from the source table  */ 
		   set work.alt_pty_no end = eof;
		   
		   /* Is the current key value stored in hash h0?  */ 
		   if (missing0 < 1) then 
		   do; 
		      rc0 = h0.find(key: INCIDENT_RK);
		      X_MANALT_REASON = UDF_VALUE;
		   end; 
		   else 
		      rc0 = missing0; 
		      
		   /* Examine success of lookups  */ 
		   if ( rc0=0 ) then 
		   do; 
		      /* Write row to target  */ 
		      output work.alt_manreason; 
		   end; 
		   else 
		   do; 
		      error_total + 1; 
		      
		      if (rc0 ^= 0) then 
		      do; 
		         exception_total + 1; 
		         if (rc0 = 98) then 
		         do; 
		            /* Check: Lookup table has no rows-Write exception row  */ 
		            /* Check: Lookup table has no rows-Set target columns to value  */ 
		            X_MANALT_REASON = '';
		            etls_assign_target_value = 1; 
		         end; 
		         
		         else 
		         do; 
		            /* Check: Lookup value not found-Set target columns to value  */ 
		            X_MANALT_REASON = '';
		            etls_assign_target_value = 1; 
		         end; 
		      end; 
		      
		      /* Set target columns to value/missing requested?  */ 
		      if (etls_assign_target_value eq 1) then 
		         /* Write row to target  */ 
		         output work.alt_manreason; 
		   
		   end; /* One or more lookups failed */
		   
		   if (eof = 1) then 
		   do; 
		      put "Source records with errors: " error_total ; 
		      put "Total lookup exceptions: " exception_total ; 
		   end; 

		run; 

	    %fcf_rcset(&syserr);

	    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
	        %let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_FB_MAN_ALERTS_RISKS;

	        %fcf_put_error;
	        %goto MACRO_END;
	    %end;

		/*==========================================================================* 
		 * Step:            Lookup                                                  *
		 * Transform:       Lookup                                                  * 
		 * Description:                                                             * 
		 *                                                                          * 
		 * Source Tables:   Lookup - work.alt_manreason                             *
		 *                  Join - work.all_alt_char_udfs                           *
		 * Target Table:    Lookup - work.alt_reason                                *
		 *==========================================================================*/ 

		proc datasets lib = work nolist nowarn memtype = (data view);
		   delete etls_temp_lookupview0;
		quit;

		proc sql; create view etls_temp_lookupview0 as 
		   select * from work.all_alt_char_udfs
		   where UDF_NM = 'X_REASON';
		quit;
		proc datasets lib = work nolist nowarn memtype = (data view);
		   delete alt_reason;
		quit;

		/*---- Begin DATA step to perform lookups  ----*/ 
		DATA  work.alt_reason
		         (keep = INCIDENT_RK VALID_FROM_DTTM VALID_TO_DTTM CASE_RK INCIDENT_ID 
		                 SOURCE_SYSTEM_CD INCIDENT_TYPE_CD INCIDENT_CATEGORY_CD 
		                 INCIDENT_SUBCATEGORY_CD INCIDENT_DESC INCIDENT_FROM_DT 
		                 INCIDENT_FROM_TM INCIDENT_TO_DT INCIDENT_TO_TM DETECTION_DT 
		                 DETECTION_TM NOTIFICATION_DT NOTIFICATION_TM UI_DEF_FILE_NM 
		                 CREATE_USER_ID CREATE_DTTM UPDATE_USER_ID VERSION_NO DELETE_FLG 
		                 INVESTIGATOR_USER_ID INCIDENT_DISPOSITION_CD CLOSE_DTTM 
		                 INCIDENT_STATUS_CD PRIMARY_ENTITY_LEVEL_CODE X_SASFCS_PTY_KEY
						 X_SASFCS_ML_RS X_ALT_RUN_DATE ALT_SUPPRESSION_END_DATE
  						 X_SASFCS_PTY_NM X_SASFCS_PTY_NO X_MANALT_REASON X_REASON )
		      ; 
		   attrib UDF_VALUE length= $4000;	   
		   retain missing0-missing0 0; 
		   
		   etls_assign_target_value = 0; 
		   /* Build hash objects from lookup tables before reading first source row  */ 
		   if (_n_ = 1) then 
		   do; 
		      /* Build hash h0 from lookup table etls_temp_lookupview0 */ 
		      nlobs = .; 
		      dsid = open("etls_temp_lookupview0"); 
		      if (dsid > 0) then 
		      do; 
		         if ( attrc(dsid, 'MTYPE') = 'DATA' ) then 
		            nlobs = attrn(dsid, 'NLOBS'); 
		         else 
		            nlobs = -1; 
		            
		         dsid = close(dsid); 
		         if (nlobs ^= 0) then 
		         do; 
		            if (nlobs > 0) then 
		               exponent = ceil(log2(nlobs)); 
		            else 
		               exponent = 8; 
		            declare hash h0(dataset: "etls_temp_lookupview0", hashexp: exponent);
		            h0.defineKey( "INCIDENT_RK");
		            h0.defineData( "UDF_VALUE");
		            h0.defineDone();
		            if (nlobs = -1) then 
		            do; 
		               if (h0.Num_Items < 1) then
		               do; 
		                  put "NOTE: Lookup table is empty: etls_temp_lookupview0";
		                  missing0 = 98; 
		               end; 
		            end; 
		         end; 
		         else 
		         do; 
		            put "NOTE: Lookup table is empty: etls_temp_lookupview0";
		            missing0 = 98; 
		         end; 
		      end; 
		      else 
		      do; 
		         put "NOTE: Lookup table does not exist or cannot be opened:"
		              " etls_temp_lookupview0";
		         
		         put "NOTE: Abort action indicated, condition= Lookup table missing:"
		              " etls_temp_lookupview0";
		         
		         abort 3; 
		      end; 
		      
		      call missing (INCIDENT_RK, UDF_VALUE); 
		   
		   end; /* All hash objects have been defined */
		   
		   /* Read a row from the source table  */ 
		   set work.alt_manreason end = eof;
		   
		   /* Is the current key value stored in hash h0?  */ 
		   if (missing0 < 1) then 
		   do; 
		      rc0 = h0.find(key: INCIDENT_RK);
		      X_REASON = UDF_VALUE;
		   end; 
		   else 
		      rc0 = missing0; 
		      
		   /* Examine success of lookups  */ 
		   if ( rc0=0 ) then 
		   do; 
		      /* Write row to target  */ 
		      output work.alt_reason; 
		   end; 
		   else 
		   do; 
		      error_total + 1; 
		      
		      if (rc0 ^= 0) then 
		      do; 
		         exception_total + 1; 
		         if (rc0 = 98) then 
		         do; 
		            /* Check: Lookup table has no rows-Write exception row  */ 
		            /* Check: Lookup table has no rows-Set target columns to value  */ 
		            X_REASON = '';
		            etls_assign_target_value = 1; 
		         end; 
		         
		         else 
		         do; 
		            /* Check: Lookup value not found-Set target columns to value  */ 
		            X_REASON = '';
		            etls_assign_target_value = 1; 
		         end; 
		      end; 
		      
		      /* Set target columns to value/missing requested?  */ 
		      if (etls_assign_target_value eq 1) then 
		         /* Write row to target  */ 
		         output work.alt_reason; 
		   
		   end; /* One or more lookups failed */
		   
		   if (eof = 1) then 
		   do; 
		      put "Source records with errors: " error_total ; 
		      put "Total lookup exceptions: " exception_total ; 
		   end; 

		run; 

	    %fcf_rcset(&syserr);

	    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
	        %let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_FB_MAN_ALERTS_RISKS;

	        %fcf_put_error;
	        %goto MACRO_END;
	    %end;

		/*==========================================================================* 
		 * Step:            Table Loader                                            *
		 * Transform:       Table Loader (Version 2.1)                              * 
		 * Description:                                                             * 
		 *                                                                          * 
		 * Source Table:    Lookup - work.alt_reason                                *
		 * Target Table:    FSK_ALERT - &db_kc_libname..FSK_ALERT                   *
		 *                                                                          * 
		 *==========================================================================*/ 

		%let SOURCEDS = %nrquote(work.alt_reason); 

		proc datasets lib = work nolist nowarn memtype = (data view);
		  delete temp_map;
		quit;

		data temp_map;
			if 0 then set &db_kc_libname..FSK_ALERT;
			set &SOURCEDS. (keep= INCIDENT_ID SOURCE_SYSTEM_CD INCIDENT_DISPOSITION_CD
			INCIDENT_DESC PRIMARY_ENTITY_LEVEL_CODE X_SASFCS_ML_RS X_MANALT_REASON
			X_SASFCS_PTY_KEY X_SASFCS_PTY_NO X_SASFCS_PTY_NM X_REASON X_ALT_RUN_DATE
			CREATE_DTTM INVESTIGATOR_USER_ID INCIDENT_TYPE_CD INCIDENT_CATEGORY_CD
			INCIDENT_SUBCATEGORY_CD );
			ALERT_ID = INCIDENT_ID;
			SEGMENT_ID = SOURCE_SYSTEM_CD;
			PRODUCT_TYPE = "AML";
			ALERT_STATUS_CODE = INCIDENT_DISPOSITION_CD;
			MONEY_LAUNDERING_RISK_SCORE = X_SASFCS_ML_RS;
			ALERT_DESCRIPTION = X_MANALT_REASON;
			PRIMARY_ENTITY_LEVEL_CODE = PRIMARY_ENTITY_LEVEL_CODE;
			PRIMARY_ENTITY_NUMBER = X_SASFCS_PTY_NO;
			PRIMARY_ENTITY_KEY = X_SASFCS_PTY_KEY;
			PRIMARY_ENTITY_NAME = X_SASFCS_PTY_NM;
			SCENARIO_ID = -1;
			SCENARIO_NAME = "scen0001";
			ACTUAL_VALUES_TEXT = INCIDENT_DESC;
			RUN_DATE = X_ALT_RUN_DATE;
			CREATE_DATE = CREATE_DTTM;
			CREATE_USER_ID = INVESTIGATOR_USER_ID;
			VERSION_NUMBER = 1;
			LOGICAL_DELETE_IND = "N";
			ALERT_TYPE_CD = INCIDENT_TYPE_CD;
			ALERT_CATEGORY_CD = INCIDENT_CATEGORY_CD;
			ALERT_SUBCATEGORY_CD = INCIDENT_SUBCATEGORY_CD;
			OWNER_USER_LONG_ID = INVESTIGATOR_USER_ID;
			drop INCIDENT_ID SOURCE_SYSTEM_CD INCIDENT_DISPOSITION_CD INCIDENT_DESC
			X_SASFCS_ML_RS X_MANALT_REASON X_SASFCS_PTY_KEY X_SASFCS_PTY_NM X_SASFCS_PTY_NO
			X_REASON X_ALT_RUN_DATE CREATE_DTTM INVESTIGATOR_USER_ID INCIDENT_TYPE_CD
			INCIDENT_CATEGORY_CD INCIDENT_SUBCATEGORY_CD ;
		run;

	   %let SOURCEDS = work.temp_map;

		proc append base = &db_kc_libname..FSK_ALERT 
		  data = &SOURCEDS force ; 
		run; 

	    %fcf_rcset(&sqlrc);

	    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
	        %let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_FB_MAN_ALERTS_RISKS;

	        %fcf_put_error;
	        %goto MACRO_END;
	    %end;
		  
		proc datasets lib = work nolist nowarn memtype = (data view);
		  delete temp_map;
		quit;
		   

		/*==========================================================================* 
		 * Step:            Table Loader                                            *
		 * Transform:       Table Loader (Version 2.1)                              * 
		 * Description:                                                             * 
		 *                                                                          * 
		 * Source Table:    Lookup - work.alt_reason                                *
		 * Target Table:    New Alert Events -                                      *
		 *                   iem.new_alert_events                                   * 
		 *==========================================================================*/ 

		%let SOURCEDS = %nrquote(work.alt_reason); 

		proc datasets lib = iem nolist nowarn memtype = (data view);
		  delete new_alert_events;
		quit;
		
		data iem.new_alert_events;
			if 0 then set &db_kc_libname..FSK_ALERT_EVENT;
			set &SOURCEDS. (keep=SOURCE_SYSTEM_CD INCIDENT_ID X_MANALT_REASON CREATE_DTTM INVESTIGATOR_USER_ID);
			SEGMENT_ID = SOURCE_SYSTEM_CD;
			ALERT_ID = INCIDENT_ID;
			EVENT_TYPE_CODE = "CRE";
			EVENT_DESCRIPTION = X_MANALT_REASON;
			CREATE_DATE = CREATE_DTTM;
			CREATE_USER_ID = INVESTIGATOR_USER_ID;
			drop SOURCE_SYSTEM_CD INCIDENT_ID X_MANALT_REASON CREATE_DTTM INVESTIGATOR_USER_ID;
		run;

		%let obs_count = %fcf_get_observation_count(table=iem.new_alert_events, SASDATASET=Y);
		%let numevents = &obs_count;

		%fcf_get_sequence_ids(numIDs=&numevents);

		data iem.new_alert_events;
			set iem.new_alert_events;
			event_id = symgetn('seqId'||(strip(put(_n_,best32.))));
		run;

	    %fcf_rcset(&syserr);

	    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
	        %let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_FB_MAN_ALERTS_RISKS;

	        %fcf_put_error;
	        %goto MACRO_END;
	    %end;

		/*==========================================================================* 
		 * Step:            Table Loader                                            *
		 * Transform:       Table Loader (Version 2.1)                              * 
		 * Description:                                                             * 
		 *                                                                          * 
		 * Source Table:    New Alert Events -                                      *
		 *                   iem.new_alert_events                                   * 
		 * Target Table:    FSK_ALERT_EVENT -                                       *
		 *                   &db_kc_libname..FSK_ALERT_EVENT                        * 
		 *==========================================================================*/ 

		%let SOURCEDS = %nrquote(iem.new_alert_events); 

		proc append base = &db_kc_libname..FSK_ALERT_EVENT 
		  data = &SOURCEDS force ; 
		run; 

	    %fcf_rcset(&syserr);

	    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
	        %let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_FB_MAN_ALERTS_RISKS;

	        %fcf_put_error;
	        %goto MACRO_END;
	    %end;

	%end;

	/*==========================================================================* 
	 * Step:            Lookup                                                  *
	 * Transform:       Lookup                                                  * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   Error Table - work.new_risks                            *
	 *                  Join - work.all_rsk_num_udfs                            *
	 * Target Table:    Lookup - work.rsk_prp_clssf                             *
	 *==========================================================================*/ 

	%let SOURCEDS=work.new_risks;

    %let obs_count = %fcf_get_observation_count(table=&SOURCEDS,SASDATASET=Y);

    %if &obs_count>0 %then %do;

		proc datasets lib = work nolist nowarn memtype = (data view);
		   delete etls_temp_lookupview0;
		quit;

		proc sql; create view etls_temp_lookupview0 as 
		   select * from work.all_rsk_num_udfs
		   where UDF_NM = 'X_SASFCS_PRP_RSK_CLSSF';
		quit;
		proc datasets lib = work nolist nowarn memtype = (data view);
		   delete rsk_prp_clssf;
		quit;

		DATA  work.rsk_prp_clssf
		         (keep = INCIDENT_RK VALID_FROM_DTTM VALID_TO_DTTM CASE_RK INCIDENT_ID 
		                 SOURCE_SYSTEM_CD INCIDENT_TYPE_CD INCIDENT_CATEGORY_CD 
		                 INCIDENT_SUBCATEGORY_CD INCIDENT_DESC INCIDENT_FROM_DT 
		                 INCIDENT_FROM_TM INCIDENT_TO_DT INCIDENT_TO_TM DETECTION_DT 
		                 DETECTION_TM NOTIFICATION_DT NOTIFICATION_TM UI_DEF_FILE_NM 
		                 CREATE_USER_ID CREATE_DTTM UPDATE_USER_ID VERSION_NO DELETE_FLG 
		                 INVESTIGATOR_USER_ID INCIDENT_DISPOSITION_CD CLOSE_DTTM 
		                 INCIDENT_STATUS_CD X_SASFCS_PRP_RSK_CLSSF )
		      ; 
		   
		   retain missing0-missing0 0; 
		   
		   etls_assign_target_value = 0; 
		   /* Build hash objects from lookup tables before reading first source row  */ 
		   if (_n_ = 1) then 
		   do; 
		      /* Build hash h0 from lookup table etls_temp_lookupview0 */ 
		      nlobs = .; 
		      dsid = open("etls_temp_lookupview0"); 
		      if (dsid > 0) then 
		      do; 
		         if ( attrc(dsid, 'MTYPE') = 'DATA' ) then 
		            nlobs = attrn(dsid, 'NLOBS'); 
		         else 
		            nlobs = -1; 
		            
		         dsid = close(dsid); 
		         if (nlobs ^= 0) then 
		         do; 
		            if (nlobs > 0) then 
		               exponent = ceil(log2(nlobs)); 
		            else 
		               exponent = 8; 
		            declare hash h0(dataset: "etls_temp_lookupview0", hashexp: exponent);
		            h0.defineKey( "INCIDENT_RK");
		            h0.defineData( "UDF_VALUE");
		            h0.defineDone();
		            if (nlobs = -1) then 
		            do; 
		               if (h0.Num_Items < 1) then
		               do; 
		                  put "NOTE: Lookup table is empty: etls_temp_lookupview0";
		                  missing0 = 98; 
		               end; 
		            end; 
		         end; 
		         else 
		         do; 
		            put "NOTE: Lookup table is empty: etls_temp_lookupview0";
		            missing0 = 98; 
		         end; 
		      end; 
		      else 
		      do; 
		         put "NOTE: Lookup table does not exist or cannot be opened:"
		              " etls_temp_lookupview0";
		         
		         put "NOTE: Abort action indicated, condition= Lookup table missing:"
		              " etls_temp_lookupview0";
		         
		         abort 3; 
		      end; 
		      
		      call missing (INCIDENT_RK, UDF_VALUE); 
		   
		   end; /* All hash objects have been defined */
		   
		   /* Read a row from the source table  */ 
		   set work.new_risks end = eof;
		   
		   /* Is the current key value stored in hash h0?  */ 
		   if (missing0 < 1) then 
		   do; 
		      rc0 = h0.find(key: INCIDENT_RK);
		      X_SASFCS_PRP_RSK_CLSSF = UDF_VALUE;
		   end; 
		   else 
		      rc0 = missing0; 
		      
		   /* Examine success of lookups  */ 
		   if ( rc0=0 ) then 
		   do; 
		      /* Write row to target  */ 
		      output work.rsk_prp_clssf; 
		   end; 
		   else 
		   do; 
		      error_total + 1; 
		      
		      if (rc0 ^= 0) then 
		      do; 
		         exception_total + 1; 
		         if (rc0 = 98) then 
		         do; 
		            /* Check: Lookup table has no rows-Write exception row  */ 
		            /* Check: Lookup table has no rows-Set target columns to value  */ 
		            X_SASFCS_PRP_RSK_CLSSF = .;
		            etls_assign_target_value = 1; 
		         end; 
		         
		         else 
		         do; 
		            /* Check: Lookup value not found-Set target columns to value  */ 
		            X_SASFCS_PRP_RSK_CLSSF = .;
		            etls_assign_target_value = 1; 
		         end; 
		      end; 
		      
		      /* Set target columns to value/missing requested?  */ 
		      if (etls_assign_target_value eq 1) then 
		         /* Write row to target  */ 
		         output work.rsk_prp_clssf; 
		   
		   end; /* One or more lookups failed */
		   
		   if (eof = 1) then 
		   do; 
		      put "Source records with errors: " error_total ; 
		      put "Total lookup exceptions: " exception_total ; 
		   end; 

		run; 

	    %fcf_rcset(&syserr);

	    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
	        %let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_FB_MAN_ALERTS_RISKS;

	        %fcf_put_error;
	        %goto MACRO_END;
	    %end;

		/*==========================================================================* 
		 * Step:            Lookup                                                  *
		 * Transform:       Lookup                                                  * 
		 * Description:                                                             * 
		 *                                                                          * 
		 * Source Tables:   Lookup - work.rsk_prp_clssf                             *
		 *                  Join - work.all_rsk_num_udfs                            *
		 * Target Table:    Lookup - work.rsk_clssf                                 *
		 *==========================================================================*/ 

		proc datasets lib = work nolist nowarn memtype = (data view);
		   delete etls_temp_lookupview0;
		quit;

		proc sql; create view etls_temp_lookupview0 as 
		   select * from work.all_rsk_num_udfs
		   where UDF_NM = 'X_SASFCS_RSK_CLSSF';
		quit;
		proc datasets lib = work nolist nowarn memtype = (data view);
		   delete rsk_clssf;
		quit;

		/*---- Begin DATA step to perform lookups  ----*/ 
		DATA  work.rsk_clssf
		         (keep = INCIDENT_RK VALID_FROM_DTTM VALID_TO_DTTM CASE_RK INCIDENT_ID 
		                 SOURCE_SYSTEM_CD INCIDENT_TYPE_CD INCIDENT_CATEGORY_CD 
		                 INCIDENT_SUBCATEGORY_CD INCIDENT_DESC INCIDENT_FROM_DT 
		                 INCIDENT_FROM_TM INCIDENT_TO_DT INCIDENT_TO_TM DETECTION_DT 
		                 DETECTION_TM NOTIFICATION_DT NOTIFICATION_TM UI_DEF_FILE_NM 
		                 CREATE_USER_ID CREATE_DTTM UPDATE_USER_ID VERSION_NO DELETE_FLG 
		                 INVESTIGATOR_USER_ID INCIDENT_DISPOSITION_CD CLOSE_DTTM 
		                 INCIDENT_STATUS_CD X_SASFCS_PRP_RSK_CLSSF X_SASFCS_RSK_CLSSF )
		      ; 
		   	   
		   retain missing0-missing0 0; 
		   
		   etls_assign_target_value = 0; 
		   /* Build hash objects from lookup tables before reading first source row  */ 
		   if (_n_ = 1) then 
		   do; 
		      /* Build hash h0 from lookup table etls_temp_lookupview0 */ 
		      nlobs = .; 
		      dsid = open("etls_temp_lookupview0"); 
		      if (dsid > 0) then 
		      do; 
		         if ( attrc(dsid, 'MTYPE') = 'DATA' ) then 
		            nlobs = attrn(dsid, 'NLOBS'); 
		         else 
		            nlobs = -1; 
		            
		         dsid = close(dsid); 
		         if (nlobs ^= 0) then 
		         do; 
		            if (nlobs > 0) then 
		               exponent = ceil(log2(nlobs)); 
		            else 
		               exponent = 8; 
		            declare hash h0(dataset: "etls_temp_lookupview0", hashexp: exponent);
		            h0.defineKey( "INCIDENT_RK");
		            h0.defineData( "UDF_VALUE");
		            h0.defineDone();
		            if (nlobs = -1) then 
		            do; 
		               if (h0.Num_Items < 1) then
		               do; 
		                  put "NOTE: Lookup table is empty: etls_temp_lookupview0";
		                  missing0 = 98; 
		               end; 
		            end; 
		         end; 
		         else 
		         do; 
		            put "NOTE: Lookup table is empty: etls_temp_lookupview0";
		            missing0 = 98; 
		         end; 
		      end; 
		      else 
		      do; 
		         put "NOTE: Lookup table does not exist or cannot be opened:"
		              " etls_temp_lookupview0";
		         
		         put "NOTE: Abort action indicated, condition= Lookup table missing:"
		              " etls_temp_lookupview0";
		         
		         abort 3; 
		      end; 
		      
		      call missing (INCIDENT_RK, UDF_VALUE); 
		   
		   end; /* All hash objects have been defined */
		   
		   /* Read a row from the source table  */ 
		   set work.rsk_prp_clssf end = eof;
		   
		   /* Is the current key value stored in hash h0?  */ 
		   if (missing0 < 1) then 
		   do; 
		      rc0 = h0.find(key: INCIDENT_RK);
		      X_SASFCS_RSK_CLSSF = UDF_VALUE;
		   end; 
		   else 
		      rc0 = missing0; 
		      
		   /* Examine success of lookups  */ 
		   if ( rc0=0 ) then 
		   do; 
		      /* Write row to target  */ 
		      output work.rsk_clssf; 
		   end; 
		   else 
		   do; 
		      error_total + 1; 
		      
		      if (rc0 ^= 0) then 
		      do; 
		         exception_total + 1; 
		         if (rc0 = 98) then 
		         do; 
		            /* Check: Lookup table has no rows-Write exception row  */ 
		            /* Check: Lookup table has no rows-Set target columns to value  */ 
		            X_SASFCS_RSK_CLSSF = .;
		            etls_assign_target_value = 1; 
		         end; 
		         
		         else 
		         do; 
		            /* Check: Lookup value not found-Set target columns to value  */ 
		            X_SASFCS_RSK_CLSSF = .;
		            etls_assign_target_value = 1; 
		         end; 
		      end; 
		      
		      /* Set target columns to value/missing requested?  */ 
		      if (etls_assign_target_value eq 1) then 
		         /* Write row to target  */ 
		         output work.rsk_clssf; 
		   
		   end; /* One or more lookups failed */
		   
		   if (eof = 1) then 
		   do; 
		      put "Source records with errors: " error_total ; 
		      put "Total lookup exceptions: " exception_total ; 
		   end; 

		run; 

	    %fcf_rcset(&syserr);

	    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
	        %let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_FB_MAN_ALERTS_RISKS;

	        %fcf_put_error;
	        %goto MACRO_END;
	    %end;

		/*==========================================================================* 
		 * Step:            Lookup                                                  *
		 * Transform:       Lookup                                                  * 
		 * Description:                                                             * 
		 *                                                                          * 
		 * Source Tables:   Lookup - work.rsk_clssf                                 *
		 *                  Join - work.all_rsk_num_udfs                            *
		 * Target Table:    Lookup - work.rsk_pty_key                               *
		 *==========================================================================*/ 

		proc datasets lib = work nolist nowarn memtype = (data view);
		   delete etls_temp_lookupview0;
		quit;

		proc sql; create view etls_temp_lookupview0 as 
		   select * from work.all_rsk_num_udfs
		   where UDF_NM = 'X_SASFCS_PTY_KEY';
		quit;
		proc datasets lib = work nolist nowarn memtype = (data view);
		   delete rsk_pty_key;
		quit;

		/*---- Begin DATA step to perform lookups  ----*/ 
		DATA  work.rsk_pty_key
		         (keep = INCIDENT_RK VALID_FROM_DTTM VALID_TO_DTTM CASE_RK INCIDENT_ID 
		                 SOURCE_SYSTEM_CD INCIDENT_TYPE_CD INCIDENT_CATEGORY_CD 
		                 INCIDENT_SUBCATEGORY_CD INCIDENT_DESC INCIDENT_FROM_DT 
		                 INCIDENT_FROM_TM INCIDENT_TO_DT INCIDENT_TO_TM DETECTION_DT 
		                 DETECTION_TM NOTIFICATION_DT NOTIFICATION_TM UI_DEF_FILE_NM 
		                 CREATE_USER_ID CREATE_DTTM UPDATE_USER_ID VERSION_NO DELETE_FLG 
		                 INVESTIGATOR_USER_ID INCIDENT_DISPOSITION_CD CLOSE_DTTM 
		                 INCIDENT_STATUS_CD X_SASFCS_PRP_RSK_CLSSF X_SASFCS_RSK_CLSSF 
		                 X_SASFCS_PTY_KEY )
		      ;
	 
		   retain missing0-missing0 0; 
		   
		   etls_assign_target_value = 0; 
		   /* Build hash objects from lookup tables before reading first source row  */ 
		   if (_n_ = 1) then 
		   do; 
		      /* Build hash h0 from lookup table etls_temp_lookupview0 */ 
		      nlobs = .; 
		      dsid = open("etls_temp_lookupview0"); 
		      if (dsid > 0) then 
		      do; 
		         if ( attrc(dsid, 'MTYPE') = 'DATA' ) then 
		            nlobs = attrn(dsid, 'NLOBS'); 
		         else 
		            nlobs = -1; 
		            
		         dsid = close(dsid); 
		         if (nlobs ^= 0) then 
		         do; 
		            if (nlobs > 0) then 
		               exponent = ceil(log2(nlobs)); 
		            else 
		               exponent = 8; 
		            declare hash h0(dataset: "etls_temp_lookupview0", hashexp: exponent);
		            h0.defineKey( "INCIDENT_RK");
		            h0.defineData( "UDF_VALUE");
		            h0.defineDone();
		            if (nlobs = -1) then 
		            do; 
		               if (h0.Num_Items < 1) then
		               do; 
		                  put "NOTE: Lookup table is empty: etls_temp_lookupview0";
		                  missing0 = 98; 
		               end; 
		            end; 
		         end; 
		         else 
		         do; 
		            put "NOTE: Lookup table is empty: etls_temp_lookupview0";
		            missing0 = 98; 
		         end; 
		      end; 
		      else 
		      do; 
		         put "NOTE: Lookup table does not exist or cannot be opened:"
		              " etls_temp_lookupview0";
		         
		         put "NOTE: Abort action indicated, condition= Lookup table missing:"
		              " etls_temp_lookupview0";
		         
		         abort 3; 
		      end; 
		      
		      call missing (INCIDENT_RK, UDF_VALUE); 
		   
		   end; /* All hash objects have been defined */
		   
		   /* Read a row from the source table  */ 
		   set work.rsk_clssf end = eof;
		   
		   /* Is the current key value stored in hash h0?  */ 
		   if (missing0 < 1) then 
		   do; 
		      rc0 = h0.find(key: INCIDENT_RK);
		      X_SASFCS_PTY_KEY = UDF_VALUE;
		   end; 
		   else 
		      rc0 = missing0; 
		      
		   /* Examine success of lookups  */ 
		   if ( rc0=0 ) then 
		   do; 
		      /* Write row to target  */ 
		      output work.rsk_pty_key; 
		   end; 
		   else 
		   do; 
		      error_total + 1; 
		      
		      if (rc0 ^= 0) then 
		      do; 
		         exception_total + 1; 
		         if (rc0 = 98) then 
		         do; 
		            /* Check: Lookup table has no rows-Write exception row  */ 
		            /* Check: Lookup table has no rows-Set target columns to value  */ 
		            X_SASFCS_PTY_KEY = .;
		            etls_assign_target_value = 1; 
		         end; 
		         
		         else 
		         do; 
		            /* Check: Lookup value not found-Set target columns to value  */ 
		            X_SASFCS_PTY_KEY = .;
		            etls_assign_target_value = 1; 
		         end; 
		      end; 
		      
		      /* Set target columns to value/missing requested?  */ 
		      if (etls_assign_target_value eq 1) then 
		         /* Write row to target  */ 
		         output work.rsk_pty_key; 
		   
		   end; /* One or more lookups failed */
		   
		   if (eof = 1) then 
		   do; 
		      put "Source records with errors: " error_total ; 
		      put "Total lookup exceptions: " exception_total ; 
		   end; 

		run; 

	    %fcf_rcset(&syserr);

	    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
	        %let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_FB_MAN_ALERTS_RISKS;

	        %fcf_put_error;
	        %goto MACRO_END;
	    %end;

		/*==========================================================================* 
		 * Step:            Lookup                                                  *
		 * Transform:       Lookup                                                  * 
		 * Description:                                                             * 
		 *                                                                          * 
		 * Source Tables:   Lookup - work.rsk_pty_key                               *
		 *                  Join - work.all_rsk_char_udfs                           *
		 * Target Table:    Lookup - work.rsk_manreason                             *
		 *==========================================================================*/ 

		proc datasets lib = work nolist nowarn memtype = (data view);
		   delete etls_temp_lookupview0;
		quit;

		proc sql; create view etls_temp_lookupview0 as 
		   select * from work.all_rsk_char_udfs
		   where UDF_NM = 'X_MANRISK_REASON';
		quit;
		proc datasets lib = work nolist nowarn memtype = (data view);
		   delete rsk_manreason;
		quit;

		/*---- Begin DATA step to perform lookups  ----*/ 
		DATA  work.rsk_manreason
		         (keep = INCIDENT_RK VALID_FROM_DTTM VALID_TO_DTTM CASE_RK INCIDENT_ID 
		                 SOURCE_SYSTEM_CD INCIDENT_TYPE_CD INCIDENT_CATEGORY_CD 
		                 INCIDENT_SUBCATEGORY_CD INCIDENT_DESC INCIDENT_FROM_DT 
		                 INCIDENT_FROM_TM INCIDENT_TO_DT INCIDENT_TO_TM DETECTION_DT 
		                 DETECTION_TM NOTIFICATION_DT NOTIFICATION_TM UI_DEF_FILE_NM 
		                 CREATE_USER_ID CREATE_DTTM UPDATE_USER_ID VERSION_NO DELETE_FLG 
		                 INVESTIGATOR_USER_ID INCIDENT_DISPOSITION_CD CLOSE_DTTM 
		                 INCIDENT_STATUS_CD X_SASFCS_PRP_RSK_CLSSF X_SASFCS_RSK_CLSSF 
		                 X_SASFCS_PTY_KEY X_MANRISK_REASON )
		      ; 
	       attrib UDF_VALUE length= $4000;	
	 
		   retain missing0-missing0 0; 
		   
		   etls_assign_target_value = 0; 
		   /* Build hash objects from lookup tables before reading first source row  */ 
		   if (_n_ = 1) then 
		   do; 
		      /* Build hash h0 from lookup table etls_temp_lookupview0 */ 
		      nlobs = .; 
		      dsid = open("etls_temp_lookupview0"); 
		      if (dsid > 0) then 
		      do; 
		         if ( attrc(dsid, 'MTYPE') = 'DATA' ) then 
		            nlobs = attrn(dsid, 'NLOBS'); 
		         else 
		            nlobs = -1; 
		            
		         dsid = close(dsid); 
		         if (nlobs ^= 0) then 
		         do; 
		            if (nlobs > 0) then 
		               exponent = ceil(log2(nlobs)); 
		            else 
		               exponent = 8; 
		            declare hash h0(dataset: "etls_temp_lookupview0", hashexp: exponent);
		            h0.defineKey( "INCIDENT_RK");
		            h0.defineData( "UDF_VALUE");
		            h0.defineDone();
		            if (nlobs = -1) then 
		            do; 
		               if (h0.Num_Items < 1) then
		               do; 
		                  put "NOTE: Lookup table is empty: etls_temp_lookupview0";
		                  missing0 = 98; 
		               end; 
		            end; 
		         end; 
		         else 
		         do; 
		            put "NOTE: Lookup table is empty: etls_temp_lookupview0";
		            missing0 = 98; 
		         end; 
		      end; 
		      else 
		      do; 
		         put "NOTE: Lookup table does not exist or cannot be opened:"
		              " etls_temp_lookupview0";
		         
		         put "NOTE: Abort action indicated, condition= Lookup table missing:"
		              " etls_temp_lookupview0";
		         
		         abort 3; 
		      end; 
		      
		      call missing (INCIDENT_RK, UDF_VALUE); 
		   
		   end; /* All hash objects have been defined */
		   
		   /* Read a row from the source table  */ 
		   set work.rsk_pty_key end = eof;
		   
		   /* Is the current key value stored in hash h0?  */ 
		   if (missing0 < 1) then 
		   do; 
		      rc0 = h0.find(key: INCIDENT_RK);
		      X_MANRISK_REASON = UDF_VALUE;
		   end; 
		   else 
		      rc0 = missing0; 
		      
		   /* Examine success of lookups  */ 
		   if ( rc0=0 ) then 
		   do; 
		      /* Write row to target  */ 
		      output work.rsk_manreason; 
		   end; 
		   else 
		   do; 
		      error_total + 1; 
		      
		      if (rc0 ^= 0) then 
		      do; 
		         exception_total + 1; 
		         if (rc0 = 98) then 
		         do; 
		            /* Check: Lookup table has no rows-Write exception row  */ 
		            /* Check: Lookup table has no rows-Set target columns to value  */ 
		            X_MANRISK_REASON = '';
		            etls_assign_target_value = 1; 
		         end; 
		         
		         else 
		         do; 
		            /* Check: Lookup value not found-Set target columns to value  */ 
		            X_MANRISK_REASON = '';
		            etls_assign_target_value = 1; 
		         end; 
		      end; 
		      
		      /* Set target columns to value/missing requested?  */ 
		      if (etls_assign_target_value eq 1) then 
		         /* Write row to target  */ 
		         output work.rsk_manreason; 
		   
		   end; /* One or more lookups failed */
		   
		   if (eof = 1) then 
		   do; 
		      put "Source records with errors: " error_total ; 
		      put "Total lookup exceptions: " exception_total ; 
		   end; 

		run; 

	    %fcf_rcset(&syserr);

	    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
	        %let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_FB_MAN_ALERTS_RISKS;

	        %fcf_put_error;
	        %goto MACRO_END;
	    %end;

		/*==========================================================================* 
		 * Step:            Lookup                                                  *
		 * Transform:       Lookup                                                  * 
		 * Description:                                                             * 
		 *                                                                          * 
		 * Source Tables:   Lookup - work.rsk_manreason                             *
		 *                  Join - work.all_rsk_char_udfs                           *
		 * Target Table:    Lookup - work.rsk_pty_nm                                *
		 *==========================================================================*/ 

		proc datasets lib = work nolist nowarn memtype = (data view);
		   delete etls_temp_lookupview0;
		quit;

		proc sql; create view etls_temp_lookupview0 as 
		   select * from work.all_rsk_char_udfs
		   where UDF_NM = 'X_SASFCS_PTY_NM';
		quit;
		proc datasets lib = work nolist nowarn memtype = (data view);
		   delete rsk_pty_nm;
		quit;

		/*---- Begin DATA step to perform lookups  ----*/ 
		DATA  work.rsk_pty_nm
		         (keep = INCIDENT_RK VALID_FROM_DTTM VALID_TO_DTTM CASE_RK INCIDENT_ID 
		                 SOURCE_SYSTEM_CD INCIDENT_TYPE_CD INCIDENT_CATEGORY_CD 
		                 INCIDENT_SUBCATEGORY_CD INCIDENT_DESC INCIDENT_FROM_DT 
		                 INCIDENT_FROM_TM INCIDENT_TO_DT INCIDENT_TO_TM DETECTION_DT 
		                 DETECTION_TM NOTIFICATION_DT NOTIFICATION_TM UI_DEF_FILE_NM 
		                 CREATE_USER_ID CREATE_DTTM UPDATE_USER_ID VERSION_NO DELETE_FLG 
		                 INVESTIGATOR_USER_ID INCIDENT_DISPOSITION_CD CLOSE_DTTM 
		                 INCIDENT_STATUS_CD X_SASFCS_PRP_RSK_CLSSF X_SASFCS_RSK_CLSSF 
		                 X_SASFCS_PTY_KEY X_MANRISK_REASON X_SASFCS_PTY_NM )
		      ; 
		
		   attrib UDF_VALUE length= $4000;	
	 
		   retain missing0-missing0 0; 
		   
		   etls_assign_target_value = 0; 
		   /* Build hash objects from lookup tables before reading first source row  */ 
		   if (_n_ = 1) then 
		   do; 
		      /* Build hash h0 from lookup table etls_temp_lookupview0 */ 
		      nlobs = .; 
		      dsid = open("etls_temp_lookupview0"); 
		      if (dsid > 0) then 
		      do; 
		         if ( attrc(dsid, 'MTYPE') = 'DATA' ) then 
		            nlobs = attrn(dsid, 'NLOBS'); 
		         else 
		            nlobs = -1; 
		            
		         dsid = close(dsid); 
		         if (nlobs ^= 0) then 
		         do; 
		            if (nlobs > 0) then 
		               exponent = ceil(log2(nlobs)); 
		            else 
		               exponent = 8; 
		            declare hash h0(dataset: "etls_temp_lookupview0", hashexp: exponent);
		            h0.defineKey( "INCIDENT_RK");
		            h0.defineData( "UDF_VALUE");
		            h0.defineDone();
		            if (nlobs = -1) then 
		            do; 
		               if (h0.Num_Items < 1) then
		               do; 
		                  put "NOTE: Lookup table is empty: etls_temp_lookupview0";
		                  missing0 = 98; 
		               end; 
		            end; 
		         end; 
		         else 
		         do; 
		            put "NOTE: Lookup table is empty: etls_temp_lookupview0";
		            missing0 = 98; 
		         end; 
		      end; 
		      else 
		      do; 
		         put "NOTE: Lookup table does not exist or cannot be opened:"
		              " etls_temp_lookupview0";
		         
		         put "NOTE: Abort action indicated, condition= Lookup table missing:"
		              " etls_temp_lookupview0";
		         
		         abort 3; 
		      end; 
		      
		      call missing (INCIDENT_RK, UDF_VALUE); 
		   
		   end; /* All hash objects have been defined */
		   
		   /* Read a row from the source table  */ 
		   set work.rsk_manreason end = eof;
		   
		   /* Is the current key value stored in hash h0?  */ 
		   if (missing0 < 1) then 
		   do; 
		      rc0 = h0.find(key: INCIDENT_RK);
		      X_SASFCS_PTY_NM = UDF_VALUE;
		   end; 
		   else 
		      rc0 = missing0; 
		      
		   /* Examine success of lookups  */ 
		   if ( rc0=0 ) then 
		   do; 
		      /* Write row to target  */ 
		      output work.rsk_pty_nm; 
		   end; 
		   else 
		   do; 
		      error_total + 1; 
		      
		      if (rc0 ^= 0) then 
		      do; 
		         exception_total + 1; 
		         if (rc0 = 98) then 
		         do; 
		            /* Check: Lookup table has no rows-Write exception row  */ 
		            /* Check: Lookup table has no rows-Set target columns to value  */ 
		            X_SASFCS_PTY_NM = '';
		            etls_assign_target_value = 1; 
		         end; 
		         
		         else 
		         do; 
		            /* Check: Lookup value not found-Set target columns to value  */ 
		            X_SASFCS_PTY_NM = '';
		            etls_assign_target_value = 1; 
		         end; 
		      end; 
		      
		      /* Set target columns to value/missing requested?  */ 
		      if (etls_assign_target_value eq 1) then 
		         /* Write row to target  */ 
		         output work.rsk_pty_nm; 
		   
		   end; /* One or more lookups failed */
		   
		   if (eof = 1) then 
		   do; 
		      put "Source records with errors: " error_total ; 
		      put "Total lookup exceptions: " exception_total ; 
		   end; 

		run; 

	    %fcf_rcset(&syserr);

	    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
	        %let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_FB_MAN_ALERTS_RISKS;

	        %fcf_put_error;
	        %goto MACRO_END;
	    %end;

		/*==========================================================================* 
		 * Step:            Lookup                                                  *
		 * Transform:       Lookup                                                  * 
		 * Description:                                                             * 
		 *                                                                          * 
		 * Source Tables:   Lookup - work.rsk_pty_nm                                *
		 *                  Join - work.all_rsk_char_udfs                           *
		 * Target Table:    Lookup - work.rsk_pty_no                                *
		 *==========================================================================*/ 

		proc datasets lib = work nolist nowarn memtype = (data view);
		   delete etls_temp_lookupview0;
		quit;

		proc sql; create view etls_temp_lookupview0 as 
		   select * from work.all_rsk_char_udfs
		   where UDF_NM = 'X_SASFCS_PTY_NO';
		quit;
		proc datasets lib = work nolist nowarn memtype = (data view);
		   delete rsk_pty_no;
		quit;

		/*---- Begin DATA step to perform lookups  ----*/ 
		DATA  work.rsk_pty_no
		         (keep = INCIDENT_RK VALID_FROM_DTTM VALID_TO_DTTM CASE_RK INCIDENT_ID 
		                 SOURCE_SYSTEM_CD INCIDENT_TYPE_CD INCIDENT_CATEGORY_CD 
		                 INCIDENT_SUBCATEGORY_CD INCIDENT_DESC INCIDENT_FROM_DT 
		                 INCIDENT_FROM_TM INCIDENT_TO_DT INCIDENT_TO_TM DETECTION_DT 
		                 DETECTION_TM NOTIFICATION_DT NOTIFICATION_TM UI_DEF_FILE_NM 
		                 CREATE_USER_ID CREATE_DTTM UPDATE_USER_ID VERSION_NO DELETE_FLG 
		                 INVESTIGATOR_USER_ID INCIDENT_DISPOSITION_CD CLOSE_DTTM 
		                 INCIDENT_STATUS_CD X_SASFCS_PRP_RSK_CLSSF X_SASFCS_RSK_CLSSF 
		                 X_SASFCS_PTY_KEY X_MANRISK_REASON X_SASFCS_PTY_NM X_SASFCS_PTY_NO )
		      ; 
		   
		   attrib UDF_VALUE length= $4000;	
	 
		   retain missing0-missing0 0; 
		   
		   etls_assign_target_value = 0; 
		   /* Build hash objects from lookup tables before reading first source row  */ 
		   if (_n_ = 1) then 
		   do; 
		      /* Build hash h0 from lookup table etls_temp_lookupview0 */ 
		      nlobs = .; 
		      dsid = open("etls_temp_lookupview0"); 
		      if (dsid > 0) then 
		      do; 
		         if ( attrc(dsid, 'MTYPE') = 'DATA' ) then 
		            nlobs = attrn(dsid, 'NLOBS'); 
		         else 
		            nlobs = -1; 
		            
		         dsid = close(dsid); 
		         if (nlobs ^= 0) then 
		         do; 
		            if (nlobs > 0) then 
		               exponent = ceil(log2(nlobs)); 
		            else 
		               exponent = 8; 
		            declare hash h0(dataset: "etls_temp_lookupview0", hashexp: exponent);
		            h0.defineKey( "INCIDENT_RK");
		            h0.defineData( "UDF_VALUE");
		            h0.defineDone();
		            if (nlobs = -1) then 
		            do; 
		               if (h0.Num_Items < 1) then
		               do; 
		                  put "NOTE: Lookup table is empty: etls_temp_lookupview0";
		                  missing0 = 98; 
		               end; 
		            end; 
		         end; 
		         else 
		         do; 
		            put "NOTE: Lookup table is empty: etls_temp_lookupview0";
		            missing0 = 98; 
		         end; 
		      end; 
		      else 
		      do; 
		         put "NOTE: Lookup table does not exist or cannot be opened:"
		              " etls_temp_lookupview0";
		         
		         put "NOTE: Abort action indicated, condition= Lookup table missing:"
		              " etls_temp_lookupview0";
		         
		         abort 3; 
		      end; 
		      
		      call missing (INCIDENT_RK, UDF_VALUE); 
		   
		   end; /* All hash objects have been defined */
		   
		   /* Read a row from the source table  */ 
		   set work.rsk_pty_nm end = eof;
		   
		   /* Is the current key value stored in hash h0?  */ 
		   if (missing0 < 1) then 
		   do; 
		      rc0 = h0.find(key: INCIDENT_RK);
		      X_SASFCS_PTY_NO = UDF_VALUE;
		   end; 
		   else 
		      rc0 = missing0; 
		      
		   /* Examine success of lookups  */ 
		   if ( rc0=0 ) then 
		   do; 
		      /* Write row to target  */ 
		      output work.rsk_pty_no; 
		   end; 
		   else 
		   do; 
		      error_total + 1; 
		      
		      if (rc0 ^= 0) then 
		      do; 
		         exception_total + 1; 
		         if (rc0 = 98) then 
		         do; 
		            /* Check: Lookup table has no rows-Write exception row  */ 
		            /* Check: Lookup table has no rows-Set target columns to value  */ 
		            X_SASFCS_PTY_NO = .;
		            etls_assign_target_value = 1; 
		         end; 
		         
		         else 
		         do; 
		            /* Check: Lookup value not found-Set target columns to value  */ 
		            X_SASFCS_PTY_NO = .;
		            etls_assign_target_value = 1; 
		         end; 
		      end; 
		      
		      /* Set target columns to value/missing requested?  */ 
		      if (etls_assign_target_value eq 1) then 
		         /* Write row to target  */ 
		         output work.rsk_pty_no; 
		   
		   end; /* One or more lookups failed */
		   
		   if (eof = 1) then 
		   do; 
		      put "Source records with errors: " error_total ; 
		      put "Total lookup exceptions: " exception_total ; 
		   end; 

		run; 

	    %fcf_rcset(&syserr);

	    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
	        %let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_FB_MAN_ALERTS_RISKS;

	        %fcf_put_error;
	        %goto MACRO_END;
	    %end;

		/*==========================================================================* 
		 * Step:            Table Loader                                            *
		 * Transform:       Table Loader (Version 2.1)                              * 
		 * Description:                                                             * 
		 *                                                                          * 
		 * Source Table:    Lookup - work.rsk_pty_no                                *
		 * Target Table:    FSK_RISK_ASSESSMENT -                                   *
		 *                   &db_kc_libname..FSK_RISK_ASSESSMENT                    *
		 *==========================================================================*/ 

		%let SOURCEDS = %nrquote(work.rsk_pty_no); 

		data _null_;
			set rsk_pty_no;
			test_month = datepart(create_dttm);
			call symput('starting_month',put(test_month,yymmn7.));
		run;

		proc datasets lib = work nolist nowarn memtype = (data view);
		  delete temp_map;
		quit;

		data temp_map;
			if 0 then set &db_kc_libname..FSK_RISK_ASSESSMENT;
			set &SOURCEDS. (keep= INCIDENT_ID SOURCE_SYSTEM_CD INCIDENT_DISPOSITION_CD X_MANRISK_REASON X_SASFCS_RSK_CLSSF X_SASFCS_PRP_RSK_CLSSF X_SASFCS_PTY_KEY X_SASFCS_PTY_NM X_SASFCS_PTY_NO CREATE_DTTM INVESTIGATOR_USER_ID INCIDENT_TYPE_CD INCIDENT_CATEGORY_CD INCIDENT_SUBCATEGORY_CD );
			RISK_ASSESSMENT_ID = INCIDENT_ID;
			SEGMENT_ID = SOURCE_SYSTEM_CD;
			RISK_ASSESSMENT_STATUS_CODE = INCIDENT_DISPOSITION_CD;
			RISK_CLASSIFICATION = X_SASFCS_RSK_CLSSF;
			PROPOSED_RISK_CLASSIFICATION = X_SASFCS_PRP_RSK_CLSSF;
			PARTY_NUMBER = X_SASFCS_PTY_NO;
			PARTY_KEY = X_SASFCS_PTY_KEY;
			PARTY_NAME = X_SASFCS_PTY_NM;
			STARTING_MONTH_KEY = &starting_month;
			RISK_ASSESSMENT_DURATION = 0;
			CREATE_DATE = CREATE_DTTM;
			CREATE_USER_ID = INVESTIGATOR_USER_ID;
			VERSION_NUMBER = 1;
			OWNER_USER_LONG_ID = INVESTIGATOR_USER_ID;
			ASSESSMENT_TYPE_CD = INCIDENT_TYPE_CD;
			ASSESSMENT_CATEGORY_CD = INCIDENT_CATEGORY_CD;
			ASSESSMENT_SUBCATEGORY_CD = INCIDENT_SUBCATEGORY_CD;
			drop  INCIDENT_ID SOURCE_SYSTEM_CD INCIDENT_DISPOSITION_CD X_MANRISK_REASON X_SASFCS_RSK_CLSSF X_SASFCS_PRP_RSK_CLSSF X_SASFCS_PTY_KEY X_SASFCS_PTY_NM X_SASFCS_PTY_NO CREATE_DTTM INVESTIGATOR_USER_ID INCIDENT_TYPE_CD INCIDENT_CATEGORY_CD INCIDENT_SUBCATEGORY_CD;
		run;

	   %let SOURCEDS = work.temp_map;

		proc append base = &db_kc_libname..FSK_RISK_ASSESSMENT
		  data = &SOURCEDS force ; 
		run; 

	    %fcf_rcset(&syserr);

	    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
	        %let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_FB_MAN_ALERTS_RISKS;

	        %fcf_put_error;
	        %goto MACRO_END;
	    %end;


		/*==========================================================================* 
		 * Step:            Table Loader                                            *
		 * Transform:       Table Loader (Version 2.1)                              * 
		 * Description:                                                             * 
		 *                                                                          * 
		 * Source Table:    Lookup - work.rsk_pty_no                                *
		 * Target Table:    New Risk Assessments Events -                           *
		 *                   iem.new_risk_assessment_events                         *  
		 *==========================================================================*/ 

		%let SOURCEDS = %nrquote(work.rsk_pty_no); 

		proc datasets lib = iem nolist nowarn memtype = (data view);
		  delete new_risk_assessment_events;
		quit;
		
		data iem.new_risk_assessment_events;
			if 0 then set &db_kc_libname..FSK_RISK_ASSESSMENT_EVENT;
			set &SOURCEDS. (keep=SOURCE_SYSTEM_CD INCIDENT_ID X_MANRISK_REASON CREATE_DTTM INVESTIGATOR_USER_ID);
			SEGMENT_ID = SOURCE_SYSTEM_CD;
			RISK_ASSESSMENT_ID = INCIDENT_ID;
			EVENT_TYPE_CODE = "CRE";
			EVENT_DESC = X_MANRISK_REASON;
			CREATE_DATE = CREATE_DTTM;
			CREATE_USER_ID = INVESTIGATOR_USER_ID;
			drop SOURCE_SYSTEM_CD INCIDENT_ID X_MANRISK_REASON CREATE_DTTM INVESTIGATOR_USER_ID;
		run;

		%let obs_count = %fcf_get_observation_count(table=iem.new_risk_assessment_events, SASDATASET=Y);
		%let numevents = &obs_count;

		%fcf_get_sequence_ids(numIDs=&numevents);

		data iem.new_risk_assessment_events;
			set iem.new_risk_assessment_events;
			event_id = symgetn('seqId'||(strip(put(_n_,best32.))));
		run;

	    %fcf_rcset(&syserr);

	    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
	        %let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_FB_MAN_ALERTS_RISKS;

	        %fcf_put_error;
	        %goto MACRO_END;
	    %end;


		/*==========================================================================* 
		 * Step:            Table Loader                                            *
		 * Transform:       Table Loader (Version 2.1)                              * 
		 * Description:                                                             * 
		 *                                                                          * 
		 * Source Table:    New Risk Assessments Events -                           *
		 *                   iem.new_risk_assessment_events                         * 
		 * Target Table:    FSK_RISK_ASSESSMENT_EVENT -                             *
		 *                   &db_kc_libname..FSK_RISK_ASSESSMENT_EVENT              * 
		 *==========================================================================*/ 

		%let SOURCEDS = %nrquote(iem.new_risk_assessment_events); 

		proc append base = &db_kc_libname..FSK_RISK_ASSESSMENT_EVENT 
		  data = &SOURCEDS force ; 
		run; 

		%fcf_rcset(&syserr);

	    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
	        %let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_FB_MAN_ALERTS_RISKS;

	        %fcf_put_error;
	        %goto MACRO_END;
	    %end;
	%end;
   
%MACRO_END:

    %if &trans_rc eq 3 or &trans_rc ge 5 %then
        %fcf_execute_macro(fcf_error);
    %else
        %do;
            %fcf_execute_macro(fcf_success);
        %end;
   
%mend;
