/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_iem_load_alert_rels.sas
| PRODUCT: SAS Financial Crimes Foundation
| SYSTEM:  Windows, Unix
| DESC:    FCF to ECM Incident creation and synchronization
+------------------------------------------------------------------------------+
| USAGE:   %fcf_iem_load_alert_rels(ds=);
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2012-2013 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_iem_load_alert_rels(ds=);

    /*------------------------------------------
     * Set error code to 0 and message to blank
     *------------------------------------------*/

    %let trans_rc = 0;
    %let _fcfmsg_=;

	/*==========================================================================* 
	 * Step:            Get ACC Subject keys                                    *
	 * Transform:       Extract                                                 * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    PARTY_LIVE - ecm_db.PARTY_LIVE                          *
	 * Target Table:    ACC Subject keys - work.acc_rk                          *
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(ecm_db.PARTY_LIVE); 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete acc_rk;
	quit;

	proc sql;
	   create view work.acc_rk as
	      select
	         PARTY_RK,
	         PARTY_ID
	   from &SOURCEDS
	      where PARTY_TYPE_CD = "FCFSUB"  & PARTY_CATEGORY_CD = "ACC"
	   ;
	quit;

	%let SOURCEDS = work.acc_rk;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_ALERT_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Get ASC Subject keys                                    *
	 * Transform:       Extract                                                 * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    PARTY_LIVE - ecm_db.PARTY_LIVE                          *
	 * Target Table:    ASC Subject keys - work.asc_rk                          *
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(ecm_db.PARTY_LIVE); 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete asc_rk;
	quit;

	proc sql;
	   create view work.asc_rk as
	      select
	         PARTY_RK,
	         PARTY_ID
	   from &SOURCEDS
	      where PARTY_TYPE_CD = "FCFSUB"  & PARTY_CATEGORY_CD = "ASC"
	   ;
	quit;

	%let SOURCEDS = work.asc_rk;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_ALERT_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Get HHD Subject keys                                    *
	 * Transform:       Extract                                                 * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    PARTY_LIVE - ecm_db.PARTY_LIVE                          *
	 * Target Table:    HHD Subject keys - work.hhd_rk                          *
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(ecm_db.PARTY_LIVE); 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete hhd_rk;
	quit;

	proc sql;
	   create view work.hhd_rk as
	      select
	         PARTY_RK,
	         PARTY_ID
	   from &SOURCEDS
	      where PARTY_TYPE_CD = "FCFSUB"  & PARTY_CATEGORY_CD = "HHD"
	   ;
	quit;

	%let SOURCEDS = work.hhd_rk;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_ALERT_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Get PTY Subject keys                                    *
	 * Transform:       Extract                                                 * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    PARTY_LIVE - ecm_db.PARTY_LIVE                          *
	 * Target Table:    PTY Subject keys - work.pty_rk                          *
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(ecm_db.PARTY_LIVE); 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete pty_rk;
	quit;

	proc sql;
	   create view work.pty_rk as
	      select
	         PARTY_RK,
	         PARTY_ID
	   from &SOURCEDS
	      where PARTY_TYPE_CD = "FCFSUB"  & PARTY_CATEGORY_CD = "PTY"
	   ;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_ALERT_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Get EXT Subject keys                                    *
	 * Transform:       Extract                                                 * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    PARTY_LIVE - ecm_db.PARTY_LIVE                          *
	 * Target Table:    EXT Subject keys - work.ext_rk                          *
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(ecm_db.PARTY_LIVE); 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete ext_rk;
	quit;

	proc sql;
	   create view work.ext_rk as
	      select
	         PARTY_RK,
	         PARTY_ID
	   from &SOURCEDS
	      where PARTY_TYPE_CD = "FCFSUB"  & PARTY_CATEGORY_CD = "EXT"
	   ;
	quit;

	%let SOURCEDS = work.ext_rk;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_ALERT_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Get Incident keys                                       *
	 * Transform:       Extract                                                 * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    INCIDENT_LIVE - ecm_db.INCIDENT_LIVE                    *
	 * Target Table:    Incident keys - work.alt_rk                             *
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(ecm_db.INCIDENT_LIVE); 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete alt_rk;
	quit;

	proc sql;
	   create view work.alt_rk as
	      select
	         INCIDENT_RK,
	         INCIDENT_ID
	   from &SOURCEDS
	      where INCIDENT_TYPE_CD = "AMLALT"
	   ;
	quit;

	%let SOURCEDS = work.alt_rk;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_ALERT_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Get ACC Alerts                                          *
	 * Transform:       Extract                                                 * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    Input DS - &ds                                          *
	 * Target Table:    ACC Alerts - work.acc_alt                               *
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(&ds); 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete acc_alt;
	quit;

	proc sql;
	   create view work.acc_alt as
	      select
	         (CATX('-','ALT',compress(put(ALERT_ID,32.)))) as alert_id,
	         (CATX('-','ACC',PRIMARY_ENTITY_NUMBER)) as primary_entity_number
	   from &SOURCEDS
	      where PRIMARY_ENTITY_LEVEL_CODE = "ACC" /*& SEGMENT_ID = "&segKCSchema"*/
	   ;
	quit;

	%let SOURCEDS = work.acc_alt;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_ALERT_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Get ECM keys for ACC Alerts                             *
	 * Transform:       Join                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   ACC Alerts - work.acc_alt                               *
	 *                  ACC Subject keys - work.acc_rk                          *
	 *                  Incident keys - work.alt_rk                             *
	 * Target Table:    ACC Alerts with keys -                                  *
	 *                   work.acc_alt_w_rk                                      * 
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete acc_alt_w_rk;
	quit;

	proc sql;
	   create view work.acc_alt_w_rk as
	   select
	      alt_rk.INCIDENT_RK,
	      acc_rk.PARTY_RK
	   from
	      work.acc_alt, 
	      work.acc_rk, 
	      work.alt_rk
	   where
	      acc_alt.primary_entity_number = acc_rk.PARTY_ID
	      and acc_alt.alert_id = alt_rk.INCIDENT_ID
	   ;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_ALERT_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Get ASC Alerts                                          *
	 * Transform:       Extract                                                 * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    Input DS - &ds			                                *
	 * Target Table:    ASC Alerts - work.asc_alt                               *
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(&ds); 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete asc_alt;
	quit;

	proc sql;
	   create view work.asc_alt as
	      select
	         (CATX('-','ALT',compress(put(ALERT_ID,32.)))) as alert_id,
	         (CATX('-','ASC',PRIMARY_ENTITY_NUMBER)) as primary_entity_number
	   from &SOURCEDS
	      where primary_entity_level_code = "ASC" /*& SEGMENT_ID = "&segKCSchema"*/
	   ;
	quit;

	%let SOURCEDS = work.asc_alt;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_ALERT_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Get ECM keys for ASC Alerts                             *
	 * Transform:       Join                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   ASC Alerts - work.asc_alt                               *
	 *                  ASC Subject keys - work.asc_rk                          *
	 *                  Incident keys - work.alt_rk                             *
	 * Target Table:    ASC Alerts with keys -                                  *
	 *                   work.asc_alt_w_rk                                      * 
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete asc_alt_w_rk;
	quit;

	proc sql;
	   create view work.asc_alt_w_rk as
	   select
	      alt_rk.INCIDENT_RK,
	      asc_rk.PARTY_RK
	   from
	      work.asc_alt, 
	      work.asc_rk, 
	      work.alt_rk
	   where
	      asc_alt.primary_entity_number = asc_rk.PARTY_ID
	      and asc_alt.alert_id = alt_rk.INCIDENT_ID
	   ;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_ALERT_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Get HHD Alerts                                          *
	 * Transform:       Extract                                                 * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    Input DS - &ds                                          *
	 * Target Table:    HHD Alerts - work.hhd_alt                               *
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(&ds); 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete hhd_alt;
	quit;

	proc sql;
	   create view work.hhd_alt as
	      select
	         (CATX('-','ALT',compress(put(ALERT_ID,32.)))) as alert_id,
	         (CATX('-','HHD',PRIMARY_ENTITY_NUMBER)) as primary_entity_number
	   from &SOURCEDS
	      where primary_entity_level_code = "HHD" /*& SEGMENT_ID = "&segKCSchema"*/
	   ;
	quit;

	%let SOURCEDS = work.hhd_alt;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_ALERT_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Get ECM keys for HHD Alerts                             *
	 * Transform:       Join                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   HHD Alerts - work.hhd_alt                               *
	 *                  HHD Subject keys - work.hhd_rk                          *
	 *                  Incident keys - work.alt_rk                             *
	 * Target Table:    HHD Alerts with keys -                                  *
	 *                   work.hhd_alt_w_rk                                      * 
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete hhd_alt_w_rk;
	quit;

	proc sql;
	   create view work.hhd_alt_w_rk as
	   select
	      alt_rk.INCIDENT_RK,
	      hhd_rk.PARTY_RK
	   from
	      work.hhd_alt, 
	      work.hhd_rk, 
	      work.alt_rk
	   where
	      hhd_alt.primary_entity_number = hhd_rk.PARTY_ID
	      and hhd_alt.alert_id = alt_rk.INCIDENT_ID
	   ;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_ALERT_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Get PTY Alerts                                          *
	 * Transform:       Extract                                                 * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    Input DS - &ds                                          *
	 * Target Table:    PTY Alerts - work.pty_alt                               *
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(&ds); 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete pty_alt;
	quit;

	proc sql;
	   create view work.pty_alt as
	      select
	         (CATX('-','ALT',compress(put(ALERT_ID,32.)))) as alert_id,
	         (CATX('-','PTY',PRIMARY_ENTITY_NUMBER)) as primary_entity_number
	   from &SOURCEDS
	      where primary_entity_level_code = "PTY" /*& SEGMENT_ID = "&segKCSchema"*/
	   ;
	quit;

	%let SOURCEDS = work.pty_alt;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_ALERT_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Get ECM keys for PTY Alerts                             *
	 * Transform:       Join                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   PTY Alerts - work.pty_alt                               *
	 *                  PTY Subject keys - work.pty_rk                          *
	 *                  Incident keys - work.alt_rk                             *
	 * Target Table:    PTY Alerts with keys -                                  *
	 *                   work.pty_alt_w_rk                                      * 
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete pty_alt_w_rk;
	quit;

	proc sql;
	   create view work.pty_alt_w_rk as
	   select
	      alt_rk.INCIDENT_RK,
	      pty_rk.PARTY_RK
	   from
	      work.pty_alt, 
	      work.pty_rk, 
	      work.alt_rk
	   where
	      pty_alt.primary_entity_number = pty_rk.PARTY_ID
	      and pty_alt.alert_id = alt_rk.INCIDENT_ID
	   ;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_ALERT_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Get EXT Alerts                                          *
	 * Transform:       Extract                                                 * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    Input DS - &ds                                          *
	 * Target Table:    EXT Alerts - work.ext_alt                               *
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(&ds); 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete ext_alt;
	quit;

	proc sql;
	   create view work.ext_alt as
	      select
	         (CATX('-','ALT',compress(put(ALERT_ID,32.)))) as alert_id,
	         (CATX('-','EXT',PRIMARY_ENTITY_KEY)) as primary_entity_key
	   from &SOURCEDS
	      where primary_entity_level_code = "EXT" /*& SEGMENT_ID = "&segKCSchema"*/
	   ;
	quit;

	%let SOURCEDS = work.ext_alt;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_ALERT_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Get ECM keys for EXT Alerts                             *
	 * Transform:       Join                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   EXT Alerts - work.ext_alt                               *
	 *                  EXT Subject keys - work.ext_rk                          *
	 *                  Incident keys - work.alt_rk                             *
	 * Target Table:    EXT Alerts with keys -                                  *
	 *                   work.ext_alt_w_rk                                      * 
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete ext_alt_w_rk;
	quit;

	proc sql;
	   create view work.ext_alt_w_rk as
	   select
	      alt_rk.INCIDENT_RK,
	      ext_rk.PARTY_RK
	   from
	      work.ext_alt, 
	      work.ext_rk, 
	      work.alt_rk
	   where
	      ext_alt.primary_entity_key = ext_rk.PARTY_ID
	      and ext_alt.alert_id = alt_rk.INCIDENT_ID
	   ;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_ALERT_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Append                                                  *
	 * Transform:       Append                                                  * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   ACC Alerts with keys -                                  *
	 *                   work.acc_alt_w_rk                                      * 
	 *                  ASC Alerts with keys -                                  *
	 *                   work.asc_alt_w_rk                                      * 
	 *                  HHD Alerts with keys -                                  *
	 *                   work.hhd_alt_w_rk                                      * 
	 *                  PTY Alerts with keys -                                  *
	 *                   work.pty_alt_w_rk                                      * 
	 *                  EXT Alerts with keys -                                  *
	 *                   work.ext_alt_w_rk                                      * 
	 * Target Table:    All Alert Incident to Party                             *
	 *                  relationships -                                         * 
	 *                   work.all_alt_inc_x_pty                                 * 
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete all_alt_inc_x_pty;
	quit;

	data work.all_alt_inc_x_pty / view=work.all_alt_inc_x_pty;
	   set work.acc_alt_w_rk
	       work.asc_alt_w_rk
	       work.hhd_alt_w_rk
	       work.pty_alt_w_rk
	       work.ext_alt_w_rk;
	   keep PARTY_RK INCIDENT_RK; 
	run;

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_ALERT_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Table Loader                                            *
	 * Transform:       Table Loader (Version 2.1)                              * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    All Alert Incident to Party                             *
	 *                  relationships -                                         * 
	 *                   work.all_alt_inc_x_pty                                 * 
	 * Target Table:    INCIDENT_X_PARTY -                                      *
	 *                   ecm_db.INCIDENT_X_PARTY                                * 
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(work.all_alt_inc_x_pty); 

	data _null_;
    	call symput('current_dt',put(datetime(),datetime22.));
	run;

    proc datasets lib = iem nolist nowarn memtype = (data view);
        delete new_inc_x_pty;
    quit;

	data iem.new_inc_x_pty;
		if 0 then set ecm_db.incident_x_party;
		set &SOURCEDS. (keep= INCIDENT_RK PARTY_RK);
		INCIDENT_RK = INCIDENT_RK;
		PARTY_RK = PARTY_RK;
		RELATION_TYPE_CD = "ALT2SUB";
		CREATE_DTTM = "&current_dt"dt;
	run;

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_ALERT_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	%let SOURCEDS = iem.new_inc_x_pty;
	    
	proc append base = ecm_db.INCIDENT_X_PARTY 
		data = &SOURCEDS force; 
	run; 

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_ALERT_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 
	   
	/*==========================================================================* 
	 * Step:            Table Loader                                            *
	 * Transform:       Table Loader (Version 2.1)                              * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    All Alert Incident to Party                             *
	 *                  relationships -                                         * 
	 *                   work.all_alt_inc_x_pty                                 * 
	 * Target Table:    ECM_EVENT - ecm_db.ECM_EVENT                            *
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(work.all_alt_inc_x_pty); 

	data _null_;
    	call symput('current_dt',put(datetime(),datetime22.));
	run;

    proc datasets lib = iem nolist nowarn memtype = (data view);
        delete new_events;
    quit;

	data iem.new_events;
		if 0 then set ecm_db.ecm_event;
		set &SOURCEDS. (keep= INCIDENT_RK PARTY_RK);
		BUSINESS_OBJECT_NM = "INCIDENT";
		BUSINESS_OBJECT_RK = INCIDENT_RK;
		EVENT_TYPE_CD = "ADDPAR";
		EVENT_DESC = "event.etl.load.txt";
		CREATE_USER_ID = "&ecm_user";
		CREATE_DTTM = "&current_dt"dt;
		LINKED_OBJECT_NM = "PARTY";
		LINKED_OBJECT_RK = PARTY_RK;
		drop INCIDENT_RK PARTY_RK;
	run;

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_ALERT_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	%let SOURCEDS = iem.new_events;
   
	%fcf_get_ecm_sequence_ids(in=iem.new_events,out=iem.new_events,type=event,block=&iem_performance_load);

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_ALERT_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	proc append base = ecm_db.ECM_EVENT
		data = &SOURCEDS force; 
	run; 

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_LOAD_ALERT_RELS;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

%MACRO_END:

    %if &trans_rc eq 3 or &trans_rc ge 5 %then
        %fcf_execute_macro(fcf_error);
    %else
        %do;
            %fcf_execute_macro(fcf_success);
        %end;
%mend;

