/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_iem_load_ext_party.sas
| PRODUCT: SAS Financial Crimes Foundation
| SYSTEM:  Windows, Unix
| DESC:    FCF to ECM load of FCF Ext Party Accounts
+------------------------------------------------------------------------------+
| USAGE:   %fcf_iem_load_ext_party(ds=);
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2012-2013 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_iem_load_ext_party(ds=);

    /*------------------------------------------
     * Set error code to 0 and message to blank
     *------------------------------------------*/

    %let trans_rc = 0;
    %let _fcfmsg_=;

    /*==========================================================================* 
     * Step:            Extract                                                 * 
     * Transform:       Extract                                                 * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    Incoming DS - &ds                                       * 
     * Target Table:    New Ext Parties -                                       * 
     *                   work.new_ext_parties                                   * 
     *==========================================================================*/

    %let SOURCEDS = %nrquote(&ds);

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete new_ext_parties;
    quit;

    proc sql;
        create view work.new_ext_parties as
            select
                EXT_PARTY_ACCOUNT_KEY,
                SEGMENT_ID,
                FULL_NAME
            from &SOURCEDS
            /*where SEGMENT_ID = "&segKCSchema"*/
        ;
    quit;

    %let SOURCEDS = work.new_ext_parties;

    %fcf_rcset(&sqlrc);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to create NEW_EXT_PARTIES from input table for EXT load;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*==========================================================================* 
     * Step:            Table Loader                                            * 
     * Transform:       Table Loader (Version 2.1)                              * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    New Ext Parties -                                       * 
     *                   work.new_ext_parties                                   * 
     * Target Table:    New Parties - iem.new_parties                           * 
     *==========================================================================*/

    %let SOURCEDS = %nrquote(work.new_ext_parties);

	data _null_;
		call symput('current_dt',put(datetime(),datetime22.));
	run;
	
    proc datasets lib = iem nolist nowarn memtype = (data view);
        delete new_parties;
    quit;

	data iem.new_parties;
		if 0 then set ecm_db.party_live;
		set &SOURCEDS. (keep= EXT_PARTY_ACCOUNT_KEY SEGMENT_ID FULL_NAME);
		VALID_FROM_DTTM = "&current_dt"dt;
		PARTY_ID = CATX('-','EXT',EXT_PARTY_ACCOUNT_KEY);
		SOURCE_SYSTEM_CD = SEGMENT_ID;
		PARTY_TYPE_CD = 'FCFSUB';
		PARTY_CATEGORY_CD = 'EXT';
		INDIVIDUAL_FLG = "0";
		PARTY_FULL_NM = FULL_NAME;
		CREATE_USER_ID = "&ecm_user";
		CREATE_DTTM =  "&current_dt"dt;
		UPDATE_USER_ID = "&ecm_user";
		VERSION_NO = 1;
		DELETE_FLG = "0";
		X_SASFCS_PTY_KEY = EXT_PARTY_ACCOUNT_KEY;
        X_SASFCS_RSK_CLSSF = .;
        X_SASFCS_TIN = "";
		drop EXT_PARTY_ACCOUNT_KEY SEGMENT_ID FULL_NAME;
	run;

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to create IEM.NEW_PARTIES for EXT load;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete temp_map;
    quit;

	%fcf_get_ecm_sequence_ids(in=iem.new_parties,out=iem.new_parties,type=party,block=&iem_performance_load);

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to generate ECM PARTY primary keys for EXT load;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    %fcf_iem_load_subject(type='external-party',ds=iem.new_parties);

%MACRO_END:

    %if &trans_rc eq 3 or &trans_rc ge 5 %then
        %fcf_execute_macro(fcf_error);
    %else
        %do;
            %fcf_execute_macro(fcf_success);
        %end;
%mend;
