/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_iem_load_incident.sas
| PRODUCT: SAS Financial Crimes Foundation
| SYSTEM:  Windows, Unix
| DESC:    FCF to ECM Case and Incident creation and synchronization
+------------------------------------------------------------------------------+
| USAGE:   %fcf_iem_load_incident(type=, ds=);
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2012-2013 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_iem_load_incident(type=, ds=);

    /*------------------------------------------
     * Set error code to 0 and message to blank
     *------------------------------------------*/

    %let trans_rc = 0;
    %let _fcfmsg_=;

	/*==========================================================================* 
	 * Step:            Join                                                    * 
	 * Transform:       Join                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   INCIDENT_CONFIG -                                       * 
	 *                   ecm_db.INCIDENT_CONFIG                                 * 
	 *                  Input ds - &ds                       					* 
	 * Target Table:    Type, Cat, and Subcat match -                           * 
	 *                   work.type_cat_subcat_match                             * 
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
		delete type_cat_subcat_match;
	quit;

	proc sql;
	   create view work.type_cat_subcat_match as
	   select
			new_incidents.INCIDENT_RK,
			new_incidents.VALID_FROM_DTTM,
			new_incidents.VALID_TO_DTTM,
			new_incidents.CASE_RK,
			new_incidents.INCIDENT_ID,
			new_incidents.SOURCE_SYSTEM_CD,
			new_incidents.INCIDENT_TYPE_CD,
			new_incidents.INCIDENT_CATEGORY_CD,
			new_incidents.INCIDENT_SUBCATEGORY_CD,
			new_incidents.INCIDENT_DESC,
			new_incidents.INCIDENT_FROM_DT,
			new_incidents.INCIDENT_FROM_TM,
			new_incidents.INCIDENT_TO_DT,
			new_incidents.INCIDENT_TO_TM,
			new_incidents.DETECTION_DT,
			new_incidents.DETECTION_TM,
			new_incidents.NOTIFICATION_DT,
			new_incidents.NOTIFICATION_TM,
			new_incidents.UI_DEF_FILE_NM,
			new_incidents.CREATE_USER_ID,
			new_incidents.CREATE_DTTM,
			new_incidents.UPDATE_USER_ID,
			new_incidents.VERSION_NO,
			new_incidents.DELETE_FLG,
			new_incidents.INVESTIGATOR_USER_ID,
			new_incidents.INCIDENT_DISPOSITION_CD,
			new_incidents.CLOSE_DTTM,
			new_incidents.INCIDENT_STATUS_CD,
			INCIDENT_CONFIG.INCIDENT_CONFIG_SEQ_NO
	   from
			&ds, 
			ecm_db.INCIDENT_CONFIG
	   where
			new_incidents.INCIDENT_TYPE_CD = INCIDENT_CONFIG.INCIDENT_TYPE_CD
			and new_incidents.INCIDENT_CATEGORY_CD = INCIDENT_CONFIG.INCIDENT_CATEGORY_CD
			and new_incidents.INCIDENT_SUBCATEGORY_CD = INCIDENT_CONFIG.INCIDENT_SUBCATEGORY_CD
			and INCIDENT_CONFIG.INCIDENT_CATEGORY_CD NOT IS NULL 
			and INCIDENT_CONFIG.INCIDENT_SUBCATEGORY_CD NOT IS NULL 
	   ;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to match Type, Category, and Subcategory for Incident load;
		%fcf_put_error;
		%goto MACRO_END;
	%end;
	 

	/*==========================================================================* 
	 * Step:            Join                                                    * 
	 * Transform:       Join                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   INCIDENT_CONFIG -                                       * 
	 *                   ecm_db.INCIDENT_CONFIG                                 * 
	 *                  Input ds - &ds                       					* 
	 * Target Table:    Type, and Cat match -                                   * 
	 *                   work.type_cat_match                                    * 
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
		delete type_cat_match;
	quit;

	proc sql;
		create view work.type_cat_match as
		select
			new_incidents.INCIDENT_RK,
			new_incidents.VALID_FROM_DTTM,
			new_incidents.VALID_TO_DTTM,
			new_incidents.CASE_RK,
			new_incidents.INCIDENT_ID,
			new_incidents.SOURCE_SYSTEM_CD,
			new_incidents.INCIDENT_TYPE_CD,
			new_incidents.INCIDENT_CATEGORY_CD,
			new_incidents.INCIDENT_SUBCATEGORY_CD,
			new_incidents.INCIDENT_DESC,
			new_incidents.INCIDENT_FROM_DT,
			new_incidents.INCIDENT_FROM_TM,
			new_incidents.INCIDENT_TO_DT,
			new_incidents.INCIDENT_TO_TM,
			new_incidents.DETECTION_DT,
			new_incidents.DETECTION_TM,
			new_incidents.NOTIFICATION_DT,
			new_incidents.NOTIFICATION_TM,
			new_incidents.UI_DEF_FILE_NM,
			new_incidents.CREATE_USER_ID,
			new_incidents.CREATE_DTTM,
			new_incidents.UPDATE_USER_ID,
			new_incidents.VERSION_NO,
			new_incidents.DELETE_FLG,
			new_incidents.INVESTIGATOR_USER_ID,
			new_incidents.INCIDENT_DISPOSITION_CD,
			new_incidents.CLOSE_DTTM,
			new_incidents.INCIDENT_STATUS_CD,
			INCIDENT_CONFIG.INCIDENT_CONFIG_SEQ_NO
		from
			&ds, 
			ecm_db.INCIDENT_CONFIG
		where
			new_incidents.INCIDENT_TYPE_CD = INCIDENT_CONFIG.INCIDENT_TYPE_CD
			and new_incidents.INCIDENT_CATEGORY_CD = INCIDENT_CONFIG.INCIDENT_CATEGORY_CD
			and new_incidents.INCIDENT_SUBCATEGORY_CD IS NULL 
			and INCIDENT_CONFIG.INCIDENT_SUBCATEGORY_CD NOT IS NULL 
		;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to match Type, and Category for Incident load;
		%fcf_put_error;
		%goto MACRO_END;
	%end;
 
	/*==========================================================================* 
 	 * Step:            Join                                                    * 
	 * Transform:       Join                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   INCIDENT_CONFIG -                                       * 
	 *                   ecm_db.INCIDENT_CONFIG                                 * 
	 *                  Input ds - &ds                       					* 
	 * Target Table:    Type Match - work.type_match                            * 
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
		delete type_match;
	quit;

	proc sql;
		create view work.type_match as
		select
			new_incidents.INCIDENT_RK,
			new_incidents.VALID_FROM_DTTM,
			new_incidents.VALID_TO_DTTM,
			new_incidents.CASE_RK,
			new_incidents.INCIDENT_ID,
			new_incidents.SOURCE_SYSTEM_CD,
			new_incidents.INCIDENT_TYPE_CD,
			new_incidents.INCIDENT_CATEGORY_CD,
			new_incidents.INCIDENT_SUBCATEGORY_CD,
			new_incidents.INCIDENT_DESC,
			new_incidents.INCIDENT_FROM_DT,
			new_incidents.INCIDENT_FROM_TM,
			new_incidents.INCIDENT_TO_DT,
			new_incidents.INCIDENT_TO_TM,
			new_incidents.DETECTION_DT,
			new_incidents.DETECTION_TM,
			new_incidents.NOTIFICATION_DT,
			new_incidents.NOTIFICATION_TM,
			new_incidents.UI_DEF_FILE_NM,
			new_incidents.CREATE_USER_ID,
			new_incidents.CREATE_DTTM,
			new_incidents.UPDATE_USER_ID,
			new_incidents.VERSION_NO,
			new_incidents.DELETE_FLG,
			new_incidents.INVESTIGATOR_USER_ID,
			new_incidents.INCIDENT_DISPOSITION_CD,
			new_incidents.CLOSE_DTTM,
			new_incidents.INCIDENT_STATUS_CD,
			INCIDENT_CONFIG.INCIDENT_CONFIG_SEQ_NO
		from
			&ds, 
			ecm_db.INCIDENT_CONFIG
		where
			new_incidents.INCIDENT_TYPE_CD = INCIDENT_CONFIG.INCIDENT_TYPE_CD
			and INCIDENT_CONFIG.INCIDENT_CATEGORY_CD IS NULL 
			and INCIDENT_CONFIG.INCIDENT_SUBCATEGORY_CD IS NULL 
		;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to match Type for Incident load;
		%fcf_put_error;
		%goto MACRO_END;
	%end;
	 
	/*==========================================================================* 
	 * Step:            Append                                                  * 
	 * Transform:       Append                                                  * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   Type, Cat, and Subcat match -                           * 
	 *                   work.type_cat_subcat_match                             * 
	 *                  Type, and Cat match -                                   * 
	 *                   work.type_cat_match                                    * 
	 *                  Type Match - work.type_match                            * 
	 * Target Table:    Incident RKs with Group Sequence                        * 
	 *                   Numbers - work.inc_rks_w_all_seq                       * 
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
		delete inc_rks_w_all_seq;
	quit;

	data work.inc_rks_w_all_seq / view=work.inc_rks_w_all_seq;
		set work.type_cat_subcat_match
			work.type_cat_match
			work.type_match;
		keep INCIDENT_RK VALID_FROM_DTTM VALID_TO_DTTM CASE_RK INCIDENT_ID 
			SOURCE_SYSTEM_CD INCIDENT_TYPE_CD INCIDENT_CATEGORY_CD 
			INCIDENT_SUBCATEGORY_CD INCIDENT_DESC INCIDENT_FROM_DT INCIDENT_FROM_TM 
			INCIDENT_TO_DT INCIDENT_TO_TM DETECTION_DT DETECTION_TM NOTIFICATION_DT 
			NOTIFICATION_TM UI_DEF_FILE_NM CREATE_USER_ID CREATE_DTTM UPDATE_USER_ID 
			VERSION_NO DELETE_FLG INVESTIGATOR_USER_ID INCIDENT_DISPOSITION_CD CLOSE_DTTM 
			INCIDENT_STATUS_CD INCIDENT_CONFIG_SEQ_NO; 
	run;

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to append Type, Cat, and Subcat matches for Incident load;
		%fcf_put_error;
		%goto MACRO_END;
	%end;
	
 
	/*==========================================================================* 
	 * Step:            Get lowest Group Sequence numbers                       * 
	 * Transform:       Extract                                                 * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    Incident RKs with Group Sequence                        * 
	 *                   Numbers - work.inc_rks_w_all_seq                       * 
	 * Target Table:    Incident RKs with Minimum Group                         * 
	 *                  Sequence Numbers -                                      * 
	 *                   work.inc_rk_w_min_seq                                  * 
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(work.inc_rks_w_all_seq); 

	proc datasets lib = work nolist nowarn memtype = (data view);
		delete inc_rk_w_min_seq;
	quit;

	proc sql;
		create view work.inc_rk_w_min_seq as
		select
			INCIDENT_RK,
			VALID_FROM_DTTM,
			VALID_TO_DTTM,
			CASE_RK,
			INCIDENT_ID,
			SOURCE_SYSTEM_CD,
			INCIDENT_TYPE_CD,
			INCIDENT_CATEGORY_CD,
			INCIDENT_SUBCATEGORY_CD,
			INCIDENT_DESC,
			INCIDENT_FROM_DT,
			INCIDENT_FROM_TM,
			INCIDENT_TO_DT,
			INCIDENT_TO_TM,
			DETECTION_DT,
			DETECTION_TM,
			NOTIFICATION_DT,
			NOTIFICATION_TM,
			UI_DEF_FILE_NM,
			CREATE_USER_ID,
			CREATE_DTTM,
			UPDATE_USER_ID,
			VERSION_NO,
			DELETE_FLG,
			INVESTIGATOR_USER_ID,
			INCIDENT_DISPOSITION_CD,
			CLOSE_DTTM,
			INCIDENT_STATUS_CD,
			INCIDENT_CONFIG_SEQ_NO
		from &SOURCEDS
		where INCIDENT_CONFIG_SEQ_NO IN (SELECT MIN(INCIDENT_CONFIG_SEQ_NO) AS INCIDENT_CONFIG_SEQ_NO FROM inc_rks_w_all_seq GROUP BY INCIDENT_RK)
		;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to find lowest Group Sequence Number for Incident load;
		%fcf_put_error;
		%goto MACRO_END;
	%end;


	/*==========================================================================* 
	 * Step:            Join                                                    * 
	 * Transform:       Join                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   Incident RKs with Minimum Group                         * 
	 *                  Sequence Numbers -                                      * 
	 *                   work.inc_rk_w_min_seq                                  * 
	 *                  INCIDENT_CONFIG -                                       * 
	 *                   ecm_db.INCIDENT_CONFIG                                 * 
	 * Target Table:    Staged Incidents -                                      * 
	 *                   work.staged_incidents                                  * 
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
		delete staged_incidents;
	quit;

	proc sql;
		create view work.staged_incidents as
		select
			inc_rk_w_min_seq.INCIDENT_RK,
			inc_rk_w_min_seq.VALID_FROM_DTTM,
			inc_rk_w_min_seq.VALID_TO_DTTM,
			inc_rk_w_min_seq.CASE_RK,
			inc_rk_w_min_seq.INCIDENT_ID,
			inc_rk_w_min_seq.SOURCE_SYSTEM_CD,
			inc_rk_w_min_seq.INCIDENT_TYPE_CD,
			inc_rk_w_min_seq.INCIDENT_CATEGORY_CD,
			inc_rk_w_min_seq.INCIDENT_SUBCATEGORY_CD,
			inc_rk_w_min_seq.INCIDENT_DESC,
			inc_rk_w_min_seq.INCIDENT_FROM_DT,
			inc_rk_w_min_seq.INCIDENT_FROM_TM,
			inc_rk_w_min_seq.INCIDENT_TO_DT,
			inc_rk_w_min_seq.INCIDENT_TO_TM,
			inc_rk_w_min_seq.DETECTION_DT,
			inc_rk_w_min_seq.DETECTION_TM ,
			inc_rk_w_min_seq.NOTIFICATION_DT,
			inc_rk_w_min_seq.NOTIFICATION_TM,
			INCIDENT_CONFIG.UI_DEF_FILE_NM,
			inc_rk_w_min_seq.CREATE_USER_ID,
			inc_rk_w_min_seq.CREATE_DTTM,
			inc_rk_w_min_seq.UPDATE_USER_ID,
			inc_rk_w_min_seq.VERSION_NO,
			inc_rk_w_min_seq.DELETE_FLG,
			inc_rk_w_min_seq.INVESTIGATOR_USER_ID,
			inc_rk_w_min_seq.INCIDENT_DISPOSITION_CD,
			inc_rk_w_min_seq.CLOSE_DTTM,
			inc_rk_w_min_seq.INCIDENT_STATUS_CD
		from
			work.inc_rk_w_min_seq, 
			ecm_db.INCIDENT_CONFIG
		where
			inc_rk_w_min_seq.INCIDENT_CONFIG_SEQ_NO = INCIDENT_CONFIG.INCIDENT_CONFIG_SEQ_NO
		;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to retrieve UI Def file name for Incident load;
		%fcf_put_error;
		%goto MACRO_END;
	%end;

	/*==========================================================================* 
	 * Step:            Table Loader                                            * 
	 * Transform:       Table Loader (Version 2.1)                              * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    Staged Incidents -                                      * 
	 *                   work.staged_incidents                                  * 
	 * Target Table:    INCIDENT_LIVE - ecm_db.INCIDENT_LIVE                    * 
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(work.staged_incidents); 

	proc append base = ecm_db.INCIDENT_LIVE 
		data = &SOURCEDS force; 
	run; 
   
	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to append to INCIDENT_LIVE for Incident load;
		%fcf_put_error;
		%goto MACRO_END;
	%end;
    
	proc datasets lib = work nolist nowarn memtype = (data view);
		delete temp_map;
	quit;

	/*==========================================================================* 
	 * Step:            Table Loader                                            * 
	 * Transform:       Table Loader (Version 2.1)                              * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    Staged Incidents -                                      * 
	 *                   work.staged_incidents                                  * 
	 * Target Table:    INCIDENT_VERSION -                                      * 
	 *                   ecm_db.INCIDENT_VERSION                                * 
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(work.staged_incidents); 

	proc append base = ecm_db.INCIDENT_VERSION 
		data = &SOURCEDS force; 
	run; 

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to append to INCIDENT_VERSION for Incident load;
		%fcf_put_error;
		%goto MACRO_END;
	%end;
 
	proc datasets lib = work nolist nowarn memtype = (data view);
		delete temp_map;
	quit;

	/*==========================================================================* 
	 * Step:            Join                                                    * 
	 * Transform:       Join                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   Incident RKs with Minimum Group                         * 
	 *                  Sequence Numbers -                                      * 
	 *                   work.inc_rk_w_min_seq                                  * 
	 *                  INCIDENT_CONFIG_X_USER_GROUP -                          * 
	 *                   ecm_db.INCIDENT_CONFIG_X_USER_GROUP                    * 
	 * Target Table:    Incident RKs with User Groups -                         * 
	 *                   work.inc_rks_w_user_groups                             * 
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
		delete inc_rks_w_user_groups;
	quit;

	proc sql;
		create view work.inc_rks_w_user_groups as
		select
			inc_rk_w_min_seq.INCIDENT_RK,
			inc_rk_w_min_seq.INCIDENT_CONFIG_SEQ_NO,
			INCIDENT_CONFIG_X_USER_GROUP.USER_GROUP_NM
		from
			work.inc_rk_w_min_seq, 
			ecm_db.INCIDENT_CONFIG_X_USER_GROUP
		where
			inc_rk_w_min_seq.INCIDENT_CONFIG_SEQ_NO = INCIDENT_CONFIG_X_USER_GROUP.INCIDENT_CONFIG_SEQ_NO
		;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to retrieve User Groups for Incident load;
		%fcf_put_error;
		%goto MACRO_END;
	%end;
	 
	/*==========================================================================* 
	 * Step:            Table Loader                                            * 
	 * Transform:       Table Loader (Version 2.1)                              * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    Incident RKs with User Groups -                         * 
	 *                   work.inc_rks_w_user_groups                             * 
	 * Target Table:    INCIDENT_X_USER_GROUP -                                 * 
	 *                   ecm_db.INCIDENT_X_USER_GROUP                           * 
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(work.inc_rks_w_user_groups); 

	proc datasets lib = work nolist nowarn memtype = (data view);
		delete temp_map;
	quit;

	proc sql;
		create view work.temp_map as
		select
			INCIDENT_RK,
			USER_GROUP_NM
		from &SOURCEDS
		;
	quit;
   
   %let SOURCEDS = work.temp_map;

	proc append base = ecm_db.INCIDENT_X_USER_GROUP 
		data = &SOURCEDS force; 
	run; 
   
	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to append to INCIDENT_X_USER_GROUP for Incident load;
		%fcf_put_error;
		%goto MACRO_END;
	%end;
 
	proc datasets lib = work nolist nowarn memtype = (data view);
		delete temp_map;
	quit;


	/*==========================================================================* 
     * Step:            Table Loader                                            * 
     * Transform:       Table Loader (Version 2.1)                              * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    Input ds - &ds											* 
     * Target Table:    New Events - iem.new_events                             * 
     *==========================================================================*/

    %let SOURCEDS = %nrquote(&ds);

    proc datasets lib = iem nolist nowarn memtype = (data view);
        delete new_events;
    quit;
	
	data iem.new_events;
		if 0 then set ecm_db.ecm_event;
		set &SOURCEDS. (keep= INCIDENT_RK CREATE_USER_ID CREATE_DTTM);
		BUSINESS_OBJECT_NM = "INCIDENT";
		BUSINESS_OBJECT_RK = INCIDENT_RK;
		EVENT_TYPE_CD = "LOADEN";
		EVENT_DESC = "event.etl.load.txt";
		CREATE_USER_ID = CREATE_USER_ID;
		CREATE_DTTM = CREATE_DTTM;
		drop INCIDENT_RK;
	run;

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to create IEM.NEW_EVENTS;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete temp_map;
    quit;

	%fcf_get_ecm_sequence_ids(in=iem.new_events,out=iem.new_events,type=event,block=&iem_performance_load);

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to create new EVENT_RK primary keys for INCIDENT load;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*==========================================================================* 
     * Step:            Table Loader                                            * 
     * Transform:       Table Loader (Version 2.1)                              * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    New Events - iem.new_events                             * 
     * Target Table:    ECM_EVENT - ecm_db.ECM_EVENT                            * 
     *==========================================================================*/

    %let SOURCEDS = %nrquote(iem.new_events);

    proc append base = ecm_db.ECM_EVENT 
        data = &SOURCEDS force;
    run;

    %fcf_rcset(&syserr);


    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to load into ECM_EVENT;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*==========================================================================* 
     * Step:            Extract                                                 * 
     * Transform:       Extract                                                 * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    Input ds - &ds  		                                * 
     * Target Table:    Numeric UDFs - work.numeric_udfs                        * 
     *==========================================================================*/

    %let SOURCEDS = %nrquote(&ds);

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete numeric_udfs;
    quit;

    proc sql;
        create view work.numeric_udfs as
            select
                INCIDENT_RK as ECM_RK,
                VALID_FROM_DTTM,
                X_SASFCS_ML_RS,
                X_SASFCS_TF_RS,
                X_SASFCS_SCEN_ID,
                X_SASFCS_RSK_CLSSF,
                X_SASFCS_PRP_RSK_CLSSF
            from &SOURCEDS
        ;
    quit;

    %fcf_rcset(&sqlrc);


    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to get Numeric UDFs;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

     /*==========================================================================* 
     * Step:            Sort                                                    * 
     * Transform:       Sort                                                    * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    Numeric UDFs - work.numeric_udfs                        * 
     * Target Table:    Sorted Numeric UDFs -                                   * 
     *                   work.sorted_numeric_udfs                               * 
     *==========================================================================*/ 

    %let SOURCEDS = %nrquote(work.numeric_udfs); 

    proc datasets lib = work nolist nowarn memtype = (data view);
       delete sorted_numeric_udfs;
    quit;

    proc sort data = &SOURCEDS 
       out = work.sorted_numeric_udfs; 
       by
          ECM_RK
          VALID_FROM_DTTM
       ;
    run; 

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to sort Numeric UDFs;

        %fcf_put_error;
        %goto MACRO_END;
    %end;


    /*==========================================================================* 
     * Step:            Transpose                                               * 
     * Transform:       Transpose                                               * 
     *                                                                          * 
     * Source Table:    Sorted Numeric UDFs - work.sorted_numeric_udfs          * 
     * Target Table:    Transposed Numeric UDFs -                               * 
     *                   work.transposed_numeric_udfs                           * 
     *==========================================================================*/

    %let _INPUT = work.sorted_numeric_udfs;
    %let _OUTPUT = work.transposed_numeric_udfs;
	%let _OUTPUT_options = %nrquote(where = (n(ecm_rk)>0));

    proc datasets lib=work nolist nowarn memtype = (data view);
        delete transposed_numeric_udfs;
    quit;

    %let varColumns = X_SASFCS_ML_RS X_SASFCS_TF_RS X_SASFCS_SCEN_ID X_SASFCS_RSK_CLSSF 
        X_SASFCS_PRP_RSK_CLSSF;
    %let byColumns = ECM_RK VALID_FROM_DTTM;

    proc transpose data = &_INPUT out = &_OUTPUT (&_OUTPUT_options);
        by &byColumns;
        var &varColumns;
    run;

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to transpose Numeric UDFs;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*==========================================================================* 
     * Step:            Data Validation                                         * 
     * Transform:       Data Validation                                         * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    Transposed Numeric UDFs -                               * 
     *                   work.transposed_numeric_udfs                           * 
     * Target Table:    Staged Numeric UDFs -                                   * 
     *                   work.staged_numeric_udfs                               * 
     *==========================================================================*/

    %let SOURCEDS = %nrquote(work.transposed_numeric_udfs);

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete staged_numeric_udfs;
    quit;

    /*---- Begin checks for Custom, Missing Values, and Invalid Values  ----*/
    data work.staged_numeric_udfs
        (keep = ECM_RK VALID_FROM_DTTM _NAME_ COL1)
        work.temp_errors
        (keep = ECM_RK VALID_FROM_DTTM _NAME_ COL1);
        set &SOURCEDS end=eof;
        i_move = 0;

        /*---- Custom Validation: No X_SASFCS_RSK_CLSSF  ----*/
        if (_NAME_ = "X_SASFCS_RSK_CLSSF"  & COL1 = .) then
            do;
                i_move = 1;
            end;  /* if (_NAME_ = "X_SASFCS_RSK_CLSSF"  & COL1 = .) */
        else
            do;
            end;  /* else */

        /*---- Custom Validation: No X_SASFCS_PRP_RSK_CLSSF  ----*/
        if (_NAME_ = "X_SASFCS_PRP_RSK_CLSSF"  & COL1 = .) then
            do;
                i_move = 1;
            end;  /* if (_NAME_ = "X_SASFCS_PRP_RSK_CLSSF"  & COL1 = .) */
        else
            do;
            end;  /* else */

        /*---- Custom Validation: No X_SASFCS_ML_RS  ----*/
        if (_NAME_ = "X_SASFCS_ML_RS"  & COL1 = .) then
            do;
                i_move = 1;
            end;  /* if (_NAME_ = "X_SASFCS_ML_RS"  & COL1 = .) */
        else
            do;
            end;  /* else */

        /*---- Custom Validation: No X_SASFCS_TF_RS  ----*/
        if (_NAME_ = "X_SASFCS_TF_RS"  & COL1 = .) then
            do;
                i_move = 1;
            end;  /* if (_NAME_ = "X_SASFCS_TF_RS"  & COL1 = .) */
        else
            do;
            end;  /* else */

        /*---- Custom Validation: No X_SASFCS_SCEN_ID  ----*/
        if (_NAME_ = "X_SASFCS_SCEN_ID"  & COL1 = .) then
            do;
                i_move = 1;
            end;  /* if (_NAME_ = "X_SASFCS_SCEN_ID"  & COL1 = .) */
        else
            do;
            end;  /* else */

        /* move record to error file  */
        if (i_move) then
            output work.temp_errors;

        /* move record to output file  */
        else output work.staged_numeric_udfs;

        if eof then
            do;
            end;
    run;

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to validate Numeric UDFs;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*==========================================================================* 
     * Step:            Table Loader                                            * 
     * Transform:       Table Loader (Version 2.1)                              * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    Staged Numeric UDFs -                                   * 
     *                   work.staged_numeric_udfs                               * 
     * Target Table:    INCIDENT_UDF_NUM_VALUE -                                * 
     *                   ecm_db.INCIDENT_UDF_NUM_VALUE                          * 
     *==========================================================================*/

    %let SOURCEDS = %nrquote(work.staged_numeric_udfs);

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete temp_map;
    quit;

	data temp_map;
		if 0 then set ecm_db.incident_udf_num_value;
		set &SOURCEDS. (keep= ECM_RK _NAME_ COL1 VALID_FROM_DTTM);
		INCIDENT_RK = ECM_RK;
		VALID_FROM_DTTM = VALID_FROM_DTTM;
		UDF_TABLE_NM = "INCIDENT";
		UDF_NM = _NAME_;
		ROW_NO = 1;
		UDF_VALUE = COL1;
		drop ECM_RK _NAME_ COL1;
	run;

    %let SOURCEDS = work.temp_map;

    proc append base = ecm_db.INCIDENT_UDF_NUM_VALUE 
        data = &SOURCEDS force;
    run;

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to append to INCIDENT_UDF_NUM_VALUE;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete temp_map;
    quit;

    /*==========================================================================* 
     * Step:            Extract                                                 * 
     * Transform:       Extract                                                 * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    Input ds - &ds			                                * 
     * Target Table:    Character UDFs - work.character_udfs                    * 
     *==========================================================================*/

    %let SOURCEDS = %nrquote(&ds);

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete character_udfs;
    quit;

    proc sql;
        create view work.character_udfs as
            select
                INCIDENT_RK as ECM_RK,
                VALID_FROM_DTTM,
                X_SASFCS_ALT_DESC,
                X_SASFCS_SCEN_NM,
                X_SASFCS_ACT_VAL_TXT,
                X_SASFCS_PTY_NM,
                X_SASFCS_PTY_NO,
                X_SASFCS_ALT_STATUS_CD
            from &SOURCEDS
        ;
    quit;

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to create CHARACTER_UDFS;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*==========================================================================* 
     * Step:            Sort                                                    * 
     * Transform:       Sort                                                    * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    Character UDFs - work.character_udfs                    * 
     * Target Table:    Sorted Character UDFs -                                 * 
     *                   work.sorted_character_udfs                             * 
     *==========================================================================*/ 

    %let SOURCEDS = %nrquote(work.character_udfs); 

    proc datasets lib = work nolist nowarn memtype = (data view);
       delete sorted_character_udfs;
    quit;

    proc sort data = &SOURCEDS 
       out = work.sorted_character_udfs; 
       by
          ECM_RK
          VALID_FROM_DTTM
       ;
    run; 

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to sort CHARACTER_UDFS;

        %fcf_put_error;
        %goto MACRO_END;
    %end;


    /*==========================================================================* 
     * Step:            Transpose                                               * 
     * Transform:       Transpose                                               * 
     *                                                                          * 
     * Source Table:    Character UDFs - work.sorted_character_udfs             * 
     * Target Table:    Transposed Character UDFs -                             * 
     *                   work.transposed_character_udfs                         * 
     *==========================================================================*/

    %let _INPUT = work.sorted_character_udfs;
    %let _OUTPUT = work.transposed_character_udfs;
	%let _OUTPUT_options = %nrquote(where = (n(ecm_rk)>0));

    proc datasets lib=work nolist nowarn memtype = (data view);
        delete transposed_character_udfs;
    quit;

    %let varColumns = X_SASFCS_ALT_DESC X_SASFCS_SCEN_NM X_SASFCS_ACT_VAL_TXT X_SASFCS_PTY_NM 
        X_SASFCS_PTY_NO X_SASFCS_ALT_STATUS_CD;
    %let byColumns = ECM_RK VALID_FROM_DTTM;

    proc transpose data = &_INPUT out = &_OUTPUT (&_OUTPUT_options);
        by &byColumns;
        var &varColumns;
    run;

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to transpose CHAR_UDFS;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*==========================================================================* 
     * Step:            Data Validation                                         * 
     * Transform:       Data Validation                                         * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    Transposed Character UDFs -                             * 
     *                   work.transposed_character_udfs                         * 
     * Target Table:    Staged Character UDFs -                                 * 
     *                   work.staged_character_udfs                             * 
     *==========================================================================*/

    %let SOURCEDS = %nrquote(work.transposed_character_udfs);

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete staged_character_udfs;
    quit;

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete temp_errors;
    quit;

    data work.staged_character_udfs
        (keep = ECM_RK VALID_FROM_DTTM _NAME_ COL1)
        work.temp_errors
        (keep = ECM_RK VALID_FROM_DTTM _NAME_ COL1);
        set &SOURCEDS end=eof;
        i_move = 0;

        /*---- Custom Validation: No X_SASFCS_ALT_DESC  ----*/
        if ( _NAME_ = "X_SASFCS_ALT_DESC"  & COL1 = '') then
            do;
                i_move = 1;
            end;  /* if ( _NAME_ = "X_SASFCS_ALT_DESC"  & COL1 = .) */
        else
            do;
            end;  /* else */

        /*---- Custom Validation: No X_SASFCS_PTY_NO  ----*/
        if (_NAME_ = "X_SASFCS_PTY_NO"  & COL1 = '') then
            do;
                i_move = 1;
            end;  /* if (_NAME_ = "X_SASFCS_PTY_NO"  & COL1 = .) */
        else
            do;
            end;  /* else */

        /*---- Custom Validation: No X_SASFCS_PTY_NM  ----*/
        if (_NAME_ = "X_SASFCS_PTY_NM"  & COL1 = '') then
            do;
                i_move = 1;
            end;  /* if (_NAME_ = "X_SASFCS_PTY_NM"  & COL1 = .) */
        else
            do;
            end;  /* else */

        /*---- Custom Validation: No X_SASFCS_SCEN_NM  ----*/
        if (_NAME_ = "X_SASFCS_SCEN_NM"  & COL1 = '') then
            do;
                i_move = 1;
            end;  /* if (_NAME_ = "X_SASFCS_SCEN_NM"  & COL1 = .) */
        else
            do;
            end;  /* else */

        /*---- Custom Validation: No X_SASFCS_ACT_VAL_TXT  ----*/
        if (_NAME_ = "X_SASFCS_ACT_VAL_TXT"  & COL1 = '') then
            do;
                i_move = 1;
            end;  /* if (_NAME_ = "X_SASFCS_ACT_VAL_TXT"  & COL1 = .) */
        else
            do;
            end;  /* else */

        /*---- Custom Validation: No X_SASFCS_ALT_STATUS_CD  ----*/
        if (_NAME_ = "X_SASFCS_ALT_STATUS_CD"  & COL1 = '') then
            do;
                i_move = 1;
            end;  /* if (_NAME_ = "X_SASFCS_ALT_STATUS_CD"  & COL1 = .) */
        else
            do;
            end;  /* else */

        /* move record to error file  */
        if (i_move) then
            output work.temp_errors;

        /* move record to output file  */
        else output work.staged_character_udfs;

        if eof then
            do;
            end;
    run;

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to validate TRANSPOSED_CHARACTER_UDFS;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*==========================================================================* 
     * Step:            Table Loader                                            * 
     * Transform:       Table Loader (Version 2.1)                              * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    Staged Character UDFs -                                 * 
     *                   work.staged_character_udfs                             * 
     * Target Table:    INCIDENT_UDF_CHAR_VALUE -                               * 
     *                   ecm_db.INCIDENT_UDF_CHAR_VALUE                         * 
     *==========================================================================*/

    %let SOURCEDS = %nrquote(work.staged_character_udfs);

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete temp_map;
    quit;

	data temp_map;
		if 0 then set ecm_db.incident_udf_char_value;
		set &SOURCEDS. (keep= ECM_RK _NAME_ COL1 VALID_FROM_DTTM);
		INCIDENT_RK = ECM_RK;
		VALID_FROM_DTTM = VALID_FROM_DTTM;
		UDF_TABLE_NM = "INCIDENT";
		UDF_NM = _NAME_;
		ROW_NO = 1;
		UDF_VALUE = COL1;
		drop ECM_RK _NAME_ COL1;
	run;

    %let SOURCEDS = work.temp_map;

    proc append base = ecm_db.INCIDENT_UDF_CHAR_VALUE 
        data = &SOURCEDS force;
    run;

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to append to INCIDENT_UDF_CHAR_VALUE;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete temp_map;
    quit;

    /*==========================================================================* 
     * Step:            Extract                                                 * 
     * Transform:       Extract                                                 * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    Input ds - &ds			                                * 
     * Target Table:    Date UDFs - work.date_udfs                              * 
    *==========================================================================*/

    %let SOURCEDS = %nrquote(&ds);

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete date_udfs;
    quit;

    proc sql;
        create view work.date_udfs as
            select
                INCIDENT_RK as ECM_RK,
                VALID_FROM_DTTM,
                X_SASFCS_CREATE_DT,
				X_SUPPRESSION_END_DATE
            from &SOURCEDS
        ;
    quit;

    %let SOURCEDS = work.date_udfs;

    %fcf_rcset(&sqlrc);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to create DATE_UDFS;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    
    /*==========================================================================* 
     * Step:            Sort                                                    * 
     * Transform:       Sort                                                    * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    Date UDFs - work.date_udfs                              * 
     * Target Table:    Sorted Date UDFs -                                      * 
     *                   work.sorted_date_udfs                                  * 
     *==========================================================================*/ 

    %let SOURCEDS = %nrquote(work.date_udfs); 

    proc datasets lib = work nolist nowarn memtype = (data view);
       delete sorted_date_udfs;
    quit;

    proc sort data = &SOURCEDS 
       out = work.sorted_date_udfs; 
       by
          ECM_RK
          VALID_FROM_DTTM
       ;
    run; 

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to sort DATE_UDFS;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*==========================================================================* 
     * Step:            Transpose                                               * 
     * Transform:       Transpose                                               * 
     *                                                                          * 
     * Source Table:    Date UDFs - work.date_udfs                              * 
     * Target Table:    Transposed Date UDFs -                                  * 
     *                   work.transposed_date_udfs                               * 
     *==========================================================================*/

    %let _INPUT = work.date_udfs;
    %let _OUTPUT = work.transposed_date_udfs;
	%let _OUTPUT_options = %nrquote(where = (n(ecm_rk)>0));

    proc datasets lib=work nolist nowarn memtype = (data view);
        delete transposed_date_udfs;
    quit;

    %let varColumns = X_SASFCS_CREATE_DT X_SUPPRESSION_END_DATE;
    %let byColumns = ECM_RK VALID_FROM_DTTM;

    proc transpose data = &_INPUT out = &_OUTPUT (&_OUTPUT_options);
        by &byColumns;
        var &varColumns;
    run;

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to transpose DATE_UDFS;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*==========================================================================* 
     * Step:            Data Validation                                         * 
     * Transform:       Data Validation                                         * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    Transposed Date UDFs -                                  * 
     *                   work.transposed_date_udfs                               * 
     * Target Table:    Staged Date UDFs - work.staged_date_udfs                * 
     *==========================================================================*/

    %let SOURCEDS = %nrquote(work.transposed_date_udfs);

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete staged_date_udfs;
    quit;

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete temp_errors;
    quit;

    data work.staged_date_udfs
        (keep = ECM_RK VALID_FROM_DTTM _NAME_ COL1)
        work.temp_errors
        (keep = ECM_RK VALID_FROM_DTTM _NAME_ COL1);
        set &SOURCEDS end=eof;
        i_move = 0;

        /*---- Custom Validation: No X_SASFCS_CREATE_DT  ----*/
        if (_NAME_ = "X_SASFCS_CREATE_DT"  & COL1 = .) then
            do;
                i_move = 1;
            end;  /* if (_NAME_ = "X_SASFCS_CREATE_DT"  & COL1 = .) */
        else
            do;
            end;  /* else */
		/*---- Custom Validation: No X_SASFCS_CREATE_DT  ----*/
        if (_NAME_ = "X_SUPPRESSION_END_DATE"  & COL1 = .) then
            do;
                i_move = 1;
            end;  /* if (_NAME_ = "X_SUPPRESSION_END_DATE"  & COL1 = .) */
        else
            do;
            end;  /* else */

        /* move record to error file  */
        if (i_move) then
            output work.temp_errors;

        /* move record to output file  */
        else output work.staged_date_udfs;

        if eof then
            do;
            end;
    run;

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to validate TRANSPOSED_DATE_UDFS;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*==========================================================================* 
     * Step:            Table Loader                                            * 
     * Transform:       Table Loader (Version 2.1)                              * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    Staged Date UDFs - work.staged_date_udfs                * 
     * Target Table:    INCIDENT_UDF_DATE_VALUE -                               * 
     *                   ecm_db.INCIDENT_UDF_DATE_VALUE                         * 
     *==========================================================================*/

    %let SOURCEDS = %nrquote(work.staged_date_udfs);

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete temp_map;
    quit;

	data temp_map;
		if 0 then set ecm_db.incident_udf_date_value;
		set &SOURCEDS. (keep= ECM_RK _NAME_ COL1 VALID_FROM_DTTM);
		INCIDENT_RK = ECM_RK;
		VALID_FROM_DTTM = VALID_FROM_DTTM;
		UDF_TABLE_NM = "INCIDENT";
		UDF_NM = _NAME_;
		ROW_NO = 1;
		UDF_VALUE = COL1;
		drop ECM_RK _NAME_ COL1;
	run;

    %let SOURCEDS = work.temp_map;

    proc append base = ecm_db.INCIDENT_UDF_DATE_VALUE 
        data = &SOURCEDS force;
    run;

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to append to INCIDENT_UDF_CHAR_VALUE;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete temp_map;
    quit;
 
%MACRO_END:

    %if &trans_rc eq 3 or &trans_rc ge 5 %then
        %fcf_execute_macro(fcf_error);
    %else
        %do;
            %fcf_execute_macro(fcf_success);
        %end;
%mend;
