﻿/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_iem_load_party.sas
| PRODUCT: SAS Financial Crimes Foundation
| SYSTEM:  Windows, Unix
| DESC:    FCF to ECM load of FCF Parties
+------------------------------------------------------------------------------+
| USAGE:   %fcf_iem_load_party(ds=);
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2012-2013 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_iem_load_party(ds=);

    /*------------------------------------------
     * Set error code to 0 and message to blank
     *------------------------------------------*/

    %let trans_rc = 0;
    %let _fcfmsg_=;

    /*==========================================================================* 
     * Step:            Extract                                                 * 
     * Transform:       Extract                                                 * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    Incoming DS - &ds                                       * 
     * Target Table:    New Parties - work.new_parties                          * 
     *==========================================================================*/

    %let SOURCEDS = %nrquote(&ds);

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete new_parties;
    quit;

    proc sql;
        create table work.new_parties as
            select
                PARTY_KEY,
                SEGMENT_ID,
                PARTY_NUMBER,
                PARTY_TYPE_DESC,
                PARTY_TAX_ID,
                PARTY_NAME,
                RISK_CLASSIFICATION
            from &SOURCEDS
                where CHANGE_CURRENT_IND = "Y" /*& SEGMENT_ID = "&segKCSchema"*/
        ;
    quit;

    %let SOURCEDS = work.new_parties;

    %fcf_rcset(&sqlrc);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to create NEW_PARTIES from input table for PTY load;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    data work.new_parties;
        set work.new_parties;
        select (PARTY_TYPE_DESC);
            when ('INDIVIDUAL') PARTY_TYPE_DESC='1';
            otherwise PARTY_TYPE_DESC='0';
        end;
    run;

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to set INDIVIDUAL flag for PTY load;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*==========================================================================* 
     * Step:            Table Loader                                            * 
     * Transform:       Table Loader (Version 2.1)                              * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    New Parties - work.new_parties                          * 
     * Target Table:    New Parties - iem.new_parties                           * 
     *==========================================================================*/

    %let SOURCEDS = %nrquote(work.new_parties);

	data _null_;
    	call symput('current_dt',put(datetime(),datetime22.));
	run;

    proc datasets lib = iem nolist nowarn memtype = (data view);
        delete new_parties;
    quit;

	data iem.new_parties;
		if 0 then set ecm_db.party_live;
		set &SOURCEDS. (keep= PARTY_NUMBER SEGMENT_ID PARTY_TYPE_DESC PARTY_NAME PARTY_KEY RISK_CLASSIFICATION PARTY_TAX_ID);
		VALID_FROM_DTTM = "&current_dt"dt;
		PARTY_ID = CATX('-','PTY',PARTY_NUMBER);
		SOURCE_SYSTEM_CD = SEGMENT_ID;
		PARTY_TYPE_CD = 'FCFSUB';
		PARTY_CATEGORY_CD = 'PTY';
		INDIVIDUAL_FLG = PARTY_TYPE_DESC;
		PARTY_FULL_NM = PARTY_NAME;
		CREATE_USER_ID = "&ecm_user";
		CREATE_DTTM =  "&current_dt"dt;
		UPDATE_USER_ID = "&ecm_user";
		VERSION_NO = 1;
		DELETE_FLG = "0";
		X_SASFCS_PTY_KEY = PARTY_KEY;
        X_SASFCS_RSK_CLSSF = RISK_CLASSIFICATION;
        X_SASFCS_TIN = PARTY_TAX_ID;
		drop PARTY_NUMBER SEGMENT_ID PARTY_TYPE_DESC PARTY_NAME PARTY_KEY RISK_CLASSIFICATION PARTY_TAX_ID;
	run;

	%fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to create IEM.NEW_PARTIES for PTY load;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

	%fcf_get_ecm_sequence_ids(in=iem.new_parties,out=iem.new_parties,type=party,block=&iem_performance_load);

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to generate ECM PARTY primary keys for PTY load;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    %fcf_iem_load_subject(type='customer',ds=iem.new_parties);	

%MACRO_END:

    %if &trans_rc eq 3 or &trans_rc ge 5 %then
        %fcf_execute_macro(fcf_error);
    %else
        %do;
            %fcf_execute_macro(fcf_success);
        %end;
%mend;
