/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_iem_load_subject.sas
| PRODUCT: SAS Financial Crimes Foundation
| SYSTEM:  Windows, Unix
| DESC:    FCF to ECM Case and Party/Subject creation and synchronization
+------------------------------------------------------------------------------+
| USAGE:   %fcf_iem_load_subject(type=, ds=);
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2012-2013 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_iem_load_subject(type=, ds=);

    /*------------------------------------------
     * Set error code to 0 and message to blank
     *------------------------------------------*/

    %let trans_rc = 0;
    %let _fcfmsg_=;

	/*==========================================================================* 
	 * Step:            Join                                                    * 
	 * Transform:       Join                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   PARTY_CONFIG - ecm_db.PARTY_CONFIG                      * 
	 *                  Input ds - &ds                          			    * 
	 * Target Table:    Type, Cat, and Subcat match -                           * 
	 *                   work.type_cat_subcat_match                             * 
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete type_cat_subcat_match;
	quit;

	proc sql;
	   create view work.type_cat_subcat_match as
	   select
	      new_parties.PARTY_RK,
	      new_parties.VALID_FROM_DTTM,
	      new_parties.VALID_TO_DTTM,
	      new_parties.PARTY_ID,
	      new_parties.SOURCE_SYSTEM_CD,
	      new_parties.PARTY_TYPE_CD,
	      new_parties.PARTY_CATEGORY_CD,
	      new_parties.PARTY_SUBCATEGORY_CD,
	      new_parties.IDENTICAL_PARTY_LINK_SK,
	      new_parties.INDIVIDUAL_FLG,
	      new_parties.PARTY_FULL_NM,
	      new_parties.NATIONAL_ID,
	      new_parties.NATIONAL_ID_TYPE_CD,
	      new_parties.UI_DEF_FILE_NM,
	      new_parties.CREATE_USER_ID,
	      new_parties.CREATE_DTTM,
	      new_parties.UPDATE_USER_ID,
	      new_parties.VERSION_NO,
	      new_parties.DELETE_FLG,
	      PARTY_CONFIG.PARTY_CONFIG_SEQ_NO
	   from
	      &ds, 
	      ecm_db.PARTY_CONFIG
	   where
	      new_parties.PARTY_TYPE_CD = PARTY_CONFIG.PARTY_TYPE_CD
	      and new_parties.PARTY_CATEGORY_CD = PARTY_CONFIG.PARTY_CATEGORY_CD
	      and new_parties.PARTY_SUBCATEGORY_CD = PARTY_CONFIG.PARTY_SUBCATEGORY_CD
	      and PARTY_CONFIG.PARTY_CATEGORY_CD NOT IS NULL 
	      and PARTY_CONFIG.PARTY_SUBCATEGORY_CD NOT IS NULL 
	   ;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to match Type, Category, and Subcategory for Subject load;
		%fcf_put_error;
		%goto MACRO_END;
	%end;
 

	/*==========================================================================* 
	 * Step:            Join                                                    * 
	 * Transform:       Join                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   PARTY_CONFIG - ecm_db.PARTY_CONFIG                      * 
	 *                  Input ds - &ds                           				* 
	 * Target Table:    Type, and Cat match -                                   * 
	 *                   work.type_cat_match                                    * 
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
		delete type_cat_match;
	quit;

	proc sql;
	   create view work.type_cat_match as
	   select
	      new_parties.PARTY_RK,
	      new_parties.VALID_FROM_DTTM,
	      new_parties.VALID_TO_DTTM,
	      new_parties.PARTY_ID,
	      new_parties.SOURCE_SYSTEM_CD,
	      new_parties.PARTY_TYPE_CD,
	      new_parties.PARTY_CATEGORY_CD,
	      new_parties.PARTY_SUBCATEGORY_CD,
	      new_parties.IDENTICAL_PARTY_LINK_SK,
	      new_parties.INDIVIDUAL_FLG,
	      new_parties.PARTY_FULL_NM,
	      new_parties.NATIONAL_ID,
	      new_parties.NATIONAL_ID_TYPE_CD,
	      new_parties.UI_DEF_FILE_NM,
	      new_parties.CREATE_USER_ID,
	      new_parties.CREATE_DTTM,
	      new_parties.UPDATE_USER_ID,
	      new_parties.VERSION_NO,
	      new_parties.DELETE_FLG,
	      PARTY_CONFIG.PARTY_CONFIG_SEQ_NO
	   from
	      &ds, 
	      ecm_db.PARTY_CONFIG
	   where
	      new_parties.PARTY_TYPE_CD = PARTY_CONFIG.PARTY_TYPE_CD
	      and new_parties.PARTY_CATEGORY_CD = PARTY_CONFIG.PARTY_CATEGORY_CD
	      and new_parties.PARTY_SUBCATEGORY_CD IS NULL 
	      and PARTY_CONFIG.PARTY_CATEGORY_CD NOT IS NULL 
	   ;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to match Type, and Category for Subject load;
		%fcf_put_error;
		%goto MACRO_END;
	%end;


	/*==========================================================================* 
	 * Step:            Join                                                    * 
	 * Transform:       Join                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   PARTY_CONFIG - ecm_db.PARTY_CONFIG                      * 
	 *                  Input ds - &ds                           				* 
	 * Target Table:    Type Match - work.type_match                            * 
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
		delete type_match;
	quit;

	proc sql;
	   create view work.type_match as
	   select
	      new_parties.PARTY_RK,
	      new_parties.VALID_FROM_DTTM,
	      new_parties.VALID_TO_DTTM,
	      new_parties.PARTY_ID,
	      new_parties.SOURCE_SYSTEM_CD,
	      new_parties.PARTY_TYPE_CD,
	      new_parties.PARTY_CATEGORY_CD,
	      new_parties.PARTY_SUBCATEGORY_CD,
	      new_parties.IDENTICAL_PARTY_LINK_SK,
	      new_parties.INDIVIDUAL_FLG,
	      new_parties.PARTY_FULL_NM,
	      new_parties.NATIONAL_ID,
	      new_parties.NATIONAL_ID_TYPE_CD,
	      new_parties.UI_DEF_FILE_NM,
	      new_parties.CREATE_USER_ID,
	      new_parties.CREATE_DTTM,
	      new_parties.UPDATE_USER_ID,
	      new_parties.VERSION_NO,
	      new_parties.DELETE_FLG,
	      PARTY_CONFIG.PARTY_CONFIG_SEQ_NO
		from
			&ds, 
			ecm_db.PARTY_CONFIG
		where
			new_parties.PARTY_TYPE_CD = PARTY_CONFIG.PARTY_TYPE_CD
			and new_parties.PARTY_CATEGORY_CD IS NULL 
			and new_parties.PARTY_SUBCATEGORY_CD IS NULL 
	   ;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to match Type for Subject load;
		%fcf_put_error;
		%goto MACRO_END;
	%end;
 
	/*==========================================================================* 
	 * Step:            Append                                                  * 
	 * Transform:       Append                                                  * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   Type, Cat, and Subcat match -                           * 
	 *                   work.type_cat_subcat_match                             * 
	 *                  Type, and Cat match -                                   * 
	 *                   work.type_cat_match                                    * 
	 *                  Type Match - work.type_match                            * 
	 * Target Table:    Party RKs with Group Sequence Numbers                   * 
	 *                   - work.pty_rks_w_all_seq                               * 
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
		delete pty_rks_w_all_seq;
	quit;

	data work.pty_rks_w_all_seq / view=work.pty_rks_w_all_seq;
	   set work.type_cat_subcat_match
	       work.type_cat_match
	       work.type_match;
	   keep PARTY_RK VALID_FROM_DTTM VALID_TO_DTTM PARTY_ID SOURCE_SYSTEM_CD 
	    PARTY_TYPE_CD PARTY_CATEGORY_CD PARTY_SUBCATEGORY_CD IDENTICAL_PARTY_LINK_SK 
	    INDIVIDUAL_FLG PARTY_FULL_NM NATIONAL_ID NATIONAL_ID_TYPE_CD UI_DEF_FILE_NM 
	    CREATE_USER_ID CREATE_DTTM UPDATE_USER_ID VERSION_NO DELETE_FLG 
	    PARTY_CONFIG_SEQ_NO; 
	run;

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to append Type, Cat, and Subcat matches for Subject load;
		%fcf_put_error;
		%goto MACRO_END;
	%end;


	/*==========================================================================* 
	 * Step:            Get lowest Group Sequence numbers                       * 
	 * Transform:       Extract                                                 * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    Party RKs with Group Sequence Numbers                   * 
	 *                   - work.pty_rks_w_all_seq                               * 
	 * Target Table:    Party RKs with Minimum Group Sequence                   * 
	 *                   Numbers - work.pty_rk_w_min_seq                        * 
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(work.pty_rks_w_all_seq); 

	proc datasets lib = work nolist nowarn memtype = (data view);
		delete pty_rk_w_min_seq;
	quit;

	proc sql;
		create view work.pty_rk_w_min_seq as
		select
			PARTY_RK,
			VALID_FROM_DTTM,
			VALID_TO_DTTM,
			PARTY_ID,
			SOURCE_SYSTEM_CD,
			PARTY_TYPE_CD,
			PARTY_CATEGORY_CD,
			PARTY_SUBCATEGORY_CD,
			IDENTICAL_PARTY_LINK_SK,
			INDIVIDUAL_FLG,
			PARTY_FULL_NM,
			NATIONAL_ID,
			NATIONAL_ID_TYPE_CD,
			UI_DEF_FILE_NM,
			CREATE_USER_ID,
			CREATE_DTTM,
			UPDATE_USER_ID,
			VERSION_NO,
			DELETE_FLG,
			PARTY_CONFIG_SEQ_NO
		from &SOURCEDS
			where PARTY_CONFIG_SEQ_NO IN (SELECT MIN(PARTY_CONFIG_SEQ_NO) AS PARTY_CONFIG_SEQ_NO FROM pty_rks_w_all_seq GROUP BY PARTY_RK)
		;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to find lowest Group Sequence Number for Subject load;
		%fcf_put_error;
		%goto MACRO_END;
	%end;
 

	/*==========================================================================* 
	 * Step:            Join                                                    * 
	 * Transform:       Join                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   Party RKs with Minimum Group Sequence                   * 
	 *                   Numbers - work.pty_rk_w_min_seq                        * 
	 *                  PARTY_CONFIG - ecm_db.PARTY_CONFIG                      * 
	 * Target Table:    Staged Parties - work.staged_parties                    * 
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete staged_parties;
	quit;

	proc sql;
	   create view work.staged_parties as
	   select
	      pty_rk_w_min_seq.PARTY_RK,
	      pty_rk_w_min_seq.VALID_FROM_DTTM,
	      pty_rk_w_min_seq.VALID_TO_DTTM,
	      pty_rk_w_min_seq.PARTY_ID,
	      pty_rk_w_min_seq.SOURCE_SYSTEM_CD,
	      pty_rk_w_min_seq.PARTY_TYPE_CD,
	      pty_rk_w_min_seq.PARTY_CATEGORY_CD,
	      pty_rk_w_min_seq.PARTY_SUBCATEGORY_CD,
	      pty_rk_w_min_seq.IDENTICAL_PARTY_LINK_SK,
	      pty_rk_w_min_seq.INDIVIDUAL_FLG,
	      pty_rk_w_min_seq.PARTY_FULL_NM,
	      pty_rk_w_min_seq.NATIONAL_ID,
	      pty_rk_w_min_seq.NATIONAL_ID_TYPE_CD,
	      PARTY_CONFIG.UI_DEF_FILE_NM,
	      pty_rk_w_min_seq.CREATE_USER_ID,
	      pty_rk_w_min_seq.CREATE_DTTM,
	      pty_rk_w_min_seq.UPDATE_USER_ID,
	      pty_rk_w_min_seq.VERSION_NO,
	      pty_rk_w_min_seq.DELETE_FLG
	   from
	      work.pty_rk_w_min_seq, 
	      ecm_db.PARTY_CONFIG
	   where
	      pty_rk_w_min_seq.PARTY_CONFIG_SEQ_NO = PARTY_CONFIG.PARTY_CONFIG_SEQ_NO
	   ;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM:  Unable to retrieve UI Def file name for Subject load;
		%fcf_put_error;
		%goto MACRO_END;
	%end;


	/*==========================================================================* 
	 * Step:            Table Loader                                            * 
	 * Transform:       Table Loader (Version 2.1)                              * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    Staged Parties - work.staged_parties                    * 
	 * Target Table:    PARTY_LIVE - ecm_db.PARTY_LIVE                          * 
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(work.staged_parties); 
	   
	proc append base = ecm_db.PARTY_LIVE 
		data = &SOURCEDS force; 
	run; 
	   
	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to append to PARTY_LIVE for Subject load;
		%fcf_put_error;
		%goto MACRO_END;
	%end;


	/*==========================================================================* 
	 * Step:            Table Loader                                            * 
	 * Transform:       Table Loader (Version 2.1)                              * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    Staged Parties - work.staged_parties                    * 
	 * Target Table:    PARTY_VERSION - ecm_db.PARTY_VERSION                    * 
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(work.staged_parties); 
	  
	proc append base = ecm_db.PARTY_VERSION 
		data = &SOURCEDS force; 
	run; 

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to append to PARTY_LIVE for Subject load;
		%fcf_put_error;
		%goto MACRO_END;
	%end;


	/*==========================================================================* 
	 * Step:            Join                                                    * 
	 * Transform:       Join                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   Party RKs with Minimum Group Sequence                   * 
	 *                   Numbers - work.pty_rk_w_min_seq                        * 
	 *                  PARTY_CONFIG_X_USER_GROUP -                             * 
	 *                   ecm_db.PARTY_CONFIG_X_USER_GROUP                       * 
	 * Target Table:    Party RKs with User Groups -                            * 
	 *                   work.party_rks_w_user_groups                           * 
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete party_rks_w_user_groups;
	quit;

	proc sql;
	   create view work.party_rks_w_user_groups as
	   select
	      pty_rk_w_min_seq.PARTY_RK,
	      PARTY_CONFIG_X_USER_GROUP.USER_GROUP_NM
	   from
	      work.pty_rk_w_min_seq, 
	      ecm_db.PARTY_CONFIG_X_USER_GROUP
	   where
	      pty_rk_w_min_seq.PARTY_CONFIG_SEQ_NO = PARTY_CONFIG_X_USER_GROUP.PARTY_CONFIG_SEQ_NO
	   ;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to retrieve User Groups for Subject load;
		%fcf_put_error;
		%goto MACRO_END;
	%end;
 

	/*==========================================================================* 
	 * Step:            Table Loader                                            * 
	 * Transform:       Table Loader (Version 2.1)                              * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    Party RKs with User Groups -                            * 
	 *                   work.party_rks_w_user_groups                           * 
	 * Target Table:    PARTY_X_USER_GROUP -                                    * 
	 *                   ecm_db.PARTY_X_USER_GROUP                              * 
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(work.party_rks_w_user_groups); 
	   
	proc append base = ecm_db.PARTY_X_USER_GROUP 
		data = &SOURCEDS force; 
	run; 

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Unable to append to PARTY_X_USER_GROUP for Subject load;
		%fcf_put_error;
		%goto MACRO_END;
	%end;


    /*==========================================================================* 
     * Step:            Table Loader                                            * 
     * Transform:       Table Loader (Version 2.1)                              * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    Input ds - &ds			                                * 
     * Target Table:    New Events - iem.new_events                             * 
     *==========================================================================*/

    %let SOURCEDS = %nrquote(&ds);

    proc datasets lib = iem nolist nowarn memtype = (data view);
        delete new_events;
    quit;

	data iem.new_events;
		if 0 then set ecm_db.ecm_event;
		set &SOURCEDS. (keep= PARTY_RK CREATE_USER_ID CREATE_DTTM);
		BUSINESS_OBJECT_NM = "PARTY";
		BUSINESS_OBJECT_RK = PARTY_RK;
		EVENT_TYPE_CD = "LOADEN";
		EVENT_DESC = "event.etl.load.txt";
		CREATE_USER_ID = CREATE_USER_ID;
		CREATE_DTTM = CREATE_DTTM;
		drop PARTY_RK;
	run;

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to create IEM.NEW_EVENTS;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

	%fcf_get_ecm_sequence_ids(in=iem.new_events,out=iem.new_events,type=event,block=&iem_performance_load);

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to generate new EVENT_RK primary keys for SUBJECT load;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*==========================================================================* 
     * Step:            Table Loader                                            * 
     * Transform:       Table Loader (Version 2.1)                              * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    New Events - iem.new_events                             * 
     * Target Table:    ECM_EVENT - ecm_db.ECM_EVENT                            * 
     *==========================================================================*/

    %let SOURCEDS = %nrquote(iem.new_events);

    proc append base = ecm_db.ECM_EVENT 
        data = &SOURCEDS force; 
    run;

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to append to ECM_EVENT;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*==========================================================================* 
     * Step:            Extract                                                 * 
     * Transform:       Extract                                                 * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    Input ds - &ds			                                * 
     * Target Table:    Numeric UDFs - work.numeric_udfs                        * 
     *==========================================================================*/

    %let SOURCEDS = %nrquote(&ds);

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete numeric_udfs;
    quit;

    proc sql;
        create view work.numeric_udfs as
            select
                PARTY_RK as ECM_RK,
                VALID_FROM_DTTM,
                X_SASFCS_PTY_KEY,
                X_SASFCS_RSK_CLSSF
            from &SOURCEDS
        ;
    quit;

    %let SOURCEDS = work.numeric_udfs;

    %fcf_rcset(&sqlrc);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to get Numeric UDFs;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*==========================================================================* 
     * Step:            Sort                                                    * 
     * Transform:       Sort                                                    * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    Numeric UDFs - work.numeric_udfs                        * 
     * Target Table:    Sorted Numeric UDFs -                                   * 
     *                   work.sorted_numeric_udfs                               * 
     *==========================================================================*/ 

    %let SOURCEDS = %nrquote(work.numeric_udfs); 

    proc datasets lib = work nolist nowarn memtype = (data view);
       delete sorted_numeric_udfs;
    quit;

    proc sort data = &SOURCEDS 
       out = work.sorted_numeric_udfs; 
       by
          ECM_RK
          VALID_FROM_DTTM
       ;
    run; 

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to sort Numeric UDFs;

        %fcf_put_error;
        %goto MACRO_END;
    %end;


    /*==========================================================================* 
     * Step:            Transpose                                               * 
     * Transform:       Transpose                                               * 
     *                                                                          * 
     * Source Table:    Sorted Numeric UDFs - work.sorted_numeric_udfs          * 
     * Target Table:    Transposed Numeric UDFs -                               * 
     *                   work.transposed_numeric_udfs                           * 
     *==========================================================================*/

    %let _INPUT = work.sorted_numeric_udfs;
    %let _OUTPUT = work.transposed_numeric_udfs;
	%let _OUTPUT_options = %nrquote(where = (n(ecm_rk)>0));

    proc datasets lib=work nolist nowarn memtype = (data view);
        delete transposed_numeric_udfs;
    quit;

    %let varColumns = X_SASFCS_PTY_KEY X_SASFCS_RSK_CLSSF;
    %let byColumns = ECM_RK VALID_FROM_DTTM;

    proc transpose data = &_INPUT out = &_OUTPUT (&_OUTPUT_options);
        by &byColumns;
        var &varColumns;
    run;

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to transpose Numeric UDFs;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*==========================================================================* 
     * Step:            Data Validation                                         * 
     * Transform:       Data Validation                                         * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    Transposed Numeric UDFs -                               * 
     *                   work.transposed_numeric_udfs                           * 
     * Target Table:    Staged Numeric UDFs -                                   * 
     *                   work.staged_numeric_udfs                               * 
     *==========================================================================*/

    %let SOURCEDS = %nrquote(work.transposed_numeric_udfs);

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete staged_numeric_udfs;
    quit;

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete temp_errors;
    quit;

    data work.staged_numeric_udfs
        (keep = ECM_RK VALID_FROM_DTTM _NAME_ COL1)
        work.temp_errors
        (keep = ECM_RK VALID_FROM_DTTM _NAME_ COL1);
        set &SOURCEDS end=eof;
        i_move = 0;

        if (_NAME_ = "X_SASFCS_RSK_CLSSF"  & COL1 = .) then
            do;
                i_move = 1;
            end; 
        else
            do;
            end; 


        if (_NAME_ = "X_SASFCS_PTY_KEY"  & COL1 = .) then
            do;
                i_move = 1;
            end; 
        else
            do;
            end; 

        if (i_move) then
            output work.temp_errors;

        else output work.staged_numeric_udfs;

        if eof then
            do;
            end;
    run;

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to validate Numeric UDFs;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*==========================================================================* 
     * Step:            Table Loader                                            * 
     * Transform:       Table Loader (Version 2.1)                              * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    Staged Numeric UDFs -                                   * 
     *                   work.staged_numeric_udfs                               * 
     * Target Table:    PARTY_UDF_NUM_VALUE -                                   * 
     *                   ecm_db.PARTY_UDF_NUM_VALUE                             * 
     *==========================================================================*/

    %let SOURCEDS = %nrquote(work.staged_numeric_udfs);

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete temp_map;
    quit;

	data temp_map;
		if 0 then set ecm_db.party_udf_num_value;
		set &SOURCEDS. (keep= ECM_RK _NAME_ COL1 VALID_FROM_DTTM);
		PARTY_RK = ECM_RK;
		VALID_FROM_DTTM = VALID_FROM_DTTM;
		UDF_TABLE_NM = "PARTY";
		UDF_NM = _NAME_;
		ROW_NO = 1;
		UDF_VALUE = COL1;
		drop ECM_RK _NAME_ COL1;
	run;

    %let SOURCEDS = work.temp_map;

    proc append base = ecm_db.PARTY_UDF_NUM_VALUE 
        data = &SOURCEDS force;
    run;

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to append to PARTY_UDF_NUM_VALUE;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete temp_map;
    quit;

    /*==========================================================================* 
     * Step:            Extract                                                 * 
     * Transform:       Extract                                                 * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    Input ds - &ds 				                            * 
     * Target Table:    Character UDFs - work.character_udfs                    * 
     *==========================================================================*/

    %let SOURCEDS = %nrquote(&ds);
	%let SOURCEDS = iem.new_parties;

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete character_udfs;
    quit;

    proc sql;
        create view work.character_udfs as
            select
                PARTY_RK as ECM_RK,
                VALID_FROM_DTTM,
                X_SASFCS_TIN
            from &SOURCEDS
        ;
    quit;

    %fcf_rcset(&sqlrc);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to get Character UDFs;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*==========================================================================* 
     * Step:            Sort                                                    * 
     * Transform:       Sort                                                    * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    Character UDFs - work.character_udfs                    * 
     * Target Table:    Sorted Character UDFs -                                 * 
     *                   work.sorted_character_udfs                             * 
     *==========================================================================*/ 

    %let SOURCEDS = %nrquote(work.character_udfs); 

    proc datasets lib = work nolist nowarn memtype = (data view);
       delete sorted_character_udfs;
    quit;

    proc sort data = &SOURCEDS 
       out = work.sorted_character_udfs; 
       by
          ECM_RK
          VALID_FROM_DTTM
       ;
    run; 

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to sort Character UDFs;

        %fcf_put_error;
        %goto MACRO_END;
    %end;


    /*==========================================================================* 
     * Step:            Transpose                                               * 
     * Transform:       Transpose                                               * 
     *                                                                          * 
     * Source Table:    Character UDFs - work.sorted_character_udfs             * 
     * Target Table:    Transposed Character UDFs -                             * 
     *                   work.transposed_character_udfs                         * 
     *==========================================================================*/

    %let _INPUT = work.sorted_character_udfs;
    %let _OUTPUT = work.transposed_character_udfs;
	%let _OUTPUT_options = %nrquote(where = (n(ecm_rk)>0));

    proc datasets lib=work nolist nowarn memtype = (data view);
        delete transposed_character_udfs;
    quit;

    %let varColumns = X_SASFCS_TIN;
    %let byColumns = ECM_RK VALID_FROM_DTTM;

    proc transpose data = &_INPUT out = &_OUTPUT (&_OUTPUT_options);
        by &byColumns;
        var &varColumns;
    run;

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to transpose Character UDFs;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*==========================================================================* 
     * Step:            Data Validation                                         * 
     * Transform:       Data Validation                                         * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    Transposed Character UDFs -                             * 
     *                   work.transposed_character_udfs                          * 
     * Target Table:    Staged Character UDFs -                                 * 
     *                   work.staged_character_udfs                             * 
     *==========================================================================*/

    %let SOURCEDS = %nrquote(work.transposed_character_udfs);

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete staged_character_udfs;
    quit;

    data work.staged_character_udfs
        (keep = ECM_RK VALID_FROM_DTTM _NAME_ COL1)
        work.temp_errors
        (keep = ECM_RK VALID_FROM_DTTM _NAME_ COL1);
        set &SOURCEDS end=eof;
        i_move = 0;

        if ( _NAME_ = "X_SASFCS_TIN"  & COL1 = '') then
            do;
                i_move = 1;
            end; 
        else
            do;
            end; 

        if (i_move) then
            output work.temp_errors;

        else output work.staged_character_udfs;

        if eof then
            do;
            end;
    run;

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to validate Character UDFs;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*==========================================================================* 
     * Step:            Table Loader                                            * 
     * Transform:       Table Loader (Version 2.1)                              * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    Staged Character UDFs -                                 * 
     *                   work.staged_character_udfs                             * 
     * Target Table:    PARTY_UDF_CHAR_VALUE -                                  * 
     *                   ecm_db.PARTY_UDF_CHAR_VALUE                            * 
     *==========================================================================*/

    %let SOURCEDS = %nrquote(work.staged_character_udfs);

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete temp_map;
    quit;

	data temp_map;
		if 0 then set ecm_db.party_udf_char_value;
		set &SOURCEDS. (keep= ECM_RK _NAME_ COL1 VALID_FROM_DTTM);
		PARTY_RK = ECM_RK;
		VALID_FROM_DTTM = VALID_FROM_DTTM;
		UDF_TABLE_NM = "PARTY";
		UDF_NM = _NAME_;
		ROW_NO = 1;
		UDF_VALUE = COL1;
		drop ECM_RK _NAME_ COL1;
	run;

    %let SOURCEDS = work.temp_map;

    proc append base = ecm_db.PARTY_UDF_CHAR_VALUE 
        data = &SOURCEDS force;
    run;

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to append to PARTY_UDF_CHAR_VALUE;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete temp_map;
    quit;

%MACRO_END:

    %if &trans_rc eq 3 or &trans_rc ge 5 %then
        %fcf_execute_macro(fcf_error);
    %else %do;
        %fcf_execute_macro(fcf_success);
    %end;

%mend;
