/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_iem_update_account.sas
| PRODUCT: SAS Financial Crimes Foundation
| SYSTEM:  Windows, Unix
| DESC:    FCF to ECM Case and Party/Subject creation and synchronization
+------------------------------------------------------------------------------+
| USAGE:   %fcf_iem_update_account;
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2012-2013 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_iem_update_account;

    /*------------------------------------------
     * Set error code to 0 and message to blank
     *------------------------------------------*/

    %let trans_rc = 0;
    %let _fcfmsg_=;

    /*==========================================================================* 
     * Step:            Append                                                  *
     * Transform:       Append                                                  * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Tables:   ACCOUNT_TYPE1UPDATE -                                   *
     *                   iem.ACCOUNT_TYPE1UPDATE                                * 
     *                  ACCOUNT_TYPE2NEW -                                      *
     *                   iem.ACCOUNT_TYPE2NEW                                   * 
     * Target Table:    Account Updates -                                       *
     *                   work.account_updates                                   * 
     *==========================================================================*/ 

    proc datasets lib = work nolist nowarn memtype = (data view);
       delete account_updates;
    quit;

    data work.account_updates / view=work.account_updates;
       set iem.ACCOUNT_TYPE1UPDATE
           iem.ACCOUNT_TYPE2NEW;
       keep SEGMENT_ID ACCOUNT_KEY ACCOUNT_NUMBER ACCOUNT_NAME; 
    run;

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Error executing FCF_IEM_UPDATE_ACCOUNT;
        %fcf_put_error;
        %goto MACRO_END;
    %end; 

    /*==========================================================================* 
     * Step:            Extract                                                 *
     * Transform:       Extract                                                 * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    Account Updates -                                       *
     *                   work.account_updates                                   * 
     * Target Table:    New Subject Updates -                                   *
     *                   iem.new_subject_updates                                * 
     *==========================================================================*/ 

    %let SOURCEDS = %nrquote(work.account_updates); 

    proc datasets lib = iem nolist nowarn memtype = (data view);
       delete new_subject_updates;
    quit;

    data iem.new_subject_updates;
        if 0 then set ecm_db.party_live;
        set &SOURCEDS. (keep= SEGMENT_ID ACCOUNT_NUMBER ACCOUNT_NAME ACCOUNT_KEY);
        PARTY_ID = (CATX('-','ACC',ACCOUNT_NUMBER));
        SOURCE_SYSTEM_CD = SEGMENT_ID;
        PARTY_FULL_NM = ACCOUNT_NAME;
        X_SASFCS_PTY_KEY = ACCOUNT_KEY;
        X_SASFCS_RSK_CLSSF = .;
        X_SASFCS_TIN = '';
        drop SEGMENT_ID ACCOUNT_NUMBER ACCOUNT_NAME ACCOUNT_KEY;
    run;

    %let SOURCEDS = iem.new_subject_updates;

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Error executing FCF_IEM_UPDATE_ACCOUNT;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    %fcf_iem_update_subject(ds=&SOURCEDS);

%MACRO_END:

    %if &trans_rc eq 3 or &trans_rc ge 5 %then
        %fcf_execute_macro(fcf_error);
    %else %do;
        %fcf_execute_macro(fcf_success);
    %end;


%mend;
