/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_iem_update_associate.sas
| PRODUCT: SAS Financial Crimes Foundation
| SYSTEM:  Windows, Unix
| DESC:    FCF to ECM Case and Party/Subject creation and synchronization
+------------------------------------------------------------------------------+
| USAGE:   %fcf_iem_update_associate;
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2012-2013 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_iem_update_associate;

    /*------------------------------------------
     * Set error code to 0 and message to blank
     *------------------------------------------*/

    %let trans_rc = 0;
    %let _fcfmsg_=;


    /*==========================================================================* 
     * Step:            Append                                                  *
     * Transform:       Append                                                  * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Tables:   ASSOCIATE_TYPE1UPDATE -                                 *
     *                   iem.ASSOCIATE_TYPE1UPDATE                              * 
     *                  ASSOCIATE_TYPE2NEW -                                    *
     *                   iem.ASSOCIATE_TYPE2NEW                                 * 
     * Target Table:    Associate Updates -                                     *
     *                   work.associate_updates                                 * 
     *==========================================================================*/ 

    proc datasets lib = work nolist nowarn memtype = (data view);
       delete associate_updates;
    quit;

    data work.associate_updates / view=work.associate_updates;
       set iem.ASSOCIATE_TYPE1UPDATE
           iem.ASSOCIATE_TYPE2NEW;
       keep SEGMENT_ID ASSOCIATE_KEY ASSOCIATE_NUMBER ASSOCIATE_FIRST_NAME ASSOCIATE_LAST_NAME 
        ASSOCIATE_MIDDLE_NAME; 
    run;

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Error executing FCF_IEM_UPDATE_ASSOCIATE;
        %fcf_put_error;
        %goto MACRO_END;
    %end; 

    /*==========================================================================* 
     * Step:            Extract                                                 *
     * Transform:       Extract                                                 * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    Associate Updates -                                     *
     *                   work.associate_updates                                 * 
     * Target Table:    New Subject Updates -                                   *
     *                   iem.new_subject_updates                                * 
     *==========================================================================*/ 

    %let SOURCEDS = %nrquote(work.associate_updates); 
 
    proc datasets lib = iem nolist nowarn memtype = (data view);
       delete new_subject_updates;
    quit;

    data iem.new_subject_updates;
        if 0 then set ecm_db.party_live;
        set &SOURCEDS. (keep= SEGMENT_ID ASSOCIATE_NUMBER ASSOCIATE_FIRST_NAME ASSOCIATE_MIDDLE_NAME ASSOCIATE_LAST_NAME ASSOCIATE_KEY);
        PARTY_ID = (CATX('-','ASC',ASSOCIATE_NUMBER));
        SOURCE_SYSTEM_CD = SEGMENT_ID;
        PARTY_FULL_NM = (CATX(' ',ASSOCIATE_FIRST_NAME,ASSOCIATE_MIDDLE_NAME,ASSOCIATE_LAST_NAME));
        X_SASFCS_PTY_KEY = ASSOCIATE_KEY;
        X_SASFCS_RSK_CLSSF = .;
        X_SASFCS_TIN = '';
        drop SEGMENT_ID ASSOCIATE_NUMBER ASSOCIATE_FIRST_NAME ASSOCIATE_MIDDLE_NAME ASSOCIATE_LAST_NAME ASSOCIATE_KEY;
    run;

    %let SOURCEDS = iem.new_subject_updates;

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Error executing FCF_IEM_UPDATE_ASSOCIATE;
        %fcf_put_error;
        %goto MACRO_END;
    %end; 

    %fcf_iem_update_subject(ds=&SOURCEDS);

%MACRO_END:

    %if &trans_rc eq 3 or &trans_rc ge 5 %then
        %fcf_execute_macro(fcf_error);
    %else %do;
        %fcf_execute_macro(fcf_success);
    %end;


%mend;
