/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_iem_update_household.sas
| PRODUCT: SAS Financial Crimes Foundation
| SYSTEM:  Windows, Unix
| DESC:    FCF to ECM Case and Party/Subject creation and synchronization
+------------------------------------------------------------------------------+
| USAGE:   %fcf_iem_update_account;
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2012-2013 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_iem_update_household;

    /*==========================================================================* 
     * Step:            Append                                                  *
     * Transform:       Append                                                  * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Tables:   HOUSEHOLD_TYPE1UPDATE -                                 *
     *                   iem.HOUSEHOLD_TYPE1UPDATE                              * 
     *                  HOUSEHOLD_TYPE2NEW -                                    *
     *                   iem.HOUSEHOLD_TYPE2NEW                                 * 
     * Target Table:    Household Updates -                                     *
     *                   work.household_updates                                 * 
     *==========================================================================*/ 

    proc datasets lib = work nolist nowarn memtype = (data view);
       delete household_updates;
    quit;

    data work.household_updates / view=work.household_updates;
       set iem.HOUSEHOLD_TYPE1UPDATE
           iem.HOUSEHOLD_TYPE2NEW;
       keep SEGMENT_ID HOUSEHOLD_KEY HOUSEHOLD_NUMBER HEAD_OF_HOUSEHOLD_NAME; 
    run;

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Error executing FCF_IEM_UPDATE_HOUSEHOLD;
        %fcf_put_error;
        %goto MACRO_END;
    %end; 

    /*==========================================================================* 
     * Step:            Extract                                                 *
     * Transform:       Extract                                                 * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    Household Updates -                                     *
     *                   work.household_updates                                 * 
     * Target Table:    New Subject Updates -                                   *
     *                   iem.new_subject_updates                                * 
     *==========================================================================*/ 

    %let SOURCEDS = %nrquote(work.household_updates); 
 
    proc datasets lib = iem nolist nowarn memtype = (data view);
       delete new_subject_updates;
    quit;

    data iem.new_subject_updates;
        if 0 then set ecm_db.party_live;
        set &SOURCEDS. (keep= SEGMENT_ID HOUSEHOLD_NUMBER HEAD_OF_HOUSEHOLD_NAME HOUSEHOLD_KEY);
        PARTY_ID = (CATX('-','HHD',HOUSEHOLD_NUMBER));
        SOURCE_SYSTEM_CD = SEGMENT_ID;
        PARTY_FULL_NM = (CATX(' ',HEAD_OF_HOUSEHOLD_NAME));
        X_SASFCS_PTY_KEY = HOUSEHOLD_KEY;
        X_SASFCS_RSK_CLSSF = .;
        X_SASFCS_TIN = '';
        drop SEGMENT_ID HOUSEHOLD_NUMBER HEAD_OF_HOUSEHOLD_NAME HOUSEHOLD_KEY;
    run;

    %let SOURCEDS = iem.new_subject_updates;

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Error executing FCF_IEM_UPDATE_HOUSEHOLD;
        %fcf_put_error;
        %goto MACRO_END;
    %end; 

    %fcf_iem_update_subject(ds=&SOURCEDS);

%MACRO_END:

    %if &trans_rc eq 3 or &trans_rc ge 5 %then
        %fcf_execute_macro(fcf_error);
    %else %do;
        %fcf_execute_macro(fcf_success);
    %end;

%mend;
