/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_iem_update_party.sas
| PRODUCT: SAS Financial Crimes Foundation
| SYSTEM:  Windows, Unix
| DESC:    FCF to ECM Case and Party/Subject creation and synchronization
+------------------------------------------------------------------------------+
| USAGE:   %fcf_iem_update_party;
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2012-2013 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_iem_update_party;

    /*------------------------------------------
     * Set error code to 0 and message to blank
     *------------------------------------------*/

    %let trans_rc = 0;
    %let _fcfmsg_=;

    /*==========================================================================* 
     * Step:            Append                                                  *
     * Transform:       Append                                                  * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Tables:   PARTY_TYPE1UPDATE -                                     *
     *                   iem.PARTY_TYPE1UPDATE                                  * 
     *                  PARTY_TYPE2NEW - iem.PARTY_TYPE2NEW                     *
     * Target Table:    Party Updates - work.party_updates                      *
     *==========================================================================*/ 

    proc datasets lib = work nolist nowarn memtype = (data view);
       delete party_updates;
    quit;

    data work.party_updates / view=work.party_updates;
       set iem.PARTY_TYPE1UPDATE
           iem.PARTY_TYPE2NEW;
       keep SEGMENT_ID PARTY_key PARTY_NUMBER PARTY_TAX_ID PARTY_NAME RISK_CLASSIFICATION; 
    run;

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Error executing FCF_IEM_UPDATE_PARTY;
        %fcf_put_error;
        %goto MACRO_END;
    %end; 


    /*==========================================================================* 
     * Step:            Extract                                                 *
     * Transform:       Extract                                                 * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    Party Updates - work.party_updates                      *
     * Target Table:    New Subject Updates -                                   *
     *                   iem.new_subject_updates                                * 
     *==========================================================================*/ 

    %let SOURCEDS = %nrquote(work.party_updates); 

    proc datasets lib = iem nolist nowarn memtype = (data view);
       delete new_subject_updates;
    quit;

    data iem.new_subject_updates;
        if 0 then set ecm_db.party_live;
        set &SOURCEDS. (keep= SEGMENT_ID PARTY_NUMBER PARTY_NAME PARTY_KEY RISK_CLASSIFICATION PARTY_TAX_ID);
        PARTY_ID = (CATX('-','PTY',PARTY_NUMBER));
        PARTY_FULL_NM = PARTY_NAME;
        SOURCE_SYSTEM_CD = SEGMENT_ID;
        X_SASFCS_PTY_KEY = PARTY_KEY;
        X_SASFCS_RSK_CLSSF = RISK_CLASSIFICATION;
        X_SASFCS_TIN = PARTY_TAX_ID;
        drop SEGMENT_ID PARTY_NUMBER PARTY_NAME PARTY_KEY RISK_CLASSIFICATION PARTY_TAX_ID;
    run;

    %let SOURCEDS = iem.new_subject_updates;

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Error executing FCF_IEM_UPDATE_PARTY;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    %fcf_iem_update_subject(ds=&SOURCEDS);

%MACRO_END:

    %if &trans_rc eq 3 or &trans_rc ge 5 %then
        %fcf_execute_macro(fcf_error);
    %else %do;
        %fcf_execute_macro(fcf_success);
    %end;


%mend;
