/*==============================================================================
| NAME:    fcf_sa_ecm_ui_data.sas
| PRODUCT: SAS Compliance Solutions
| SYSTEM:  UNIX/WINDOWS
================================================================================

Converts SAS data set to rows so that they can be serialized as XML format expected
by ECM to populate UI context or datagrid storage.
Parameters:
        ds - name of the table to convert
        prefix - prefix to use for ECM field names (for example, TEMP.X_CUSTOMER_)
        type - row - only one single row is converted as single ECM field,
               row_no will be set as missing value
               table - the entire table is converted as ECM table
==============================================================================*/

/*-------------------------------------------------------------------
 * Copyright (c) 2012-2014 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

data row_values;
  length field_nm $ 128
         row_no   8
         string_value $ &fcf_max_udf_len
         numeric_value 8
         datetime_value $19
      ;
  stop;
run;

%macro fcf_sa_ecm_ui_data(ds=, prefix=, type=);
    proc contents data=&ds out=ds_meta noprint;
    run;
    data &ds._num;
        set &ds.;
        length row_no 8;
        retain row_no;
        if _n_ = 1 then do;
           if "&type" = "table" then do;
             row_no = 1;
           end;
           else do;
             row_no =.;
           end;
        end;
        else do;
             row_no = row_no + 1;
        end;
        output;
    run;
    data _null_;
        set ds_meta;
        if format ne "DATETIME" then do;
            call execute ( '%nrstr(%fcf_sa_get_value(table=' || "&ds._num" || ', col_name=' || name
                            || ', type=' || type || ", prefix=&prefix" || '))');
        end;
        else do;
            call execute ( '%nrstr(%fcf_sa_get_value(table='||"&ds._num"||', col_name='||name||', type=3'
                            || ", prefix=&prefix" ||'))');
        end;
    run;
%mend fcf_sa_ecm_ui_data;

%macro fcf_sa_get_value (table=, col_name=, type=, prefix=);
  %if &type eq 1 %then %do; /*number*/
      proc sql;
       create table row_val as
       select "&prefix.&col_name" as field_nm,
               row_no,
               "" as string_value,
               &col_name as numeric_value label="numeric_value" ,
               "" as datetime_value
       from &table;
      quit;
  %end;
  %if &type eq 2 %then %do; /*char*/
      proc sql;
       create table row_val as
               select "&prefix.&col_name" as field_nm,
               row_no,
               &col_name as string_value label="string_value" ,
               . as numeric_value,
               "" as datetime_value
       from &table;
      quit;
  %end;
  %if &type eq 3 %then %do; /*datetime*/
      proc sql;
       create table row_val as
               select "&prefix.&col_name" as field_nm,
               row_no,
               "" as string_value,
               . as numeric_value,
               CASE WHEN &col_name eq . THEN "" ELSE PUT(&col_name, is8601dt.) END as datetime_value label="datetime_value"
               /* PUT(&col_name, is8601dt.) as datetime_value label="datetime_value" */
       from &table;
      quit;
  %end;

  proc append base=row_values force
              data=row_val;
  run;
%mend fcf_sa_get_value;

