/*
+-------------+----------------------------------------------------------------+
| PRODUCT:    | SAS Financial Crimes Foundation
| SUBSYSTEM:  | Windows/Unix
| DESCRIPTION | Macro to download a file from the SAS Content server
|             |
| SOURCE:     | fcf_sa_get_file_from_dav_server
+------------------------------------------------------------------------------+
| USAGE:      | this macro downloads a file
|             | if use_if_exist = Y then macro firt is goinf to check if the file
|             | exist on the Content Server, if not then no download attempt is
|             | made and error is not reported.
|             |
+-------------+----------------------------------------------------------------+
| COMMENTS:   | If error data set exist then errors are appended
|             |
+-------------+----------------------------------------------------------------+
*/

/*-------------------------------------------------------------------
 * Copyright (c) 2012-2013 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_sa_get_file_from_dav_server(davFileName=, localFileName=, use_if_exist=N);
    %let fcf_sa_dav_err_flg = 0;
    filename scrsrv WEBDAV "&davFileName" AUTHDOMAIN="@authdomain@";
    %if &syserr ne 0  %then %do;
      %let err_msg = ERROR: &syserrortext. Filename=&localFileName WEBDAV filename=&davFileName;
      %put &err_msg;
      %let fcf_sa_dav_err_flg = 1;
    %end;
    %else %do;
        %let file_exist=1;
        %if &use_if_exist = Y %then %do;
            data _null_;
               ex_flg=fexist("scrsrv");
               call symput('file_exist', ex_flg);
            run;
        %end;
        %if &file_exist = 1 %then %do;
            data _null_;
               infile scrsrv;
               file "&localFileName";
               input;
               put _infile_;
            run;
            %if &syserr ne 0  %then %do;
              %let err_msg = ERROR: &syserrortext. WEBDAV filename=&davFileName Filename=&localFileName;
              %put &err_msg;
              %let fcf_sa_dav_err_flg = 1;
            %end;
        %end;
        %else %do;
            %put NOTE: file &davFileName does not exist, skip its download.;
        %end;
    %end;
    filename scrsrv CLEAR;
    %if &fcf_sa_dav_err_flg eq 1 and %sysfunc(libref(errors)) %then %do;
      data errors1;
            length msg $400;
            msg = "&err_msg";
            output;
      run;
      proc append base=errors data=errors1 FORCE NOWARN;
      run;
    %end;
%mend fcf_sa_get_file_from_dav_server;
