/*==============================================================================
| NAME:    fcf_stp_ecm_ui_response.sas
| PRODUCT: SAS Compliance Solutions
| SYSTEM:  UNIX/WINDOWS
================================================================================

Converts SAS data set to XML format expected by ECM to populate UI context or
datagrid storage.
Parameters:
    ds - data set with data rows, that will be sent as STP response,
         must contains the following variables:
             field_nm $ 128
             row_no   8
             string_value $ 4000
             numeric_value 8
             datetime_value $19
==============================================================================*/

/*-------------------------------------------------------------------
 * Copyright (c) 2014-2015 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/
%macro fcf_stp_ecm_ui_response(ds=row_values);
    %let row_count = 0;

    proc sql;
      select count(*) into :row_count from &ds;
    quit;

    %if &row_count ne 0 %then %do;
        data _null_;
           if _N_ = 1 then rc1 = stpsrv_header('Content-type',"text/xml");
           dummy=1;
        run;
        %fcf_write_ecm_ui_xml_data(ds=&ds, fr=_webout);
    %end;
    %else %do;
        /* if table is empty then write out an empty xml element to avoid exception on the Java side */
        data _null_;
           file _webout encoding=utf8;
           dummy_table='<?xml version="1.0" encoding="utf-8" ?><TABLE></TABLE>';
           put dummy_table;
        run;
    %end;
%mend fcf_stp_ecm_ui_response;
