/*==============================================================================
| NAME:    fcf_write_ecm_ui_xml_data.sas
| PRODUCT: SAS Compliance Solutions
| SYSTEM:  UNIX/WINDOWS
================================================================================

Converts SAS data set to XML format expected by ECM to populate UI context or
datagrid storage.
Parameters:
    ds - data set with data rows, must contains the following variables:
         field_nm $ 128
         row_no   8
         string_value $ 4000
         numeric_value 8
         datetime_value $19

    fr - fileref for the output XML file
         encoding for the output file is utf-8

==============================================================================*/

/*-------------------------------------------------------------------
 * Copyright (c) 2014-2014 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_write_ecm_ui_xml_data(ds=, fr=);

	data _null_;
		set &ds end=lastRow;
		file &fr encoding=utf8 lrecl=32000;

		length line1 $&fcf_max_udf_len;
		/* output buffer must be longer than value to account for xml tags and quoted characters */
		length line $%sysevalf(&fcf_max_udf_len+60);
		length line0  $120;

		if _n_ = 1 then do;
			line = '<?xml version="1.0" encoding="utf-8" ?>';
			put line;
			line = '<TABLE>';
			put line;
		end;

		line0 = '<UI_DATA><field_nm>' || upcase(trim(field_nm)) || '</field_nm><row_no>' || compress(put(row_no, BEST8.)) || '</row_no>';
		if numeric_value ne . then do;
           line = '<numeric_value>' || compress(put(numeric_value, BEST32.7));
		   put line0 line +(-1) '</numeric_value></UI_DATA>';
		end;
		if datetime_value ne " " then do;
           line = '<datetime_value>' || datetime_value;
		   put line0 line +(-1) '</datetime_value></UI_DATA>';
		end;
	 	if string_value ne " " then do;
		   line1 = tranwrd(string_value, '&', '&amp;');
		   line1 = tranwrd(line1, '<', '&lt;');
		   line1 = tranwrd(line1, '>', '&gt;');
		   line1 = tranwrd(line1, '"', '&quot;');
		   line1 = tranwrd(line1, "'", '&apos;');
           line = '<string_value>' || line1;
		   put line0 line +(-1) '</string_value></UI_DATA>';
		end;

		if lastRow then do;
			put '</TABLE>';
		end;
	run;
%mend fcf_write_ecm_ui_xml_data;

