  /*
+-------------+----------------------------------------------------------------+
| PRODUCT:    | SAS Financial Crimes Foundation
| DESCRIPTION | Macro to read Rule Manageristrator config values into
|             | global variables
|             |
| SOURCE:     | cdd_sm_read_config_data.sas
+------------------------------------------------------------------------------+
| USAGE:      | app_name - specifies the name of the application in the metadata
|             | repository.
+-------------+----------------------------------------------------------------+
| COMMENTS    |
+------------------------------------------------------------------------------+
*/

/*-------------------------------------------------------------------
 * Copyright (c) 2012-2013 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro cdd_sm_read_config_data(app_name=);
    /* global variables for Rule Manager */
    %global cdd_sm_genmax cdd_sm_backup_folder cdd_sm_rule_path cdd_sm_hdr_path log_path cddwork cdd_sm_rc cdd_sm_codegen_path cdd_sm_prep_lib cdd_sm_rule_group_path
            fcf_dav_root
            cdd_sync_job_ds cdd_sync_folder cdd_critical_update_in_process cdd_sync_lock_failed
            fcf_subject_type cdd_sm_party_type cdd_sm_rule_cat cdd_sm_rule_group_cat cdd_alert_type cdd_inv_type;
    %global cdd_config_locale;

  %let fcf_subject_type=FCFSUB;
  %let cdd_sm_party_type=CDDSM;
  %let cdd_sm_rule_cat=CDDRULE;
  %let cdd_sm_rule_group_cat=CDDRG;
  %let cdd_alert_type=CDDALT;
  %let cdd_inv_type=CDDINV;

    %let cdd_sync_folder = !CMNROOT/fcfsync; /* location of the folder to store mutex dataset */
  libname cdd_sync  "&cdd_sync_folder";
  %let cdd_sync_job_ds = cdd_sync.cdd_sync_mutex;   /* data set to lock while job is running */

    /* use work libname for all temp datasets - can be changed for debugging */
    %let cddwork = work;
    /* number of copies to keep for CDD backup datasets */
    %let cdd_sm_genmax=17;
    %let cdd_sm_genmax_prop=cdd.backpup.genmax;
    /* location of the backups for fsk datasets */
    %let cdd_sm_backup_folder =!CMNROOT/backups/;
    %let cdd_sm_backup_folder_prop =cdd.backup.path;
    /* path to rule source code */
    %let cdd_sm_rule_path=!CDDROOT/rule/rule_code_active/;
    %let cdd_sm_rule_path_prop=cdd.rule.source.path;
    /* path to rule group source code */
    %let cdd_sm_rule_group_path=!CDDROOT/rule/rule_group_code_active/;
    %let cdd_sm_rule_group_path_prop=cdd.rule.group.source.path;
    /* path to codegen */
    %let cdd_sm_codegen_path=!CDDROOT/rule/codegen/;
    %let cdd_sm_codegen_path_prop=cdd.codegen.location;
    /* path to logs folder */
    %let log_path=!CDDROOT/logs/;
    %let log_path_prop=cdd.logs.path;
    /* name of the prep data sets library*/
    %let cdd_sm_prep_lib=CM_PREP;
    %let cdd_sm_prep_lib_prop=cdd.prep.lib;

    /* read parameters */
    %let cdd_sm_rc = 0;
    %fcf_get_app_property_value(application=&app_name,
                                  property=&cdd_sm_genmax_prop,
                                  output_property_value=cdd_sm_genmax,
                                  output_status=cdd_sm_rc
                                 );
    %if &cdd_sm_rc ne 0 %then %do;
        %let cdd_sm_genmax        = 17;
        %put WARNING: Property &cdd_sm_genmax_prop not found. Using default value: &cdd_sm_genmax.;
    %end;

    %fcf_get_app_property_value(application=&app_name,
                                  property=&cdd_sm_backup_folder_prop,
                                  output_property_value=cdd_sm_backup_folder,
                                  output_status=cdd_sm_rc
                                 );
    %if &cdd_sm_rc ne 0 %then %do;
        %let cdd_sm_backup_folder =!CMNROOT/backups/;
        %put WARNING: Property &cdd_sm_backup_folder_prop not found. Using default value: &cdd_sm_backup_folder.;
    %end;

    %fcf_get_app_property_value(application=&app_name,
                                  property=&cdd_sm_rule_path_prop,
                                  output_property_value=cdd_sm_rule_path,
                                  output_status=cdd_sm_rc
                                 );
    %if &cdd_sm_rc ne 0 %then %do;
        %let cdd_sm_rule_path=!CDDROOT/rule/rule_code_active/;
        %put WARNING: Property &cdd_sm_rule_path_prop not found. Using default value: &cdd_sm_rule_path.;
    %end;

    %fcf_get_app_property_value(application=&app_name,
                                  property=&cdd_sm_rule_group_path_prop,
                                  output_property_value=cdd_sm_rule_group_path,
                                  output_status=cdd_sm_rc
                                 );
    %if &cdd_sm_rc ne 0 %then %do;
        %let cdd_sm_rule_group_path=!CDDROOT/rule/rule_group_code_active/;
        %put WARNING: Property &cdd_sm_rule_group_path_prop not found. Using default value: &cdd_sm_rule_group_path.;
    %end;

    %fcf_get_app_property_value(application=&app_name,
                                  property=&cdd_sm_codegen_path_prop,
                                  output_property_value=cdd_sm_codegen_path,
                                  output_status=cdd_sm_rc
                                 );
    %if &cdd_sm_rc ne 0 %then %do;
        %let cdd_sm_codegen_path=!CDDROOT/rule/codegen/;
        %put WARNING: Property &cdd_sm_codegen_path_prop not found. Using default value: &cdd_sm_codegen_path.;
    %end;

    %fcf_get_app_property_value(application=&app_name,
                                  property=&log_path_prop,
                                  output_property_value=log_path,
                                  output_status=cdd_sm_rc
                                 );
    %if &cdd_sm_rc ne 0 %then %do;
        %let log_path=!CDDROOT/logs/;
        %put WARNING: Property &log_path_prop not found. Using default value: &log_path.;
    %end;

    %fcf_get_app_property_value(application=&app_name,
                                  property=&cdd_sm_prep_lib_prop,
                                  output_property_value=cdd_sm_prep_lib,
                                  output_status=cdd_sm_rc
                                 );
    %if &cdd_sm_rc ne 0 %then %do;
        %let cdd_sm_prep_lib=CM_PREP;
        %put WARNING: Property &cdd_sm_prep_lib_prop not found. Using default value: &cdd_sm_prep_lib.;
    %end;

    %macro cdd_sm_get_dav_url();
    /* assuming that DAVService URL has form:
       http://server99.foo.com:8080/SASWIPSoapServices/services/DAVService
       or
       https://server99.foo.com:8080/SASWIPSoapServices/services/DAVService
     */
         %svcreg(type=SOAP, name=DAVService);
         %let w1=%qkscan(&service_endpoint, 1, /);
         %if %upcase(&w1) eq HTTP: or %upcase(&w1) eq HTTPS: %then %do;
              %let w2=%qkscan(&service_endpoint, 2, /);
              %let fcf_dav_root=&w1//&w2/SASContentServer/repository/default/sasdav/;
          %end;
          %else %do;
              %put ERROR: WEBDAV root not found.;
          %end;
    %mend cdd_sm_get_dav_url;

    %macro cdd_set_locale(sys_locale=);
        /* sys locale is set to language part of the system locale
         *  for zh_* locales it is set as:
         *   zh_CN -> zh, zh_TW -> zt, zh_HK -> zt, zh_* -> en
         */
        %let cdd_config_locale=&sys_locale;
        %if &sys_locale = zh %then %do;
            %let cdd_config_locale=zh_CN;
        %end;
        %if &sys_locale = zt %then %do;
            %let cdd_config_locale=en;
        %end;
    %mend cdd_set_locale;

    %cdd_set_locale(sys_locale=&cdd_system_locale);
    %cdd_sm_get_dav_url();

    %put NOTE: Codegen location            : &cdd_sm_codegen_path;
    %put NOTE: Rule source code path       : &cdd_sm_rule_path;
    %put NOTE: Logs path                   : &log_path;
    %put NOTE: Backups path                : &cdd_sm_backup_folder;
    %put NOTE: Number of backup generations: &cdd_sm_genmax;
    %put NOTE: Prep data sets library name : &cdd_sm_prep_lib;
    %put NOTE: WEBDAV root                 : &fcf_dav_root;
%mend cdd_sm_read_config_data;



